$MULTIPLEFITS=0

{
  eval { require Time::HiRes };
  if( $@ )
    {
      print "warning: $@";
    }
  else
    {
      import Time::HiRes qw(time);
    }
}

sub cpu_time()
  {
    # Add user/system parent/child cpu_times.
    my $result = 0;
    map { $result += $_ } times;
    $result;
  }

sub print_timing($$$)
  {
    my ($message,$wall,$cpu) = @_;
    printf "%s: %.3fs wall, %.3fs cpu.\n", $message, $wall, $cpu;
  }

sub print_memory_use()
  {
    # Different operating systems have different data and ps uses different
    # arguments so try a series of commands in an attempt to collect the
    # information.
    system qw(ps -o vsz -p), $$;
    system qw(ps -l -p), $$;
    system qw(ps v), $$;
  }

FEM def para;r;femur;example
FEM def coor;r;femur;example
FEM def base;r;femur;example

#
# Reads in an initial mesh and calculates linear approximations
#  for the derivatives
#
FEM def node;r;femur;example
FEM def elem;r;femur;example
FEM up node deriv 1 linear node 2..401
FEM up node deriv 2 linear node 2..401
FEM update scale_factor normalise

#
# The initial mesh
#
FEM export node;femur_init as femur_init offset 10000
FEM export elem;femur_init as femur_init offset_elem 10000


#
# Calculate the intial xi projections
#
FEM def data;r;femur;example
FEM def xi;c close
#FEM def xi;w;femur 
FEM li data err 
FEM export data;femur_data as femur_data error


#
# Setup the fitting problem
#
FEM up field from geometry
FEM def fit;r;femur;example geometry

($wall_setup,$cpu_setup) = (time, cpu_time);
print_timing 'setup', $wall_setup - $^T, $cpu_setup;

#
# First fit
#
FEM fit
FEM update node fit
FEM update scale_factor normalise 
FEM def xi;c close old
FEM li data err 
FEM export data;femur_data_1 as femur_data_1 error
FEM export node;fitted_femur_1 as fitted_femur_1 offset 20000
FEM export elem;fitted_femur_1 as fitted_femur_1 offset_elem 20000

if( $TESTING ) {
  fem list data;fitted1 errors;
  fem list node;fitted1
}

if ($MULTIPLEFITS==1)
{

#
# Second fit
#
FEM def fit;r;femur_2;example geometry


FEM fit
FEM update node fit
FEM update scale_factor normalise
FEM def xi;c close old
FEM li data err 
FEM export data;femur_data_2 as femur_data_2 error
FEM export node;fitted_femur_2 as fitted_femur_2 offset 30000
FEM export elem;fitted_femur_2 as fitted_femur_2 offset_elem 30000

#
# Third fit
#
FEM fit
FEM update node fit
FEM update scale_factor normalise
FEM def xi;c close old
FEM li data err 
FEM export data;femur_data_3 as femur_data_3 error
FEM export node;fitted_femur_3 as fitted_femur_3 offset 40000
FEM export elem;fitted_femur_3 as fitted_femur_3 offset_elem 40000

#
# Fourth fit
#
FEM fit
FEM update node fit
FEM update scale_factor normalise
FEM def xi;c close old
FEM li data err 
FEM export data;femur_data_4 as femur_data_4 error
FEM export node;fitted_femur_4 as fitted_femur_4 offset 50000
FEM export elem;fitted_femur_4 as fitted_femur_4 offset_elem 50000
#
# Fifth fit
#
FEM fit
FEM update node fit
FEM update scale_factor normalise
FEM def xi;c close old
FEM li data err 
FEM export data;femur_data_5 as femur_data_5 error
FEM export node;fitted_femur_5 as fitted_femur_5 offset 60000
FEM export elem;fitted_femur_5 as fitted_femur_5 offset_elem 60000
#
# Sixth fit
#
FEM fit
FEM update node fit
FEM update scale_factor normalise
FEM def xi;c close old
FEM li data err 
FEM export data;femur_data_6 as femur_data_6 error
FEM export node;fitted_femur_6 as fitted_femur_6 offset 70000
FEM export elem;fitted_femur_6 as fitted_femur_6 offset_elem 70000

FEM def no;w;fitted_femur_6
FEM def elem;w;fitted_femur_6

}

($wall_solve,$cpu_solve) = (time, cpu_time);
print_timing 'solve', $wall_solve - $wall_setup, $cpu_solve - $cpu_setup;
print_memory_use;
