
# This command file is to be called from cmgui after
# the command file main.com has been run to setup
# some of the variables

# If this is not the first time through this comfile 
# then we have to remove the node group and textures
# from the 3d window from previous slices
if($slice_count != 0) {
    gfx modify g_element $slice_numbers[$slice_count-1] node_points glyph cross size 1 material gold label cmiss_number select_on invisible
    gfx modify window 1 background texture none
}

# Read in the image we want to create a texture
gfx create texture $slice_num image "../../images/$slice_num.rgb" height $height width $width

# Calculate where abouts this image is in terms of z
$z = $zorigin - $zdist*($slice_num - 1);

# Calculate where the viewer is in terms of z
$zeyepoint = $z + $cp_dist;

# Calculate the viewing angle in degrees. This is the angle 
# between the point from which you are looking from and the 
# diagonal span of the image
$diag_dist = sqrt($height*$height + $width*$width);

# Convert radians to degrees (!). Should use Math::Trig but
# available on all systems (physiome, grr)
$pi = atan2(1,1)*4.0;
$pi_180 = 180.0/$pi;
$view_angle = 2.0*(atan2($diag_dist/2,$cp_dist)*$pi_180);

# This is all swiped from example as

# Set the projection in the graphics window. This has several parts:
# 1. The interest_point is at the centre of the texture ie (0,0,$z)
# 2. The eye_point is where you are looking from ie (0,0,$zeyepoint)
# 3. The "up_vector" (ie the direction towards the top of the 3d window) 
#    is in the y direction ie (0,1,0) 
# 4. The near and far clipping planes are $cp_near and $cp_far units away from the eye
#    point and are hence symmetrical about the texture plane. This is 
#    important as currently, any new nodes created are placed halfway 
#    between them.
# 5. A parallel projection is used
# 6. The diagonal view_angle is used for a parallel projection as it is for
#    perspective mode - it is the angle from the eye_point to the diagonal span
#    of the texture at the interest_point.
# 7. The users viewport_coordinates are defined such that the top-left of the
#    texture (when upside-down) is the top-left of the window, and the scale
#    factors of $xscale,$yscale pixels/unit, make its physical size of 76.8 
#    expand to its size in texels of 512 in each direction, with 1 texel per pixel
# 8. The window is put in absolute_viewport mode, in which the projection is
#    drawn on the ndc_placement area in user viewport_coordinates.
# 9. The ndc_placement projects the viewing volume described by the above onto 
#    the user viewport_coordinates where the texture will be placed.

gfx modify window 1 view eye_point $xorigin $yorigin $zeyepoint interest_point $xorigin $yorigin $z up_vector 0 1 0 near_clipping_plane $cp_near far_clipping_plane $cp_far parallel view_angle $view_angle viewport_coordinates $xbottom $ytop $xscale $yscale  absolute_viewport ndc_placement $xbottom $ytop $width $height

# Draw the slice texture on the user viewport_coordinates of the window. 
gfx modify window 1 background texture $slice_num tex_placement $xbottom $ybottom $width -$height

# Set up the node editing tools
gfx create ngroup $slice_num
gfx modify g_element $slice_num node_points glyph cross size 1 material gold label cmiss_number select_on
gfx node_tool group $slice_num coordinate_field coordinates edit create

gfx modify window 1 set node_tool

# Increment, ready for the next image
$slice_count++
$slice_num = $slice_numbers[$slice_count]
