# File creates the hostmesh ipnode file based on the subject's
# ipnode file.
# read in ipnode file
# finds average wall thickness and base-apex length
# moves hostmesh nodes out by wall-thickness*const1 for epi nodes
# moves hostmesh nodes in by wall-thickness*const2 for endo nodes
# moves base nodes up by a certain precentage of base-apex length
# the middle hostmesh nodes are placed between ring 2 and 3 of org.mesh
# apex - versions?
# derivatives?


#use coordConversion;
use PDL;
#use Math::Trig;
#use File::Copy "cp";
#use PDL::Math "floor";

#set from parent function:
my $offset = 400;

my $epiNodes = pdl [[1..4],
                    [5..8],
                    [9..12],
                    [13..16]];

my $endoNodes = pdl [[21..24],
                     [25..28],
                     [29..32],
                     [33..36]];

my $file = "mesh_1.ipnode";

my $nodes = readNodes($file);
if(0) #Debugging
{
  for(my $n=1;$n<=40;$n++)
  {
    print "\nNode: $n\n";
    for(my $j=0;$j<3;$j++)
    {
      print "j: $j\n";
      for(my $nk=1;$nk<5;$nk++)
      {
         print "nk $nk: ", at($nodes,$j,$nk,$n),"\n";
      }
    }
  }
}
my $wt = findWallThickness($nodes);
my $height = findHeight($nodes);
my $newNodes = pdl $nodes;
$newNodes = changeBaseNodes($newNodes,$wt,$height);
$newNodes = changeMiddleNodes($newNodes,$wt,$height);
$newNodes = changeApexNodes($newNodes,$wt,$height);
writeHostNodes("host.ipnode", $newNodes);

system("cmo moveHost.com");

#####################################################################
#######   SUBRUTINES    #############################################
#####################################################################


sub changeBaseNodes
{
  my ($array,$wt,$height) = @_;

  my $dx = -0.018*abs($height);
  my $c1 = 2.0;
  my $c2 = 0.95;
  my $dSurf = 20;
  for(my $c=1;$c<=4;$c++)
  {
    #epi nodes
    my $x1 = at($array,0,1,$c);
    my $y1 = at($array,1,1,$c);
    my $z1 = at($array,2,1,$c);
    #endo nodes
    my $x2 = at($array,0,1,$c+$dSurf);
    my $y2 = at($array,1,1,$c+$dSurf);
    my $z2 = at($array,2,1,$c+$dSurf);
    
    my $v1 = norm(pdl [$x2-$x1, $y2-$y1, $z2-$z1]);
    #epi nodes
    my $x3 = $x1 - at($v1,0,0)*$wt*$c1 + $dx;     
    my $y3 = $y1 - at($v1,1,0)*$wt*$c1;     
    my $z3 = $z1 - at($v1,2,0)*$wt*$c1;     
    #endo nodes
    my $x4 = $x2 + at($v1,0,0)*$wt*$c2 + $dx*12;     
    my $y4 = $y2 + at($v1,1,0)*$wt*$c2;     
    my $z4 = $z2 + at($v1,2,0)*$wt*$c2;     

    print "c=$c, x1=$x1, x2=$x2, x3=$x3, x4=$x4\n";
    
    PDL::Core::set $array,0,1,$c,$x3;
    PDL::Core::set $array,1,1,$c,$y3;
    PDL::Core::set $array,2,1,$c,$z3;
    PDL::Core::set $array,0,1,$c+$dSurf,$x4;
    PDL::Core::set $array,1,1,$c+$dSurf,$y4;
    PDL::Core::set $array,2,1,$c+$dSurf,$z4;
  }
  return($array);
}


sub changeMiddleNodes
{
  my ($array,$wt,$height) = @_;

  my $dx = 0;
  my $c1 = 1.2;
  my $c2 = 0.4;
  my $dSurf = 20;
  for(my $r=1;$r<4;$r++)
  {
    my $offset = 4*$r;
    for(my $c=1;$c<=4;$c++)
    {
      #epi nodes
      my $x1 = at($array,0,1,$c+$offset);
      my $y1 = at($array,1,1,$c+$offset);
      my $z1 = at($array,2,1,$c+$offset);
      #endo nodes
      my $x2 = at($array,0,1,$c+$offset+$dSurf);
      my $y2 = at($array,1,1,$c+$offset+$dSurf);
      my $z2 = at($array,2,1,$c+$offset+$dSurf);
    
      my $v1 = norm(pdl [$x2-$x1, $y2-$y1, $z2-$z1]);
      my $x3 = $x1 - at($v1,0,0)*$wt*$c1 + $dx;     
      my $y3 = $y1 - at($v1,1,0)*$wt*$c1;     
      my $z3 = $z1 - at($v1,2,0)*$wt*$c1;     
      my $x4 = $x2 + at($v1,0,0)*$wt*$c2 + $dx;     
      my $y4 = $y2 + at($v1,1,0)*$wt*$c2;     
      my $z4 = $z2 + at($v1,2,0)*$wt*$c2;     
    
      PDL::Core::set $array,0,1,$c+$offset,$x3;
      PDL::Core::set $array,1,1,$c+$offset,$y3;
      PDL::Core::set $array,2,1,$c+$offset,$z3;
      PDL::Core::set $array,0,1,$c+$offset+$dSurf,$x4;
      PDL::Core::set $array,1,1,$c+$offset+$dSurf,$y4;
      PDL::Core::set $array,2,1,$c+$offset+$dSurf,$z4;
    }
  }
  return($array);
}


sub changeApexNodes
{
  my ($array,$wt,$height) = @_;

  my $c1 = 1.3;
  my $c2 = 0.9;
  my $dSurf = 20;
  for(my $c=17;$c<=20;$c++)
  {
    #epi nodes
    my $x1 = at($array,0,1,$c);
    my $y1 = at($array,1,1,$c);
    my $z1 = at($array,2,1,$c);
    #endo nodes
    my $x2 = at($array,0,1,$c+$dSurf);
    my $y2 = at($array,1,1,$c+$dSurf);
    my $z2 = at($array,2,1,$c+$dSurf);
    
    my $v1 = norm(pdl [$x2-$x1, $y2-$y1, $z2-$z1]);
    #epi nodes
    my $x3 = $x1 - at($v1,0,0)*$wt*$c1;     
    my $y3 = $y1 - at($v1,1,0)*$wt*$c1;     
    my $z3 = $z1 - at($v1,2,0)*$wt*$c1;     
    #endo nodes
    my $x4 = $x2 + at($v1,0,0)*$wt*$c2;     
    my $y4 = $y2 + at($v1,1,0)*$wt*$c2;     
    my $z4 = $z2 + at($v1,2,0)*$wt*$c2;     
    
    PDL::Core::set $array,0,1,$c,$x3;
    PDL::Core::set $array,1,1,$c,$y3;
    PDL::Core::set $array,2,1,$c,$z3;
    PDL::Core::set $array,0,1,$c+$dSurf,$x4;
    PDL::Core::set $array,1,1,$c+$dSurf,$y4;
    PDL::Core::set $array,2,1,$c+$dSurf,$z4;
  }
  return($array);
}



sub readNodes
{
  # Reads in nodes, but no multiples for nodes with more than 1 version
  my ($file) = @_;
  
  print "Reading nodes from $file\n";
  my $array = zeroes(3,5,1000);
  my $index=0;
  my $dir = 0;
  my $node = 0;
  CORE::open(INFID, $file) || die "Error: Couldn't open $file\n";
  while (my $line = <INFID>)
  {
    if($line =~ m/ Node number.+:(.+)/)
    {
      $node = $1;
      $index = $1;
      PDL::Core::set $array,0,0,$index,$1;
      PDL::Core::set $array,1,0,$index,$1;
      PDL::Core::set $array,2,0,$index,$1;
    }
    #Handling apical nodes with versions
    if($line =~ m/ For version number 1:/)
    {
      $index = $node + 0;
    } elsif($line =~ m/ For version number 2:/)
    {
      $index = $node + 1;
    } elsif($line =~ m/ For version number 3:/)
    {
      $index = $node + 2;
    } elsif($line =~ m/ For version number 4:/)
    {
      $index = $node + 3;
    }
    if($line =~ m/ The Xj\((.+)\) coordinate is \[.+\]:(.+)/)
    {
      $dir = $1-1;
      PDL::Core::set $array,$dir,1,$index,$2;
    }
    if($line =~ m/ The derivative wrt direction 1 is \[.+\]:(.+)/)
    {
      PDL::Core::set $array,$dir,2,$index,$1;
    }
    if($line =~ m/ The derivative wrt direction 2 is \[.+\]:(.+)/)
    {
      PDL::Core::set $array,$dir,3,$index,$1;
    }
    if($line =~ m/ The derivative wrt directions 1 & 2 is \[.+\]:(.+)/)
    {
      PDL::Core::set $array,$dir,4,$index,$1;
    }
  }
  CORE::close(INFID);
  return($array->slice(":,:,0:40"));
}


sub writeHostNodes
{
  my ($file, $array) = @_;

  print "Making node file $file\n";
  
  my $offset = 400;
  my $nNodes = $array->getdim(2)-17;
  CORE::open(OUTFID, "> $file") || die "Error: Couldn't open $file\n";

  print OUTFID " CMISS Version 1.21 ipnode File Version 2\n";
  print OUTFID " Heading: Node file updated from old version\n";
  print OUTFID " \n";
  print OUTFID sprintf(" The number of nodes is [   %2d]:    %2d\n", $nNodes-6, $nNodes-6);
  print OUTFID " Number of coordinates [3]: 3\n";
  print OUTFID " Do you want prompting for different versions of nj=1 [N]? y\n";
  print OUTFID " Do you want prompting for different versions of nj=2 [N]? y\n";
  print OUTFID " Do you want prompting for different versions of nj=3 [N]? y\n";
  print OUTFID " The number of derivatives for coordinate 1 is [0]: 3\n";
  print OUTFID " The number of derivatives for coordinate 2 is [0]: 3\n";
  print OUTFID " The number of derivatives for coordinate 3 is [0]: 3\n";
  print OUTFID " \n";
 
  my($a,$b,$c,$d);
  my $nVersions = 1;
  my $n=1;
  my $node=1;
  while($n<=$nNodes)
  {
    $node = $n;
    print OUTFID sprintf(" Node number [    %d]:     %2d\n", $node+$offset, $node+$offset);
    if( (($n>=9) && ($n<=12)) || (($n>=21) && ($n<=24)) )
    {
      $nVersions = 4;
    } else
    {
      $nVersions = 1;
    }
    for(my $k=0;$k<3;$k++)
    {
      print OUTFID sprintf(" The number of versions for nj=%d is [1]:  %d\n", $k+1, $nVersions);
      if($nVersions>1)
      {
        for(my $v=1;$v<=$nVersions;$v++)
        {
          print OUTFID sprintf(" For version number %d:\n", $v);
          ($a,$b,$c,$d) = getCoords($array, $n+$v-1, $k);
          print OUTFID sprintf(" The Xj(%d) coordinate is [ 0.00000E+00]:    %1.4E\n", $k+1, $a);
          print OUTFID sprintf(" The derivative wrt direction 1 is [ 0.00000E+00]:    %1.4E\n", $b);
          print OUTFID sprintf(" The derivative wrt direction 2 is [ 0.00000E+00]:    %1.4E\n", $c*0.8);
          print OUTFID sprintf(" The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   %1.4E\n", $d*0.1);
        }
      } else
      {
        ($a,$b,$c,$d) = getCoords($array, $n, $k);
        print OUTFID sprintf(" The Xj(%d) coordinate is [ 0.00000E+00]:    %1.4E\n", $k+1, $a);
        print OUTFID sprintf(" The derivative wrt direction 1 is [ 0.00000E+00]:    %1.4E\n", $b);
        print OUTFID sprintf(" The derivative wrt direction 2 is [ 0.00000E+00]:    %1.4E\n", $c*0.6);
        print OUTFID sprintf(" The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   %1.4E\n", $d*0.1);
      }
    }
    if($nVersions>1)
    {
      $n=$n+$nVersions;
    } else
    {
      $n++;
    }
    print OUTFID " \n";
  }
  CORE::close(OUTFID);
}



sub findWallThickness
{
  my ($array) = @_;

  my $wt = 0;
  my $nNodes = 17;
  my $dSurf = 20;

  #print "getdim = ", $array->getdim(2),"\n";

  for(my $c=1;$c<=$nNodes;$c++)
  {
    #epi nodes
    my $x1 = at($array,0,1,$c);
    my $y1 = at($array,1,1,$c);
    my $z1 = at($array,2,1,$c);
    #endo nodes
    my $x2 = at($array,0,1,$c+$dSurf);
    my $y2 = at($array,1,1,$c+$dSurf);
    my $z2 = at($array,2,1,$c+$dSurf);
    $wt = $wt + sqrt(($x2-$x1)**2 + ($y2-$y1)**2 + ($z2-$z1)**2);
  }
  $wt = $wt/$nNodes;
  print "Wallthickness = $wt\n";
  return($wt);
}


sub findHeight
{
  my ($array) = @_;

  my $height = 0;
  for(my $c=1;$c<=4;$c++)
  {
    #base node
    my $x1 = at($array,0,1,$c);
    my $y1 = at($array,1,1,$c);
    my $z1 = at($array,2,1,$c);
    #apex node
    my $x2 = at($array,0,1,17);
    my $y2 = at($array,1,1,17);
    my $z2 = at($array,2,1,17);
    $height = $height + sqrt(($x2-$x1)**2 + ($y2-$y1)**2 + ($z2-$z1)**2);
  }
  for(my $c=21;$c<=24;$c++)
  {
    #base node
    my $x1 = at($array,0,1,$c);
    my $y1 = at($array,1,1,$c);
    my $z1 = at($array,2,1,$c);
    #apex node
    my $x2 = at($array,0,1,17);
    my $y2 = at($array,1,1,17);
    my $z2 = at($array,2,1,17);
    $height = $height + sqrt(($x2-$x1)**2 + ($y2-$y1)**2 + ($z2-$z1)**2);
  }
  $height = $height/8;
  print "Height = $height\n";
  return($height);
}


sub getCoords
{
  my($array, $n, $k) = @_;
  
  my ($a, $b, $c, $d);
  if(($n>=1) && ($n<=4))
  {
    $a = at($array,$k,1,$n);
    $b = at($array,$k,2,$n);
    $c = at($array,$k,3,$n);
    $d = at($array,$k,4,$n);
  } elsif(($n>=5) && ($n<=8))
  {
    $a = (at($array,$k,1,$n)+at($array,$k,1,$n+4))/2;
    $b = (at($array,$k,2,$n)+at($array,$k,2,$n+4))/2;
    $c = (at($array,$k,3,$n)+at($array,$k,3,$n+4))/2;
    $d = (at($array,$k,4,$n)+at($array,$k,4,$n+4))/2;
  } elsif((($n>=9) && ($n<=16)))
  {
    $a = at($array,$k,1,$n+8);
    $b = at($array,$k,2,$n+8);
    $c = at($array,$k,3,$n+8);
    $d = at($array,$k,4,$n+8);
  } elsif((($n>=17) && ($n<=20)))
  {
    $a = (at($array,$k,1,$n+8)+at($array,$k,1,$n+12))/2;
    $b = (at($array,$k,2,$n+8)+at($array,$k,2,$n+12))/2;
    $c = (at($array,$k,3,$n+8)+at($array,$k,3,$n+12))/2;
    $d = (at($array,$k,4,$n+8)+at($array,$k,4,$n+12))/2;
  } elsif((($n>=21) && ($n<=24)))
  {
    $a = at($array,$k,1,$n+16);
    $b = at($array,$k,2,$n+16);
    $c = at($array,$k,3,$n+16);
    $d = at($array,$k,4,$n+16);
  }
  return($a,$b,$c,$d);
}
