/* -- Include Files -- */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>
#include <cgi-decode.h>
/* For: CGI handling functions */

#define EXAMPLE_NUM "331"
#define EXAMPLE_PATH "3/33/"EXAMPLE_NUM

#define PI 3.14159265358979323846264

#define NMAX 100

#define CGI_HEADER \
  "Content-type: text/html \n\n" \
  "<HTML><HEAD>" \
  "<TITLE>Time dependent analytic solution</TITLE>" \
  "</HEAD><BODY>" \
  "<H1>Time dependent analytic solution</H1>" \
  "<P> This form calculates the analytic solution to the heat equation (del T/del t = alpha grad^2 T) on a plate of length L and height H with boundary conditions:" \
  "<UL>" \
  "  <LI> T(0,y,t)=0" \
  "  <LI> T(L,y,t)=sin(pi.y/H)" \
  "  <LI> T(x,0,t)=0" \
  "  <LI> T(x,H,t)=0" \
  "</UL>" \
  "and initial condition :" \
  "<UL>" \
  "  <LI> T(x,y,0)=0" \
  "</UL></P>" \
  "<HR>" \
  "\n"

#define CGI_FORM \
  "<P><FORM METHOD=\"GET\" " \
  "ACTION=\""CMISS_WWW_URL"scripts/analytic_"EXAMPLE_NUM".cgi\">" \
  "Number of terms in solution, N : <input name=\"N\" size=10 value=\"10\"> <br>" \
  "Plate length, l : <input name=\"L\" size=10 value=\"2.0\"> <br>" \
  "Plate width, h : <input name=\"H\" size=10 value=\"1.0\"> <br>" \
  "Diffusivity, alpha : <input name=\"alpha\" size=10 value=\"0.01\"> <br>" \
  "X location, x : <input name=\"X\" size=10 value=\"0.5\"> <br>" \
  "Y location, y : <input name=\"Y\" size=10 value=\"0.5\"> <br>" \
  "Time, t : <input name=\"t\" size=10 value=\"1.0\"> <br>" \
  "<input type=\"submit\" value=\"Calculate Analtyic\">" \
  "<input type=\"reset\" value=\"Reset defaults\">" \
  "</FORM></P><HR>" 

#define CGI_FOOTER \
  "<A HREF=\""CMISS_WWW_URL"help/examples/example_files/"EXAMPLE_PATH"/generated/test_page.html\">Return to the example page</A>"

void GetValues(CGI *cgi,
  int *n,
  double *l,
  double *h,
  double *alpha,
  double *x,
  double *y,
  double *t )
{
  char *nstr,*lstr,*hstr,*alphastr,*xstr,*ystr,*tstr;
    
  if( nstr = lookupString( cgi, "N" ) )
  {
    *n = atoi( nstr );
  }
  else  
  {
    *n = 10;
  }
  if( lstr = lookupString( cgi, "L" ) )
  {
    *l = atof( lstr );
  }
  else  
  {
    *l = 2.0;
  }
  if( hstr = lookupString( cgi, "H" ) )
  {
    *h = atof( hstr );
  }
  else  
  {
    *h = 1.0;
  }
  if( alphastr = lookupString( cgi, "alpha" ) )
  {
    *alpha = atof( alphastr );
  }
  else  
  {
    *alpha = 0.01;
  }
  if( xstr = lookupString( cgi, "X" ) )
  {
    *x = atof( xstr );
  }
  else  
  {
    *x = 0.5;
  }
  if( ystr = lookupString( cgi, "Y" ) )
  {
    *y = atof( ystr );
  }
  else  
  {
    *y = 0.5;
  }
  if( tstr = lookupString( cgi, "t" ) )
  {
    *t = atof( tstr );
  }
  else  
  {
    *t = 0.5;
  }
}

int CheckValues( int n,
  double l,
  double h,
  double alpha,
  double x,
  double y,
  double t )
{
  int error=0;

  if(n <= 0 || n > NMAX)
  {
    error=1;
    printf("ERROR: N outside the range 0 -- %d <BR> \n",NMAX);
  }
  if(l <= 0.0)
  {
    error=1;
    printf("ERROR: L is less than zero <BR> \n");
  }
  if(h <= 0.0)
  {
    error=1;
    printf("ERROR: H is less than zero <BR> \n");
  }
  if(alpha <= 0.0)
  {
    error=1;
    printf("ERROR: Alpha is less than zero <BR> \n");
  }
  if(x <= 0.0 || x > l)
  {
    error=1;
    printf("ERROR: X is outside the range 0 -- L <BR> \n");
  }
  if(y <= 0.0 || y > h)
  {
    error=1;
    printf("ERROR: Y is outside the range 0 -- H <BR> \n");
  }
  if(t <= 0.0)
  {
    error=1;
    printf("ERROR: t is less than zero <BR> \n");
  }
  
  if(error)
  {
    printf("<HR>\n");
  }

  return error;
}

void PrintValues( int n,
  double l,
  double h,
  double alpha,
  double x,
  double y,
  double t )
{
  printf("<P>\n");
  printf("Analytic solution for :\n");
  printf("<UL>\n");
  printf("<LI> Number of terms, n = %d\n",n);
  printf("<LI> Plate length, l = %7.2f\n",l);
  printf("<LI> Plate height, h = %7.2f\n",h);
  printf("<LI> Diffusion constant, alpha = %7.2f\n",alpha);
  printf("<LI> X location, x = %7.2f\n",x);
  printf("<LI> Y location, y = %7.2f\n",y);
  printf("<LI> Time, t = %7.3f\n",t);
  printf("</UL>\n");
  printf("</P><HR>\n");
}

void CalculateSolution( int n,
  double l,
  double h,
  double alpha,
  double x,
  double y,
  double t,
  double *timedep,
  double *steadystate,
  double *solution )
{
  int i;
  double an,tn,sum;

  sum=0.0;
  for(i=1;i<=n;i++)
  {
    an=(2.0*i*h*h*pow(-1.0,i))/(PI*(l*l+i*i*h*h));
    tn=-alpha*PI*PI*(i*i/(l*l)+1.0/(h*h))*t;
    sum+=an*sin(i*PI*x/l)*sin(PI*y/h)*exp(tn);
  }
  *timedep=sum;
  *steadystate=sinh(PI*x/h)*sin(PI*y/h)/sinh(PI*l/h);
  *solution=sum + *steadystate;
}

void PrintSolution( double timedep,
  double steadystate,
  double solution )
{
  printf("<P>\n");
  printf("Time dependent component = %12.10f\n",timedep);
  printf("<BR>\n");
  printf("Steady state component   = %12.10f\n",steadystate);
  printf("</P>\n");
  printf("<P>\n");
  printf("Analytic solution        = %12.10f\n",solution);
  printf("</P><HR>\n");
}

/* -- The Program Entry Point -- */

int main( int argc, char *argv[] )
{
  int n;
  double l,h,alpha,x,y,t,timedep,steadystate,solution;
  CGI *cgi;

  printf( CGI_HEADER );
  fflush( stdout );

  cgi = getCGIEnvironment( argc, argv );
  if( cgi == NULL )
  {
    printf( "getCGIEnvironment() failed in main()\n" );
  }
  else
  {
    if( nameExists( cgi, "N" ) )
    {
      GetValues( cgi, &n, &l, &h, &alpha, &x, &y, &t );
      PrintValues( n, l, h, alpha, x, y, t );
      if( !( CheckValues( n, l, h, alpha, x, y, t ) )  ) 
      {
        CalculateSolution( n, l, h, alpha, x, y, t, &timedep, &steadystate,&solution );
        PrintSolution( timedep, steadystate, solution );
      }
    }
    else
    {
      printf( CGI_FORM );
    }
  }

  printf( CGI_FOOTER );

  disposeCGI( cgi );
  fflush( stdout );

  return 0;
}
  






