#include <stdio.h>
/*
cc -o generate_comfile generate_comfile.c
*/
#define FAILURE 0
#define SUCCESS 1
#define FALSE 0
#define TRUE 1

#define MAX_SPHERES 6
#define MAX_NODES_IN_GROUP 100
enum Sphere_type
{
  BEM,
  FEM,
  INVALID
};

int main(int argc,char **argv)
{
  int i,return_code,arg_num;
  int num_spheres,fem_present,bem_present,around,azimuthal,
    num_nodes,source_rad,top_limit;
  enum Sphere_type sphere_type[MAX_SPHERES];
  char conductivity[MAX_SPHERES];
  int radius[MAX_SPHERES];
  char source_dir,source_rad_dir;
  FILE *comfile,*shfile;

  /* delete the file in case it doesn't work... */
  system("rm -f check_analytic.com");
  return_code = SUCCESS;
  num_spheres = 0;
  fem_present = FALSE;
  bem_present = FALSE;
  around = 0;
  azimuthal = 0;
  source_rad = 0;
  source_rad_dir = 'z';
  source_dir = 'z';
  /* Get the arguments */
  arg_num = 1;
  num_spheres = atoi(argv[arg_num++]);
  printf("Number of spheres:        %i\n",num_spheres);
  for(i=0;i<num_spheres;i++)
  {
    if(!strcmp(argv[arg_num],"bem"))
    {
      sphere_type[i] = BEM;
      bem_present = TRUE;
    }
    else if(!strcmp(argv[arg_num],"fem"))
    {
      sphere_type[i] = FEM;
      fem_present = TRUE;
    }
    else
    {
      sphere_type[i] = INVALID;
    }
    arg_num++;
    radius[i]=atoi(argv[arg_num++]);
    conductivity[i]=argv[arg_num++][0];
  }
  printf("Sphere types are:\n");
  for(i=0;i<num_spheres;i++)
  {
    switch(sphere_type[i])
    {
      case BEM:
      {
        printf("                          BEM        %i %c\n",
          radius[i],conductivity[i]);
      } break;
      case FEM:
      {
        printf("                          FEM        %i %c\n",
          radius[i],conductivity[i]);
      } break;
      default:
      {
        printf("                          Invalid\n");
      } break;
    }
  }
  around = atoi(argv[arg_num++]);
  printf("Elements around sphere:   %i\n",around);
  azimuthal = atoi(argv[arg_num++]);
  printf("Elements in azimuthal:    %i\n",azimuthal);
  num_nodes = (around*(azimuthal-1)+2)*num_spheres;
  printf("Total nodes:              %i\n",num_nodes);
  source_rad = atoi(argv[arg_num++]);
  printf("Source radius is:         %i\n",source_rad);
  source_rad_dir = argv[arg_num++][0];
  printf("Source radius dirn is:    %c\n",source_rad_dir);
  source_dir = argv[arg_num++][0];
  printf("Source direction is:      %c\n",source_dir);
  /* create the comfile */
  if(arg_num==argc)
  {
    comfile = fopen("check_analytic.com","w");
    fprintf(comfile,"set dir example=3/35\n");
    fprintf(comfile,"set fatal off\n");
    fprintf(comfile,"FEM de param;r;ippara/anal_check;example\n");
    fprintf(comfile,"FEM reallocate\n");
    fprintf(comfile,"FEM de coor;r;ipcoor/global_3d;example\n");
    fprintf(comfile,"FEM de bas;r;ipbase/bicubic");
    if(bem_present)
    {
      fprintf(comfile,"_bem");
    }
    if(fem_present)
    {
      fprintf(comfile,"_fem");
    }
    fprintf(comfile,";example\n");
    fprintf(comfile,"FEM de mesh;r;ipmesh/sphere_%1i",num_spheres);
    for(i=0;i<num_spheres;i++)
    {
      switch(sphere_type[i])
      {
        case BEM:
        {
          fprintf(comfile,"_bem");
        } break;
        case FEM:
        {
          fprintf(comfile,"_fem");
        } break;
      }
      fprintf(comfile,"_%03i",radius[i]);
    }
    fprintf(comfile,"_%02i_%02i;example\n",around,azimuthal);
    /* if eccentric, multi, then rotate */
    fprintf(comfile,"FEM de equ;r;ipequa/gen_laplace_%1i",num_spheres);
    for(i=0;i<num_spheres;i++)
    {
      switch(sphere_type[i])
      {
        case BEM:
        {
          fprintf(comfile,"_bem");
        } break;
        case FEM:
        {
          fprintf(comfile,"_fem");
        } break;
      }
    }
    if(fem_present)
    {
      fprintf(comfile,"_%02i_%02i",around,azimuthal);
    }
    if(num_spheres>1)
    {
      fprintf(comfile,";example reg all\n");
    }
    else
    {
      fprintf(comfile,";example\n");
    }
    fprintf(comfile,"FEM de mate;r;ipmate/gen_laplace_%1i",num_spheres);
    for(i=0;i<num_spheres;i++)
    {
      fprintf(comfile,"_%c",conductivity[i]);
    }
    if(num_spheres>1)
    {
      fprintf(comfile,";example reg all\n");
    }
    else
    {
      fprintf(comfile,";example\n");
    }
    /*
    fprintf(comfile,"FEM group node 1 as top\n");
    for(i=0;i<=((num_nodes-2)/MAX_NODES_IN_GROUP);i++)
    {
      if(i<((num_nodes-2)/MAX_NODES_IN_GROUP))
      {
        top_limit = (i+1)*MAX_NODES_IN_GROUP+1;
      }
      else
      {
        top_limit = num_nodes-1;
      }
      fprintf(comfile,"FEM group node %i..%i as middle_%1i\n",
        i*MAX_NODES_IN_GROUP+2,top_limit,i+1);
    }
    fprintf(comfile,"FEM group node %i as bottom\n",num_nodes);
    */
    fprintf(comfile,"FEM de sour;r;ipsour/dipole_%1i_%c_%c;example reg 1\n",
      source_rad,source_rad_dir,source_dir);
    fprintf(comfile,"FEM de anal;r;ipanal/sphere_%1i",num_spheres);
    if(source_rad==0&&num_spheres==1)
    {
      fprintf(comfile,"_centric");
    }
    else
    {
      fprintf(comfile,"_eccentric");
    }
    if(num_spheres==1)
    {
      fprintf(comfile,";example\n");
    }
    else
    {
      fprintf(comfile,"_%02i_%02i;example reg all\n",around,azimuthal);
    }
    fprintf(comfile,"!FEM li var;analytic number 7\n");
    /*
    fprintf(comfile,"FEM de ini;r;fix_top_%1i\n",
      (num_nodes-1)/MAX_NODES_IN_GROUP+1);
    */
    fprintf(comfile,"FEM de ini;g");
    if(num_spheres>1)
    {
      fprintf(comfile," reg all\n");
    }
    else
    {
      fprintf(comfile,"\n");
    }
    if(num_spheres>1)
    {
      fprintf(comfile,"FEM de coup;r;ipcoup/laplace;example\n");
    }
    fprintf(comfile,"FEM de sol;r;ipsolv/solve");
    if(num_spheres>1)
    {
      fprintf(comfile,"_coup_%1i;example coup reg all\n",num_spheres);
    }
    else
    {
      fprintf(comfile,";example\n");
    }
    fprintf(comfile,"FEM solve");
    if(num_spheres>1)
    {
      fprintf(comfile," coup\n");
    }
    else
    {
      fprintf(comfile,"\n");
    }
/*     fprintf(comfile,"!FEM li matr;matrix matr gk,gq,gd value\n"); */
    fprintf(comfile,"set output;sphere_%1i",num_spheres);
    for(i=0;i<num_spheres;i++)
    {
      switch(sphere_type[i])
      {
        case BEM:
        {
          fprintf(comfile,"_bem");
        } break;
        case FEM:
        {
          fprintf(comfile,"_fem");
        } break;
      }
      fprintf(comfile,"_%03i",radius[i]);
      fprintf(comfile,"_%c",conductivity[i]);
    }
    fprintf(comfile,"_%02i_%02i",around,azimuthal);
    fprintf(comfile,"_%1i_%c_%c",source_rad,source_rad_dir,source_dir);
    fprintf(comfile,"_inner on\n");
    fprintf(comfile,"FEM check solution reg 1\n");
    fprintf(comfile,"set output off\n");
    fprintf(comfile,"set output;sphere_%1i",num_spheres);
    for(i=0;i<num_spheres;i++)
    {
      switch(sphere_type[i])
      {
        case BEM:
        {
          fprintf(comfile,"_bem");
        } break;
        case FEM:
        {
          fprintf(comfile,"_fem");
        } break;
      }
      fprintf(comfile,"_%03i",radius[i]);
      fprintf(comfile,"_%c",conductivity[i]);
    }
    fprintf(comfile,"_%02i_%02i",around,azimuthal);
    fprintf(comfile,"_%1i_%c_%c",source_rad,source_rad_dir,source_dir);
    fprintf(comfile,"_outer on\n");
    fprintf(comfile,"FEM check solution reg %i\n",num_spheres);
    fprintf(comfile,"set output off\n");
    for(i=0;i<num_spheres;i++)
    {
      fprintf(comfile,"FEM export node;geometry_%i as sphere_geometry reg %i\n",i+1,i+1);
      fprintf(comfile,"FEM export node;computed_%i as sphere_computed field reg %i\n",i+1,i+1);
      fprintf(comfile,"FEM export node;analytic_%i as sphere_analytic field iy 7 reg %i\n",i+1,i+1);
      fprintf(comfile,"FEM export elem;geometry_%i as sphere_geometry_%i reg %i\n",i+1,i+1,i+1);
      fprintf(comfile,"FEM export elem;computed_%i as sphere_computed_%i field reg %i\n",i+1,i+1,i+1);
      fprintf(comfile,"FEM export elem;analytic_%i as sphere_analytic_%i field reg %i\n",i+1,i+1,i+1);
    }
    fprintf(comfile,"quit\n");
    fclose(comfile);
  }
  else
  {
    printf("Error parsing arguments:  Wanted %i - got %i.\n",
      arg_num,argc);
  }
  return return_code;
}
