# Example 5522 - Dynamic active contraction of a high-order prolate LV using HMT

# Sets up an LV mechanics model coupled to a HMT cell model, from which
# the active fibre tension is derived.  The LV, made up of two high
# order prolate elements, has been previously inflated (passively) to
# 1kPa.  The solution phase for active contraction uses a [Ca]i pulse
# (defined using a time variable) to drive the HMT model, which computes
# the active tension at all computational points, causing the LV to
# contract.

# Martyn Nash and David Nickerson June 2000



# Command updated by fixcom.sh on Wed Aug 23 18:20:26 NZT 2000
# Old command: assign variable SOLVE_INFLATION $FALSE
$SOLVE_INFLATION = $FALSE


# Command updated by fixcom.sh on Wed Aug 23 18:20:26 NZT 2000
# Old command: assign variable EXPORT $FALSE
$EXPORT = $FALSE


# Command updated by fixcom.sh on Wed Aug 23 18:20:26 NZT 2000
# Old command: assign variable MECH_ERRTOL 0.001
$MECH_ERRTOL = 0.001


# Solve classes

# Command updated by fixcom.sh on Wed Aug 23 18:20:26 NZT 2000
# Old command: assign variable MECH 1
$MECH = 1


# Command updated by fixcom.sh on Wed Aug 23 18:20:26 NZT 2000
# Old command: assign variable CELL 2
$CELL = 2


# Model regions

# Command updated by fixcom.sh on Wed Aug 23 18:20:26 NZT 2000
# Old command: assign variable WALL 1
$WALL = 1


fem define parameters;r;mech_hmt

# Set up the ventricular wall geometry
fem define coordinate;r;wall region $WALL #prolate spheriodal coords
fem define node;r;wall region $WALL        #6 nodes
fem define base;r;wall                     #lambda tricub; mu/theta trilin
fem define element;r;wall region $WALL     #2 wall elements
fem define fibre;r;wall region $WALL       #fibre: endo  65 deg; epi -55 deg
fem define element;r;wall fibre region $WALL  #sheet: endo -45 deg; epi  45 deg
fem group node 1..4,7,8 as ALL_NODES
fem group element 1..2 as ALL_ELEMENTS

# Set up the mechanics model
fem define equation;r;wall region $WALL class $MECH lock #Incomp finelas; FEM
fem define material;r;wall region $WALL class $MECH    #pole zero law
fem define active;r;mech_hmt region $WALL class $MECH  #passiv inflation
fem define initial;r;wall region $WALL class $MECH     #inflate to 1kPa
fem define solve;r;wall region $WALL class $MECH       #GMRES

# Set up the HMT cell model
fem define grid;r;mech_hmt region $WALL class $CELL  #5*5*5 grid scheme
fem group grid 1..225 as ALL_GRID_POINTS
fem update grid geometry region $WALL
fem update grid metric region $WALL
fem define equation;r;hmt region $WALL class $CELL lock  #HMT cell model
fem define cell;r;hmt class $CELL
fem define material;r;hmt cell class $CELL
fem define time;r;cai #time variable for the conc of intracellular Ca++
fem define initial;r;hmt class $CELL #use time-varying length/Cai
fem define solve;r;hmt class $CELL

# Solve the HMT cell model for initial setup
fem solve class $CELL to 0 

# Solve the mechanics model incrementally for passive inflation
if ($SOLVE_INFLATION)

# Command updated by fixcom.sh on Wed Aug 23 18:20:26 NZT 2000
# Old command:   fem solve class $MECH increment 0.1 iter 2 error $MECH_ERRTOL
{
  fem solve class $MECH increment 0.1 iter 2 error $MECH_ERRTOL

  fem solve class $MECH increment 0.4 iter 2 error $MECH_ERRTOL
  fem solve class $MECH increment 0.5 iter 99 error $MECH_ERRTOL
  fem define initial;w;wall_infl region $WALL class $MECH #inflated

# Command updated by fixcom.sh on Wed Aug 23 18:20:26 NZT 2000
# Old command: else
} else


# Command updated by fixcom.sh on Wed Aug 23 18:20:26 NZT 2000
# Old command:   fem define initial;r;wall_infl region $WALL class $MECH #inflated
{
  fem define initial;r;wall_infl region $WALL class $MECH #inflated

  fem define solve;r;wall region $WALL class $MECH        #GMRES

# Command updated by fixcom.sh on Wed Aug 23 18:20:26 NZT 2000
# Old command: endif
}


if ($EXPORT)

# Command updated by fixcom.sh on Wed Aug 23 18:20:26 NZT 2000
# Old command:   fem export node;test as test
{
  fem export node;test as test

  fem export element;test as test
  fem export element;parameters as parameters field cell
  fem export element;numbers as numbers grid_numbers

# Command updated by fixcom.sh on Wed Aug 23 18:20:26 NZT 2000
# Old command: endif
}


#read com;mech_hmt_solve
