#This example uses a linear approximation for small deformations to estimate the residual force (natural tension) from a in-vivo multiaxial 
#loading experiment on human skin. The result is used as an input in the material parameter estimation in example 7182. Note the example takes a LONG time to run

# If the example path is not set, default to current directory
if (!defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}
#**************************************************************
set num_threads 4
@DATASETS = (5,6,8,9)								#Deformed states used in the parameter estimation

fem define parameters;r;multiaxes;example
	
fem define bases;r;bases_cube;example						#Bi-cubic hermite for geomety, bi-linear for fibre 
fem define nodes;r;coordinates_cube_undeformed;example	
fem define elements;r;elements_cube;example					#Circular mesh with 192 elements

fem define fibre;r;fibre;example			
fem define elements;r;fibreelem_cube;example fibre

fem define equation;r;equation;example						#Incompressible membrane (uses plain-stress)
fem define material;r;tongfung;example						#Tong and Fungs skin function
	
fem group nodes 1..32 as BOUNDARY
	
fem define data;r;datapoints_undeformed;example					#Reads the undeformed coordinates of the datapoints
foreach $index (@DATASETS[1..$#DATASETS]){
	fem define data;r;datapoints_undeformed;example append
}

fem define xi;c closest								#Defines the xi positions of the datapoints

fem define data;r;datapoints_${DATASETS[0]};example as IMAGE_${DATASETS[0]}	#Reads the deformed coordinates of the datapoints
foreach $index (@DATASETS[1..$#DATASETS]){
	fem define data;r;datapoints_${index};example append as IMAGE_${index}
}

fem define initial;r;initskin_cube;example					#Defines a fixed boundary coordinate problem
fem define solve;r;solve;example
fem define optimise;r;tongfung_linetwo_cube;example				#Estimate two linear parameters (alfa1 and alfa2)

fem define nodes;r;displacements_cube_ref;example dep 2 nc 1			#Reads the boundary conditions

for $M (1..10){
	fem solve increment 0.1 iterate 1000					#Solves the fixed boundary problem in 10 increments
}

fem evaluate reactions noview
fem define heading force_cube_residual;example					#Evaluates the residual force using the current parameter values
fem define nodes;w;force_cube_residual;example dep 4 nc 1			#Writes the residual force to file
fem define nodes;r;force_cube_ref;example dep 4 nc 1				#Reads in zero values to reset the array

for $M (1..10){									#Repeates the parameter estimation and residual force evaluation

	optimise								#The optimiser calles the eval_resid_cube.com file
	
	fem list data errors deformed

	fem define initial;r;initskin_cube;example				#Defines a fixed boundary coordinate problem	
	fem define solve;r;solve;example

	fem define nodes;r;displacements_cube_ref;example dep 2 nc 1		#Reads the boundary conditions

	for $M (1..10){
		fem solve increment 0.1 iterate 1000				#Solves the fixed boundary problem in 10 increments
	}

	fem evaluate reactions noview
	fem define heading force_cube_residual					#Evaluates the residual force using the current parameter values
	fem define nodes;w;force_cube_residual;example dep 4 nc 1		#Writes the residual force to file
	fem define nodes;r;force_cube_ref;example dep 4 nc 1			#Reads in zero values to reset the array
	
}
