#Example 723
# This example demonstrates nonlinear optimisation 
# of material parameters by solving the forward problem
#
# This command file is used by the optimiser to solve the 
# forward problem for each objective evaluation


$TRUE = 1;  # Initialise logicals
$FALSE = 0;

if($RESOLVE ne "TRUE") 
{ 
  # Don't need to redo these commands when 
  # resolving during optimisation
  fem define parameters;r;shear;example;
  fem define coordinates;r;shear;example;

  fem def base;r;linear;example;
  fem def ;add base;r;lagr;example;
}

fem def node;r;shear_specimen_ref;example;
fem def elem;r;shear_ref;example;
fem def fibre;r;shear_ref;example;
fem def elem;r;shear_ref;example fibre;

fem group nodes 1,3,5,7 as LEFT;
fem group nodes 2,4,6,8 as RIGHT;
fem group nodes 1,2,5,6 as BOTTOM;
fem group nodes 3,4,7,8 as TOP;
fem group nodes 1,2,3,4 as BACK;
fem group nodes 5,6,7,8 as FRONT;

# Refinement cannot be done during in 
# the optimisation run. Refine mesh
# and write it out before the optimisation
# run e.g.
#
# (new nodes and elements will automatically
#  be added to the correct groups)
#
#fem refine xi 3;
#fem def node;w;shear_specimen_ref;
#fem def elem;w;shear_ref;
#fem def fibre;w;shear_ref; 
#fem def elem;w;shear_ref fibre;

#fem export nodes;shear;example as cube;
#fem export element;shear;example as cube;

fem def equa;r;shear;example;

fem def mate;r;guiccione_can_mid;example;

if($RESOLVE eq "TRUE") 
{ 
  # Need to update the material parameters 
  # to those being evaluated by the optimiser
  fem update optimisation
}

fem def init;r;shear_NF;example;

fem def solve;r;shear;example;

fem solve increment 0 error 0.01;

fem solve increment 0.2 iter 30 error 0.01;
if($RESOLVE eq "TRUE") 
{
  # Evaluate residuals & load residual vector - 10%
  fem evaluate reaction nodes FRONT direction 1 residual 1
  fem evaluate reaction nodes FRONT direction 2 residual 2
  fem evaluate reaction nodes FRONT direction 3 residual 3
}


fem solve increment 0.2 iter 30 error 0.01;
if($RESOLVE eq "TRUE") 
{
  # Evaluate residuals & load residual vector - 20% shear
  fem evaluate reaction nodes FRONT direction 1 residual 4
  fem evaluate reaction nodes FRONT direction 2 residual 5
  fem evaluate reaction nodes FRONT direction 3 residual 6
}

fem solve increment 0.2 iter 30 error 0.01;
if($RESOLVE eq "TRUE") 
{
  # Evaluate residuals & load residual vector - 30% shear
  fem evaluate reaction nodes FRONT direction 1 residual 7
  fem evaluate reaction nodes FRONT direction 2 residual 8
  fem evaluate reaction nodes FRONT direction 3 residual 9
}

fem solve increment 0.2 iter 30 error 0.01;
if($RESOLVE eq "TRUE") 
{
  # Evaluate residuals & load residual vector - 40% shear
  fem evaluate reaction nodes FRONT direction 1 residual 10
  fem evaluate reaction nodes FRONT direction 2 residual 11
  fem evaluate reaction nodes FRONT direction 3 residual 12
}

fem solve increment 0.2 iter 30 error 0.01;
if($RESOLVE eq "TRUE") 
{
  # Evaluate residuals & load residual vector - 50% shear
  fem evaluate reaction nodes FRONT direction 1 residual 13
  fem evaluate reaction nodes FRONT direction 2 residual 14
  fem evaluate reaction nodes FRONT direction 3 residual 15
}


if($RESOLVE eq "TRUE") 
{
  $ITERATION=$ITERATION + 1;
  print "\033[0;30;43m                    Iteration = ${ITERATION}                  \033[0m\n"; 
} 

if($RESOLVE ne "TRUE") 
{
  # Load the experimental data, force components in the 
  # x, y and z directions are in indecies 1, 2 and 3 
  # respectively
  fem define data;r;shear_NF;example;
  #
  # Create a vector of the experimental reactions
  # for use by the optimiser 
  fem update residual data point 1 index 1 residual 1
  fem update residual data point 1 index 2 residual 2
  fem update residual data point 1 index 3 residual 3
  #
  fem update residual data point 2 index 1 residual 4
  fem update residual data point 2 index 2 residual 5
  fem update residual data point 2 index 3 residual 6
  #
  fem update residual data point 3 index 1 residual 7
  fem update residual data point 3 index 2 residual 8
  fem update residual data point 3 index 3 residual 9
  #
  fem update residual data point 4 index 1 residual 10
  fem update residual data point 4 index 2 residual 11
  fem update residual data point 4 index 3 residual 12
  #
  fem update residual data point 5 index 1 residual 13
  fem update residual data point 5 index 2 residual 14
  fem update residual data point 5 index 3 residual 15

  $ITERATION = 1;
  $RESOLVE = "TRUE";
  fem define optimise;r;shear;example;           
  optimise; 
}


#fem export nodes;shear_def;example field as cube;
#fem export element;shear_def;example field as cube;



