      PROGRAM GEOM                                                      

C**** Sorts, adjusts and transforms geometric and fibre 
C**** orientation data from heart coordinate rig.
                           
C**** Function   CFROMI    Converts integer to character
C**** Function   IFROMC    Converts character to integer
C**** Subroutine CLOSE     Closes a file using the FORTRAN CLOSE command
C**** Subroutine FACORR    Fibre angle correction for sloping surface
C**** Subroutine LEVINPUT  Select data layers for "special treatment"
C**** Subroutine MIDLVD    Finds and relabels LV midwall data points
C**** Subroutine OPEN      Opens a file using the CMS FILEDEF command
C**** Subroutine PELEC     Calculates plunge electrode locations
C**** Subroutine RESET     Resets the transformation matrix TRANS to the identity matrix
C**** Subroutine RSWANG    Calculates the extent of RV septal endocardium
C**** Subroutine SEPSORT   Correscts direction of angle change beween data points (used with MIDLVD)
C**** Subroutine SHIFT/1-3 Updates the transformation matrix TRANS with a translation
C**** Subroutine TRIM      Finds the first and last non-blank characters in a string
C**** Subroutine TWIST/1-3 Updates the transformation matrix TRANS with a rotation
C**** Subroutine ZZ        Transforms rectangular cartesian coordinates

                                                                        
      PARAMETER (NJMX=4,NJX=3)                                   
      PARAMETER (NDMAX=20000,NDMA2=20000*NJMX)
                                       
      IMPLICIT REAL (A-H,O-Z)                                         

      INTEGER IX(NJMX,4),NZPE(300),NNDT(2500)

      REAL  ZDEPIC(NJX,1500),ZDRVFW(NJX,1000),ZDRVSW(NJX,1000),
     '  ZDLVEN(NJX,1600),ZDVALV(NJX,200),ZDCO(NJX,600),
     '  ZDEPF(NJMX,1500),ZDMYF(NJMX,NDMAX),ZDRFF(NJMX,750),
     '  ZDRSF(NJMX,750),ZDLF(NJMX,1250),ZDIB(NJMX,1000),
     '  ZDEE(3,300),ZDPE(3,300),ZDPC(3,300),ZDFAP(3,50),ZDTFA(4,750),
     '  ZDLVMF(NJMX,2500),ZDIBL1(NJMX,250)

      REAL  ZDLVPB(NJX,200),ZDLVPE(NJX,300),ZDLVPF(NJX,50),
     '  ZDLVPI(NJX,20),ZDLVPM(NJX,100),
     '  ZDRFPB(NJX,200),ZDRFPE(NJX,300),ZDRFPF(NJX,50),
     '  ZDRFPI(NJX,20),ZDRFPM(NJX,100),
     '  ZDRSPB(NJX,200),ZDRSPE(NJX,300),ZDRSPF(NJX,50),
     '  ZDRSPI(NJX,20),ZDRSPM(NJX,100),ZDEPEM(NJX,50)

      REAL  ZDLSLV(NJMX,15000),ZDLSRV(NJMX,3000),
     '      ZDLSLI(NJMX,1000),ZDLSRI(NJMX,200),ZDLSEP(NJMX,600)

      REAL  WDEPIC(NJX,1500),WDRVFW(NJX,1000),WDRVSW(NJX,1000),
     '  WDLVEN(NJX,1600),WDVALV(NJX,200),WDCO(NJX,600),
     '  WDEPF(NJMX,1500),WDMYF(NJMX,NDMAX),WDRFF(NJMX,750),
     '  WDRSF(NJMX,750),WDLF(NJMX,1250),WDIB(NJMX,1000),
     '  WDEE(3,300),WDPE(3,300),WDPC(3,300),WDFAP(3,50),WDTFA(4,750),
     '  WDLVMF(NJMX,2500),WDIBL1(NJMX,250)

      REAL  WDLVPB(NJX,200),WDLVPE(NJX,300),WDLVPF(NJX,50),
     '  WDLVPI(NJX,20),WDLVPM(NJX,100),
     '  WDRFPB(NJX,200),WDRFPE(NJX,300),WDRFPF(NJX,50),
     '  WDRFPI(NJX,20),WDRFPM(NJX,100),
     '  WDRSPB(NJX,200),WDRSPE(NJX,300),WDRSPF(NJX,50),
     '  WDRSPI(NJX,20),WDRSPM(NJX,100),WDEPEM(NJX,50)

      REAL  WDLSLV(NJMX,15000),WDLSRV(NJMX,3000),
     '      WDLSLI(NJMX,1000),WDLSRI(NJMX,200),WDLSEP(NJMX,600)

      REAL  Z(NJMX),ZD(NJMX,NDMAX),XD(NJMX,NDMAX),XXD(NJMX,4),
     '  TRANS(3,4),ZAPEX(NJMX),ZPE(3,300),SEPANG(3,50)

      CHARACTER FORMAT*500,LABEL(4)*12,ZLABEL(NDMAX)*12,XLABEL(4)*12,
     '  EELABL(300)*5,PELABL(300)*12,ELABEL(300)*12,PCLABL(300)*12,
     '  FALABL(750)*12,FNAME*6,ERROR*500,CTRIM*500,TITLE*80,DUMPEL*3

      CHARACTER  DEC1*1,DEC2*1,C135,CFROMI*500,
     '  LABLIST(40)*2,PRELAB*2,MLAYER(3)*2

      LOGICAL PLELEC,FANGLE,MIDLVY,LVBEG,LVEND,TH1LOW(50),CROSS0(50)

      DATA WDEPIC/4500*1.00/,WDRVFW/3000*1.00/,WDRVSW/3000*1.00/,
     '  WDLVEN/4800*1.00/,WDVALV/600*1.00/,WDCO/1800*1.00/,
     '  WDEPF/6000*1.00/,WDMYF/NDMA2*1.00/,WDRFF/3000*1.00/,
     '  WDRSF/3000*1.00/,WDLF/5000*1.00/,WDIB/4000*1.00/,
     '  WDEE/900*1.00/,WDPE/900*1.00/,WDPC/900*1.00/,WDFAP/150*1.00/,
     '  WDTFA/3000*1.00/,WDLVMF/10000*1.00/,XD/NDMA2*0.00/,
     '  WDIBL1/1000*1.00/,WDLSLV/60000*1.00/,WDLSRV/12000*1.00/,
     '  WDLSLI/4000*1.00/,WDLSRI/800*1.00/,WDLSEP/2400*1.00/

      DATA WDLVPB/600*1.00/,WDLVPE/900*1.00/,WDLVPF/150*1.00/,
     '  WDLVPI/60*1.00/,WDLVPM/300*1.00/,
     '  WDRFPB/600*1.00/,WDRFPE/900*1.00/,WDRFPF/150*1.00/,
     '  WDRFPI/60*1.00/,WDRFPM/300*1.00/,
     '  WDRSPB/600*1.00/,WDRSPE/900*1.00/,WDRSPF/150*1.00/,
     '  WDRSPI/60*1.00/,WDRSPM/300*1.00/,WDEPEM/150*1.00/

  
C     ECCEN=SQRT(0.750)                                                
      ECCEN=1.00/COSH(1.00)                                           
      NJM=NJMX                                                          
      NJT=3                                                             
      JTYP9=1
      PLELEC=.FALSE.                                                     
      FANGLE=.FALSE.                                                     
      MIDLVY=.FALSE.                                                     
      CALL OPEN(3,'TERM','SYS$INPUT','NEW',' ',132,ERROR,*9999)         
      FORMAT='(/''  Specify the input filename (eg H15): '')'           
      WRITE(3,FORMAT)                                                   
      READ(3,'(A6)') FNAME                                              
      FORMAT='(/''  Any Plunge Electrode data: (Y/N)? '')'                  
      WRITE(3,FORMAT)                                                   
      READ(3,'(A1)') DEC1                                               
      IF(DEC1.EQ.'Y'.OR.DEC1.EQ.'y') THEN
        PLELEC=.TRUE.
      ENDIF                           
      FORMAT='(/''  Do you want F.Angles corrected: (Y/N)? '')'            
      WRITE(3,FORMAT)                                                   
      READ(3,'(A1)') DEC2
      IF(DEC2.EQ.'Y'.OR.DEC2.EQ.'y') THEN
        FANGLE=.TRUE.
      ENDIF                           
      IF(FANGLE) THEN
        FORMAT='(/''  Do you want LV middle layer separate: (Y/N)? '')'         
        WRITE(3,FORMAT)                                                   
        READ(3,'(A1)') DEC3
        IF(DEC3.EQ.'Y'.OR.DEC3.EQ.'y') THEN
          MIDLVY=.TRUE.
        ENDIF                           
      ENDIF
      CALL TRIM(FNAME,IBEG,IEND)                                        
      CALL OPEN(1,'DISK',FNAME(IBEG:IEND)//'.DATA','OLD','SEQUEN',128,  
     '  ERROR,*9999)                                                    
      PI=ATAN(1.00)*4.00                                              
      NDT=0
      NPEP=0                                                       
      NDBASE=1                                                          
      NDAPEX=1                                                          
      NL=0
      NLL=0        
 
C *** Read Hx.DATA file     

      WRITE(3,'(/,''  Reading Data File'',/)')                                        
      DO 2 NREC=1,10000                                                 
                                                                        
        FORMAT='(4(5I4,A12))'                                           
        READ(1,FORMAT,END=3) (N,(IX(NJ,ND),NJ=1,NJT+JTYP9),LABEL(ND),   
     '    ND=1,4)                                                       

        DO 1 ND=1,4                                                     
          IF(LABEL(ND).NE.'            ') THEN                          
            NDT=NDT+1                                                   
            XD(1,NDT)=(IX(1,ND))/10.0                                    
            XD(2,NDT)=(IX(2,ND))/10.0                               
            XD(3,NDT)=(IX(3,ND))/10.0                                    
            XD(4,NDT)=(IX(4,ND))                                
            ZLABEL(NDT)=LABEL(ND)                                       

C ***       Add spaces for interpolated plunge electrode terminals

            IF((INDEX(ZLABEL(NDT),'PE').GT.0)
     '          .AND.(INDEX(ZLABEL(NDT),'IM').EQ.0)) THEN
              NDT=NDT+1
              ZLABEL(NDT)=ZLABEL(NDT-1)
              NDT=NDT+1
              ZLABEL(NDT)=ZLABEL(NDT-2)
            ENDIF

C ***       List of LV MY layers

            IF(MIDLVY) THEN
              IF(INDEX(ZLABEL(NDT),'LV MY').GT.0) THEN
                IF(ZLABEL(NDT)(8:9).NE.PRELAB) THEN
                  NL=NL+1
                  LABLIST(NL)=ZLABEL(NDT)(8:9)
                  PRELAB=LABLIST(NL)
                ENDIF
              ENDIF
            ENDIF
          ENDIF


    1   CONTINUE                                                        
    2 CONTINUE                                                          
    3 WRITE(3,'(/,''  Read completed'')')                                  
      WRITE(*,*) ' Number of Data Points = ',NDT
      IF(MIDLVY) THEN
        CALL LEVINPUT(NL,LABLIST,NLL,MLAYER)
      ENDIF
                                                            
C *** Fibre angle correction                                             

      IF(FANGLE) THEN                                               
        CALL FACORR(XD,ZLABEL,NDT,NDMAX,PI)                                  
      ENDIF

C **  Fill ZD Array

      LVBEG=.FALSE.
      LVEND=.FALSE.
      NZZ=0         !For "sepang"

      DO 34 N = 1, NDT
	ZD(1,N) = XD(1,N)*COS(XD(2,N)*PI/180.0)
	ZD(2,N) = XD(1,N)*SIN(XD(2,N)*PI/180.0)
	ZD(3,N) = XD(3,N)
	ZD(4,N) = XD(4,N)*PI/180.0

	IF(INDEX(ZLABEL(N),'APX').GT.0)THEN
	  NDAPEX = N
	  WRITE(3,'('' Original Apex = '',I4,1X,3E11.4,2X,A12)')
     '         N,(ZD(NJ,N),NJ=1,NJT),ZLABEL(N)
	ELSE IF(INDEX(ZLABEL(N),'BASEAXIS').GT.0)THEN
	  NDBASE = N
	  WRITE(3,'('' Original Base = '',I4,1X,3E11.4,2X,A12)')
     '         N,(ZD(NJ,N),NJ=1,NJT),ZLABEL(N)
	ELSE IF(INDEX(ZLABEL(N),'LC MT OR').GT.0)THEN
	  NDLCOR = N
	  WRITE(3,'('' Original LMC  = '',I4,1X,3E11.4,2X,A12)')
     '         N,(ZD(NJ,N),NJ=1,NJT),ZLABEL(N)
	ENDIF
     
        IF((INDEX(ZLABEL(N),'PE').GT.0)
     '      .AND.(INDEX(ZLABEL(N),'IM').EQ.0)) THEN
          NPEP=NPEP+1
          ZPE(1,NPEP)=ZD(1,N)
          ZPE(2,NPEP)=ZD(2,N)
          ZPE(3,NPEP)=ZD(3,N)
          NZPE(NPEP)=N
          ELABEL(NPEP)=ZLABEL(N)
        ENDIF

        IF(MIDLVY) THEN
          IF(INDEX(ZLABEL(N),'RV EN SW').GT.0) THEN
            CALL RSWANG(NDT,N,XD,SEPANG,TH1LOW,NZZ)
          ENDIF
     
          IF(.NOT.LVBEG.AND.INDEX(ZLABEL(N),'LV MY').GT.0) THEN
            LVBEG=.TRUE.
            NLVBEG=N
          ENDIF
          IF(LVBEG.AND.(.NOT.LVEND)) THEN
            IF(INDEX(ZLABEL(N),'LV MY').EQ.0) THEN
              LVEND=.TRUE.
              NLVEND=N-1           
              WRITE (3,'(/,'' NLVBEGIN = '',I5)')NLVBEG
              WRITE (3,'('' NLVEND   = '',I5)')NLVEND
            ENDIF
          ENDIF
        ENDIF

 34   CONTINUE	  

C *** Tidy SEPANG
      
      IF(MIDLVY) THEN
        CALL SEPSORT(NZZ,TH1LOW,SEPANG,CROSS0)
      ENDIF          

C *** Find LV data points to be included in midwall surface

      IF(MIDLVY) THEN
        CALL MIDLVD(NLVBEG,NLVEND,NDT,XD,ZLABEL,
     ,              NLL,MLAYER,NZZ,SEPANG,CROSS0)
      ENDIF
 
C *** Calculate coordinates of plunge electrodes

      IF(PLELEC) THEN
        CALL PELEC(ZD,ZPE,ZLABEL,ELABEL,NZPE,NPEP,NDT)
      ENDIF

C *** Transform so that lamda=1 at apex & mu=120 degs at base are both   
C *** on central axis. This defines initial focal point.                 
                                                                        
      WRITE(3,'(/,''  Transforming Coordinates *1'')')                                        

      CALL RESET(TRANS)                                                 
      Z(1)=-(ZD(1,NDAPEX)+ZD(1,NDBASE)*2.00/ECCEN)/(1.00+2.00/ECCEN) 
      Z(2)=-(ZD(2,NDAPEX)+ZD(2,NDBASE)*2.00/ECCEN)/(1.00+2.00/ECCEN) 
      Z(3)=-(ZD(3,NDAPEX)+ZD(3,NDBASE)*2.00/ECCEN)/(1.00+2.00/ECCEN) 
      AMOUNT=SQRT(Z(1)**2+Z(2)**2+Z(3)**2)
                              
      CALL SHIFT(Z,AMOUNT,TRANS)                                        
      ZAPEX(1)=ZD(1,NDAPEX)                                             
      ZAPEX(2)=ZD(2,NDAPEX)                                             
      ZAPEX(3)=ZD(3,NDAPEX)
                                             
      CALL ZZ(ZAPEX,ZAPEX,TRANS)                                        
      Z(1)= ZAPEX(2)                                                    
      Z(2)=-ZAPEX(1)                                                    
      Z(3)=0.00                                                        
      AMOUNT=ATAN2(SQRT(ZAPEX(1)**2+ZAPEX(2)**2),ZAPEX(3))              
      WRITE(3,'(/''  Apex angle = '',F16.8)') AMOUNT*180.00/PI
          
      CALL TWIST(Z,AMOUNT,TRANS)                                        
      Z(1)=0.0                                                        
      Z(2)=1.0                                                        
      Z(3)=0.0
                                                        
      CALL TWIST(Z,PI/2.0,TRANS)                                      

C *** Transform to prolate origin                                   
C *** Accumulate r.v. thetas to define zero for theta
                   
      SUM1=0.0                                                        
      NSUM=0                                                            
      DO 4 ND=1,NDT                                                     
        CALL ZZ(ZD(1,ND),ZD(1,ND),TRANS)                                
        IF(INDEX(ZLABEL(ND),'RV EN').GT.0)THEN                          
          ANGLE=ATAN2(ZD(3,ND),ZD(2,ND))                                
          SUM1=SUM1+ANGLE                                               
          NSUM=NSUM+1                                                   
        ENDIF                                                           
    4 CONTINUE
                                                          
      Z(1)=1.0                                                        
      Z(2)=0.0                                                        
      Z(3)=0.0                                                        
                                                                        
      AMOUNT=-SUM1/NSUM                                                 
                                                                        
      WRITE(3,'(''  Rven angle = '',F16.8)') AMOUNT*180.0/PI
           
      CALL RESET(TRANS)                                                 
      CALL TWIST(Z,AMOUNT,TRANS)

C *** Shift origin along x axis so that it lies
C *** 1/2.6 the distance between origin of LAD and apex.

      AMOUNT=-((ZD(1,NDAPEX)+ZD(1,NDLCOR)*1.6)/2.6)
      CALL SHIFT(Z,AMOUNT,TRANS)                                        
                                        
C *** Rotate and shift along X axis

      WRITE(3,'(/,''  Transforming Coordinates *2'')')                                        

      DO 5 ND=1,NDT
                                                     
        CALL ZZ(ZD(1,ND),ZD(1,ND),TRANS)                                

C ***   Check that apex and base are on central axis                    

        IF(INDEX(ZLABEL(ND),'APX').GT.0)THEN                           
          WRITE(3,'('' Final Apex = '',I4,1X,3E11.4,2X,A12)')                  
     '         ND,(ZD(NJ,ND),NJ=1,NJT),ZLABEL(ND)                       
        ELSE IF(INDEX(ZLABEL(ND),'BASEAXIS').GT.0)THEN                  
          WRITE(3,'('' Final Base = '',I4,1X,3E11.4,2X,A12)')                  
     '         ND,(ZD(NJ,ND),NJ=1,NJT),ZLABEL(ND)                       
        ELSE IF(INDEX(ZLABEL(ND),'LC MT OR').GT.0)THEN                  
          WRITE(3,'('' Final LMC  = '',I4,1X,3E11.4,2X,A12)')         
     '         ND,(ZD(NJ,ND),NJ=1,NJT),ZLABEL(ND)                       
        ENDIF                                                           
    5 CONTINUE                                                          
                                                                        
C *** ** SORT **
                                                                     
      DO 7 N=1,1                                                        
        FOCUS=ECCEN*ZD(1,NDAPEX)                                        
        WRITE(3,'(/,''  Focal ratio  ='',E11.4,'' Focal length  ='',
     '           E11.4)')ECCEN,FOCUS                                     
 
        NDCO=0                                                          
        NDTFA=0
        NDEPIC=0                                                        
        NDEPF=0
        NDRVFW=0                                                        
        NDRFF=0
        NDRVSW=0                                                        
        NDRSF=0
        NDLVEN=0
        NDLF=0                                    
        NDVALV=0                                                        
        NDFAP=0
        NDMYF=0
        NDLVMF=0                                        
        NDEE=0
        NDPE=0
        NDPC=0
        NDIBL1=0
        NDIB=0
        NDRFPB=0      !Purkinje fibre border
        NDRFPE=0      !Endo cardial PF bundle                             
        NDRFPF=0      !Free PF bundle (false tendon)                      
        NDRFPI=0      !PF bundle insertion (beginning/end of false tendon)
        NDRFPM=0      !Papillary muscle base                              
        NDRSPB=0      !    etc
        NDRSPE=0
        NDRSPF=0
        NDRSPI=0
        NDRSPM=0
        NDLVPB=0
        NDLVPE=0
        NDLVPF=0
        NDLVPI=0
        NDLVPM=0
        NDEPEM=0      !EM blocks
        NDLSLV=0      !Longitudinal sections - LV
        NDLSRV=0      !Longitudinal sections - RV
        NDLSLI=0      !Longitudinal sections - LV INT
        NDLSRI=0      !Longitudinal sections - RV INT
        NDLSEP=0      !Longitudinal sections - EP


        DO 6 ND=1,NDT                                                   
          IF((ABS(ZD(2,ND)).LT.1.0E-6).AND.(ABS(ZD(3,ND)).LT.1.0E-6))
     '         THEN               
            WRITE(3,110)                                                 
110         FORMAT('  DATA POINT ON AXIS INSIDE FOCUS')                     
          ENDIF                                                         

C ***     Coronary Vessels

          IF((INDEX(ZLABEL(ND),'MT').GT.0).OR.                             
     '       (INDEX(ZLABEL(ND),'BR').GT.0))THEN
            NDCO=NDCO+1                                                     
            ZDCO(1,NDCO)=ZD(1,ND)                                           
            ZDCO(2,NDCO)=ZD(2,ND)                                           
            ZDCO(3,NDCO)=ZD(3,ND)                                           
          ENDIF                                                           

C ***     Transmural fibre angle point - (location) 
                                             
          IF(INDEX(ZLABEL(ND),'TFAP').GT.0)THEN                     
            NDFAP=NDFAP+1                                             
            ZDFAP(1,NDFAP)=ZD(1,ND)                                   
            ZDFAP(2,NDFAP)=ZD(2,ND)                                   
            ZDFAP(3,NDFAP)=ZD(3,ND)                                   
          ENDIF                                                         

C ***     Epicardium
                                                    
          IF(((INDEX(ZLABEL(ND),'EPIC').GT.0)                            
     '        .AND.(INDEX(ZLABEL(ND),'EM').EQ.0))
     '      .OR.(INDEX(ZLABEL(ND),'APX').GT.0)
     '      .OR.((INDEX(ZLABEL(ND),'BASE').GT.0)
     '         .AND.(INDEX(ZLABEL(ND),'AXIS').EQ.0))) THEN
            NDEPIC=NDEPIC+1
            ZDEPIC(1,NDEPIC)=ZD(1,ND)                                   
            ZDEPIC(2,NDEPIC)=ZD(2,ND)                                   
            ZDEPIC(3,NDEPIC)=ZD(3,ND)                                   
 
            IF ((INDEX(ZLABEL(ND),'NFA').EQ.0)
     '        .AND.(INDEX(ZLABEL(ND),'VA').EQ.0)) THEN
              NDEPF=NDEPF+1
              ZDEPF(1,NDEPF)=ZD(1,ND)
              ZDEPF(2,NDEPF)=ZD(2,ND)
              ZDEPF(3,NDEPF)=ZD(3,ND)
              ZDEPF(4,NDEPF)=ZD(4,ND)
            ENDIF
 
          ENDIF                                                         

C ***     R.V. endocardium free wall
                                             
          IF(INDEX(ZLABEL(ND),'RV EN FW').GT.0)THEN                     
            NDRVFW=NDRVFW+1                                             
            ZDRVFW(1,NDRVFW)=ZD(1,ND)                                   
            ZDRVFW(2,NDRVFW)=ZD(2,ND)                                   
            ZDRVFW(3,NDRVFW)=ZD(3,ND)                                   
 
            IF ((INDEX(ZLABEL(ND),'NFA').EQ.0)
     '        .AND.(INDEX(ZLABEL(ND),'VA').EQ.0)) THEN
              NDRFF=NDRFF+1
              ZDRFF(1,NDRFF)=ZD(1,ND)
              ZDRFF(2,NDRFF)=ZD(2,ND)
              ZDRFF(3,NDRFF)=ZD(3,ND)
              ZDRFF(4,NDRFF)=ZD(4,ND)
            ENDIF
 
          ENDIF                                                         

C ***     R.V. endocardium septal wall
                                                   
          IF(INDEX(ZLABEL(ND),'RV EN SW').GT.0)THEN                     
            NDRVSW=NDRVSW+1                                             
            ZDRVSW(1,NDRVSW)=ZD(1,ND)                                   
            ZDRVSW(2,NDRVSW)=ZD(2,ND)                                   
            ZDRVSW(3,NDRVSW)=ZD(3,ND)                                   

            IF ((INDEX(ZLABEL(ND),'NFA').EQ.0)
     '        .AND.(INDEX(ZLABEL(ND),'VA').EQ.0)) THEN
              NDRSF=NDRSF+1
              ZDRSF(1,NDRSF)=ZD(1,ND)
              ZDRSF(2,NDRSF)=ZD(2,ND)
              ZDRSF(3,NDRSF)=ZD(3,ND)
              ZDRSF(4,NDRSF)=ZD(4,ND)
            ENDIF
 
          ENDIF                                                         

C ***     L.V. endocardium
                                              
          IF(INDEX(ZLABEL(ND),'LV EN').GT.0) THEN                        
            NDLVEN=NDLVEN+1                                             
            ZDLVEN(1,NDLVEN)=ZD(1,ND)                                   
            ZDLVEN(2,NDLVEN)=ZD(2,ND)                                   
            ZDLVEN(3,NDLVEN)=ZD(3,ND)                                   

            IF ((INDEX(ZLABEL(ND),'NFA').EQ.0)
     '          .AND.(INDEX(ZLABEL(ND),'VA').EQ.0)) THEN
              NDLF=NDLF+1
              ZDLF(1,NDLF)=ZD(1,ND)
              ZDLF(2,NDLF)=ZD(2,ND)
              ZDLF(3,NDLF)=ZD(3,ND)
              ZDLF(4,NDLF)=ZD(4,ND)
            ENDIF
 
          ENDIF                                                         

C ***     Valves
                                                        
          IF(INDEX(ZLABEL(ND),'VA').GT.0) THEN                     
            NDVALV=NDVALV+1                                                 
            ZDVALV(1,NDVALV)=ZD(1,ND)                                       
            ZDVALV(2,NDVALV)=ZD(2,ND)                                       
            ZDVALV(3,NDVALV)=ZD(3,ND)                                       
          ENDIF                                                           

C ***     Transmural fibre angle point - (intramural) 
                                             
          IF(ZLABEL(ND)(10:10).EQ.'F') THEN                     
            NDTFA=NDTFA+1                                             
            ZDTFA(1,NDTFA)=ZD(1,ND)                                   
            ZDTFA(2,NDTFA)=ZD(2,ND)                                   
            ZDTFA(3,NDTFA)=ZD(3,ND)                                   
            ZDTFA(4,NDTFA)=ZD(4,ND)*180.0/PI
            FALABL(NDTFA)=ZLABEL(ND)
          ENDIF                                                         

C ***     Myocardial Fibres
                                                          
          IF((INDEX(ZLABEL(ND),'MY').GT.0)      
     ,        .AND.(INDEX(ZLABEL(ND),'NFA').EQ.0)                               
     ,        .AND.(INDEX(ZLABEL(ND),'VA').EQ.0)) THEN
            NDMYF=NDMYF+1                                                     
            ZDMYF(1,NDMYF)=ZD(1,ND)                                           
            ZDMYF(2,NDMYF)=ZD(2,ND)                                           
            ZDMYF(3,NDMYF)=ZD(3,ND)                                           
            ZDMYF(4,NDMYF)=ZD(4,ND)                                          
          ENDIF                                                           

C ***     LV Middle layer  
                                                          
          IF(INDEX(ZLABEL(ND),'LV MIDWALL').GT.0) THEN     
            NDLVMF=NDLVMF+1                                                     
            ZDLVMF(1,NDLVMF)=ZD(1,ND)                                           
            ZDLVMF(2,NDLVMF)=ZD(2,ND)                                           
            ZDLVMF(3,NDLVMF)=ZD(3,ND)                                           
            ZDLVMF(4,NDLVMF)=ZD(4,ND)
            NNDT(NDLVMF)=ND

          ENDIF                                                           

C ***     Epicardial Electrodes

          IF(INDEX(ZLABEL(ND),'EE').GT.0)THEN                             
            NDEE=NDEE+1                                                     
            ZDEE(1,NDEE)=ZD(1,ND)                                           
            ZDEE(2,NDEE)=ZD(2,ND)                                           
            ZDEE(3,NDEE)=ZD(3,ND)                                           
            EELABL(NDEE)=ZLABEL(ND)(1:5)
          ENDIF                                                           

C ***     Plunge Electrodes

          IF((INDEX(ZLABEL(ND),'PE').GT.0)
     '       .AND.(INDEX(ZLABEL(ND),'IM').EQ.0)) THEN
            NDPE=NDPE+1                                                     
            ZDPE(1,NDPE)=ZD(1,ND)                                           
            ZDPE(2,NDPE)=ZD(2,ND)                                           
            ZDPE(3,NDPE)=ZD(3,ND)
            PELABL(NDPE)=ZLABEL(ND)
          ENDIF

C ***     Plunge Electrode Checking File

          IF(INDEX(ZLABEL(ND),'IM').GT.0) THEN
            NDPC=NDPC+1                                                     
            ZDPC(1,NDPC)=ZD(1,ND)                                           
            ZDPC(2,NDPC)=ZD(2,ND)                                           
            ZDPC(3,NDPC)=ZD(3,ND)                                           
            PCLABL(NDPC)=ZLABEL(ND)
          ENDIF

C ***     Infarcted Zone Border - First level
                                                          
          IF((INDEX(ZLABEL(ND),'IB').GT.0)
     '        .AND.(INDEX(ZLABEL(ND),'L01').GT.0)) THEN
            NDIBL1=NDIBL1+1                                                     
            ZDIBL1(1,NDIBL1)=ZD(1,ND)                                           
            ZDIBL1(2,NDIBL1)=ZD(2,ND)                                           
            ZDIBL1(3,NDIBL1)=ZD(3,ND)                                           
            ZDIBL1(4,NDIBL1)=ZD(4,ND)                                          
          ENDIF                                                           

C ***     Infarcted Zone Border
                                                          
          IF((INDEX(ZLABEL(ND),'IB').GT.0))  THEN
            NDIB=NDIB+1                                                     
            ZDIB(1,NDIB)=ZD(1,ND)                                           
            ZDIB(2,NDIB)=ZD(2,ND)                                           
            ZDIB(3,NDIB)=ZD(3,ND)                                           
            ZDIB(4,NDIB)=ZD(4,ND)                                          
          ENDIF                                                           

C ***     RV FW Purkinje Fibre border        

          IF(INDEX(ZLABEL(ND),'RV FW PFB').GT.0) THEN
            NDRFPB=NDRFPB+1                                                     
            ZDRFPB(1,NDRFPB)=ZD(1,ND)                                           
            ZDRFPB(2,NDRFPB)=ZD(2,ND)                                           
            ZDRFPB(3,NDRFPB)=ZD(3,ND)                                           
          ENDIF
         
C ***     RV FW Purkinje Bundle - endocardial  

          IF(INDEX(ZLABEL(ND),'RV FW PBE').GT.0) THEN
            NDRFPE=NDRFPE+1                                                     
            ZDRFPE(1,NDRFPE)=ZD(1,ND)                                           
            ZDRFPE(2,NDRFPE)=ZD(2,ND)                                           
            ZDRFPE(3,NDRFPE)=ZD(3,ND)                                           
          ENDIF
         
C ***     RV FW Purkinje Bundle - Free

          IF(INDEX(ZLABEL(ND),'RV FW PBF').GT.0) THEN
            NDRFPF=NDRFPF+1                                                     
            ZDRFPF(1,NDRFPF)=ZD(1,ND)                                           
            ZDRFPF(2,NDRFPF)=ZD(2,ND)                                           
            ZDRFPF(3,NDRFPF)=ZD(3,ND)                                           
          ENDIF
         
C ***     RV FW Free Purkinje Bundle - insertion    

          IF(INDEX(ZLABEL(ND),'RV FW PBI').GT.0) THEN
            NDRFPI=NDRFPI+1                                                     
            ZDRFPI(1,NDRFPI)=ZD(1,ND)                                           
            ZDRFPI(2,NDRFPI)=ZD(2,ND)                                           
            ZDRFPI(3,NDRFPI)=ZD(3,ND)                                           
          ENDIF

C ***     RV FW Papillary muscle base          

          IF(INDEX(ZLABEL(ND),'RV FW PMB').GT.0) THEN
            NDRFPM=NDRFPM+1                                                     
            ZDRFPM(1,NDRFPM)=ZD(1,ND)                                           
            ZDRFPM(2,NDRFPM)=ZD(2,ND)                                           
            ZDRFPM(3,NDRFPM)=ZD(3,ND)                                           
          ENDIF
         
C ***     RV SW Purkinje Fibre border        

          IF(INDEX(ZLABEL(ND),'RV SW PFB').GT.0) THEN
            NDRSPB=NDRSPB+1                                                     
            ZDRSPB(1,NDRSPB)=ZD(1,ND)                                           
            ZDRSPB(2,NDRSPB)=ZD(2,ND)                                           
            ZDRSPB(3,NDRSPB)=ZD(3,ND)                                           
          ENDIF
         
C ***     RV SW Purkinje Bundle - endocardial  

          IF(INDEX(ZLABEL(ND),'RV SW PBE').GT.0) THEN
            NDRSPE=NDRSPE+1                                                     
            ZDRSPE(1,NDRSPE)=ZD(1,ND)                                           
            ZDRSPE(2,NDRSPE)=ZD(2,ND)                                           
            ZDRSPE(3,NDRSPE)=ZD(3,ND)                                           
          ENDIF
         
C ***     RV SW Purkinje Bundle - Free

          IF(INDEX(ZLABEL(ND),'RV SW PBF').GT.0) THEN
            NDRSPF=NDRSPF+1                                                     
            ZDRSPF(1,NDRSPF)=ZD(1,ND)                                           
            ZDRSPF(2,NDRSPF)=ZD(2,ND)                                           
            ZDRSPF(3,NDRSPF)=ZD(3,ND)                                           
          ENDIF
         
C ***     RV SW Free Purkinje Bundle - insertion    

          IF(INDEX(ZLABEL(ND),'RV SW PBI').GT.0) THEN
            NDRSPI=NDRSPI+1                                                     
            ZDRSPI(1,NDRSPI)=ZD(1,ND)                                           
            ZDRSPI(2,NDRSPI)=ZD(2,ND)                                           
            ZDRSPI(3,NDRSPI)=ZD(3,ND)                                           
          ENDIF

C ***     RV SW Papillary muscle base          

          IF(INDEX(ZLABEL(ND),'RV SW PMB').GT.0) THEN
            NDRSPM=NDRSPM+1                                                     
            ZDRSPM(1,NDRSPM)=ZD(1,ND)                                           
            ZDRSPM(2,NDRSPM)=ZD(2,ND)                                           
            ZDRSPM(3,NDRSPM)=ZD(3,ND)                                           
          ENDIF
         
C ***     LV EN Purkinje Fibre border        

          IF(INDEX(ZLABEL(ND),'LV    PFB').GT.0) THEN
            NDLVPB=NDLVPB+1                                                     
            ZDLVPB(1,NDLVPB)=ZD(1,ND)                                           
            ZDLVPB(2,NDLVPB)=ZD(2,ND)                                           
            ZDLVPB(3,NDLVPB)=ZD(3,ND)                                           
          ENDIF
         
C ***     LV EN Purkinje Bundle - endocardial  

          IF(INDEX(ZLABEL(ND),'LV    PBE').GT.0) THEN
            NDLVPE=NDLVPE+1                                                     
            ZDLVPE(1,NDLVPE)=ZD(1,ND)                                           
            ZDLVPE(2,NDLVPE)=ZD(2,ND)                                           
            ZDLVPE(3,NDLVPE)=ZD(3,ND)                                           
          ENDIF
         
C ***     LV EN Purkinje Bundle - Free

          IF(INDEX(ZLABEL(ND),'LV    PBF').GT.0) THEN
            NDLVPF=NDLVPF+1                                                     
            ZDLVPF(1,NDLVPF)=ZD(1,ND)                                           
            ZDLVPF(2,NDLVPF)=ZD(2,ND)                                           
            ZDLVPF(3,NDLVPF)=ZD(3,ND)                                           
          ENDIF
         
C ***     LV EN Free Purkinje Bundle - insertion    

          IF(INDEX(ZLABEL(ND),'LV    PBI').GT.0) THEN
            NDLVPI=NDLVPI+1                                                     
            ZDLVPI(1,NDLVPI)=ZD(1,ND)                                           
            ZDLVPI(2,NDLVPI)=ZD(2,ND)                                           
            ZDLVPI(3,NDLVPI)=ZD(3,ND)                                           
          ENDIF

C ***     LV EN Papillary muscle base          

          IF(INDEX(ZLABEL(ND),'LV    PMB').GT.0) THEN
            NDLVPM=NDLVPM+1                                                     
            ZDLVPM(1,NDLVPM)=ZD(1,ND)                                           
            ZDLVPM(2,NDLVPM)=ZD(2,ND)                                           
            ZDLVPM(3,NDLVPM)=ZD(3,ND)                                           
          ENDIF
         
C ***     EM Blocks
                                                        
          IF(INDEX(ZLABEL(ND),'EM').GT.0) THEN                     
            NDEPEM=NDEPEM+1                                                 
            ZDEPEM(1,NDEPEM)=ZD(1,ND)                                       
            ZDEPEM(2,NDEPEM)=ZD(2,ND)                                       
            ZDEPEM(3,NDEPEM)=ZD(3,ND)                                       
          ENDIF                                                           

C ***     LV Longitudinal sections
                                                        
          IF (((INDEX(ZLABEL(ND),'lv').GT.0)
     '       .OR.(INDEX(ZLABEL(ND),'ens').GT.0)                     
     '       .OR.(INDEX(ZLABEL(ND),'tns').GT.0)                     
     '       .OR.(INDEX(ZLABEL(ND),'trs').GT.0))                     
     '       .AND.(INDEX(ZLABEL(ND),'int').EQ.0)) THEN                     

!            IF ((INDEX(ZLABEL(ND),'tn').EQ.0)
!     '        .AND.(INDEX(ZLABEL(ND),'tr').EQ.0)) THEN                     

              NDLSLV=NDLSLV+1                                                 
              ZDLSLV(1,NDLSLV)=ZD(1,ND)                                       
              ZDLSLV(2,NDLSLV)=ZD(2,ND)                                       
              ZDLSLV(3,NDLSLV)=ZD(3,ND)                                       
              ZDLSLV(4,NDLSLV)=ZD(4,ND)                                  
!            ENDIF
          ENDIF                                                           
      
C ***     RV Longitudinal sections
                                                        
          IF((INDEX(ZLABEL(ND),'rv').GT.0)                     
     '       .AND.(INDEX(ZLABEL(ND),'ens').EQ.0)                     
     '       .AND.(INDEX(ZLABEL(ND),'tns').EQ.0)                     
     '       .AND.(INDEX(ZLABEL(ND),'trs').EQ.0)                     
     '       .AND.(INDEX(ZLABEL(ND),'int').EQ.0)) THEN                     

!            IF ((INDEX(ZLABEL(ND),'tn').EQ.0)
!     '        .AND.(INDEX(ZLABEL(ND),'tr').EQ.0)) THEN                     
              NDLSRV=NDLSRV+1                                                 
              ZDLSRV(1,NDLSRV)=ZD(1,ND)                                       
              ZDLSRV(2,NDLSRV)=ZD(2,ND)                                       
              ZDLSRV(3,NDLSRV)=ZD(3,ND)                                       
              ZDLSRV(4,NDLSRV)=ZD(4,ND)                    
!            ENDIF
          ENDIF                                                           

C ***     LV Longitudinal sections -Intersecting sheets
                                                        
          IF (((INDEX(ZLABEL(ND),'lv').GT.0)
     '       .OR.(INDEX(ZLABEL(ND),'ens').GT.0)                     
     '       .OR.(INDEX(ZLABEL(ND),'tns').GT.0)                     
     '       .OR.(INDEX(ZLABEL(ND),'trs').GT.0))                     
     '       .AND.(INDEX(ZLABEL(ND),'int').GT.0)) THEN                     

            NDLSLI=NDLSLI+1                                                 
            ZDLSLI(1,NDLSLI)=ZD(1,ND)                                       
            ZDLSLI(2,NDLSLI)=ZD(2,ND)                                       
            ZDLSLI(3,NDLSLI)=ZD(3,ND)                                       
            ZDLSLI(4,NDLSLI)=ZD(4,ND)                                  
          ENDIF                                                           
      
C ***     RV Longitudinal sections -Intersecting sheets
                                                        
          IF((INDEX(ZLABEL(ND),'rv').GT.0)                     
     '       .AND.(INDEX(ZLABEL(ND),'ens').EQ.0)                     
     '       .AND.(INDEX(ZLABEL(ND),'tns').EQ.0)                     
     '       .AND.(INDEX(ZLABEL(ND),'trs').EQ.0)                     
     '       .AND.(INDEX(ZLABEL(ND),'int').GT.0)) THEN                     

            NDLSRI=NDLSRI+1                                                 
            ZDLSRI(1,NDLSRI)=ZD(1,ND)                                       
            ZDLSRI(2,NDLSRI)=ZD(2,ND)                                       
            ZDLSRI(3,NDLSRI)=ZD(3,ND)                                       
            ZDLSRI(4,NDLSRI)=ZD(4,ND)                                  
          ENDIF                                                           
      
C ***     EP Longitudinal sections
                                                        
          IF((INDEX(ZLABEL(ND),'ep').GT.0)                     
     '       .AND.(INDEX(ZLABEL(ND),'int').EQ.0)) THEN                     
            NDLSEP=NDLSEP+1                                                 
            ZDLSEP(1,NDLSEP)=ZD(1,ND)                                       
            ZDLSEP(2,NDLSEP)=ZD(2,ND)                                       
            ZDLSEP(3,NDLSEP)=ZD(3,ND)                                       
            ZDLSEP(4,NDLSEP)=ZD(4,ND)                    
          ENDIF                                                           

         
    6   CONTINUE                                                        
    7 CONTINUE                                                          


  


C *** ** Write to Files **

      WRITE(3,'(/,''   Writing to files'')')

    8 CALL TRIM(FNAME,IBEG,IEND)
                          
C ***     Coronary Vessels

      IF(NDCO.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_CO.IPDATA','NEW',              
     ,            'SEQUEN',132,ERROR,*9999)                                    
        WRITE(2,'('' DATA FOR CORONARY VESSELS      Focal Length ='',
     ,             E11.4)')FOCUS                                   
        DO ND=1,NDCO                                                  
          WRITE(2,'(I5,6E11.3)')ND,(ZDCO(NJ,ND),NJ=1,3),         
     ,         (WDCO(NJ,ND),NJ=1,3)
        ENDDO                                                              
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     Transmural fibre angle point - (location) 

      IF(NDTFA.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_FAP.IPDATA','NEW',              
     ,            'SEQUEN',132,ERROR,*9999)                                    
        WRITE(2,'('' TRANSMURAL FIBRE ANGLE POINTS - intramural'',
     ,            ''    Focal Length ='', E11.4)')FOCUS
        DO ND=1,NDTFA                                                 
          WRITE(2,'(I5,8E11.3,5X,A12)')ND,(ZDTFA(NJ,ND),NJ=1,4),         
     ,         (WDTFA(NJ,ND),NJ=1,4),FALABL(ND)
        ENDDO                                                  
        CALL CLOSE(2,ERROR,*9999)
      ENDIF

C ***     Epicardium

      IF(NDEPIC.GT.0) THEN              
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_EP.IPDATA','NEW',              
     '            'SEQUEN',132,ERROR,*9999)                                       
        WRITE(2,'('' DATA FOR EPI      Focal Length ='', E11.4)')FOCUS      
        DO ND=1,NDEPIC                                                
          WRITE(2,'(I5,6E11.3)') ND,(ZDEPIC(NJ,ND),NJ=1,3),     
     '         (WDEPIC(NJ,ND),NJ=1,3)
        ENDDO
        CALL CLOSE(2,ERROR,*9999)
      ENDIF

C ***     Epicardium - fibres

      IF(NDEPF.GT.0) THEN                              
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_EP_F.IPDATA','NEW',              
     '            'SEQUEN',132,ERROR,*9999)                                       
        WRITE(2,'('' FIBRE DATA FOR EPI      Focal Length ='', E11.4)')
     '            FOCUS                                     
        DO ND=1,NDEPF                                                 
          WRITE(2,'(I5,8E11.3)') ND,(ZDEPF(NJ,ND),NJ=1,4),      
     '         (WDEPF(NJ,ND),NJ=1,4)
        ENDDO
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF
                                                                
C ***     R.V. endocardium free wall

      IF(NDRVFW.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_RF.IPDATA','NEW',              
     '            'SEQUEN',132,ERROR,*9999)                                       
        WRITE(2,'('' DATA FOR RV EN FW      Focal Length ='', E11.4)')
     '            FOCUS                                     
        DO ND=1,NDRVFW                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDRVFW(NJ,ND),NJ=1,3),       
     ,         (WDRVFW(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     R.V. endocardium free wall - fibres

      IF(NDRFF.GT.0) THEN                              
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_RF_F.IPDATA','NEW',              
     '            'SEQUEN',132,ERROR,*9999)                                       
        WRITE(2,'('' FIBRE DATA FOR RV EN FW      Focal Length ='',
     ,            E11.4)')FOCUS                             
        DO ND=1,NDRFF                                                 
          WRITE(2,'(I5,8E11.3)') ND,(ZDRFF(NJ,ND),NJ=1,4),      
     '         (WDRFF(NJ,ND),NJ=1,4)
        ENDDO
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     R.V. endocardium septal wall

      IF(NDRVSW.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_RS.IPDATA','NEW',              
     '            'SEQUEN',132,ERROR,*9999)                                       
        WRITE(2,'('' DATA FOR RV EN SW      Focal Length ='', E11.4)')
     ,             FOCUS                                     
        DO ND=1,NDRVSW                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDRVSW(NJ,ND),NJ=1,3),       
     ,         (WDRVSW(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     R.V. endocardium septal wall - fibres

      IF(NDRSF.GT.0) THEN                              
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_RS_F.IPDATA','NEW',              
     '            'SEQUEN',132,ERROR,*9999)                                       
        WRITE(2,'('' FIBRE DATA FOR RV EN SW      Focal Length ='',
     ,            E11.4)')FOCUS                                  
        DO ND=1,NDRSF                                                 
          WRITE(2,'(I5,8E11.3)') ND,(ZDRSF(NJ,ND),NJ=1,4),      
     '         (WDRSF(NJ,ND),NJ=1,4)
        ENDDO
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     L.V. endocardium

      IF(NDLVEN.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_LV.IPDATA','NEW',              
     '            'SEQUEN',132,ERROR,*9999)                                       
        WRITE(2,'('' DATA FOR LV EN     Focal Length ='', E11.4)')FOCUS                                  
        DO ND=1,NDLVEN                                                
          WRITE(2,'(I5,6E11.3)') ND,(ZDLVEN(NJ,ND),NJ=1,3),      
     .         (WDLVEN(NJ,ND),NJ=1,3)
        ENDDO
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     L.V. endocardium - fibres

      IF(NDLF.GT.0) THEN                              
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_LV_F.IPDATA','NEW',              
     '            'SEQUEN',132,ERROR,*9999)                                       
        WRITE(2,'('' FIBRE DATA FOR LV EN      Focal Length ='', E11.4)
     ,            ')FOCUS                                     
        DO ND=1,NDLF                                                  
         WRITE(2,'(I5,8E11.3)') ND,(ZDLF(NJ,ND),NJ=1,4),      
     '        (WDLF(NJ,ND),NJ=1,4)
        ENDDO
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     Valves

      IF(NDVALV.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_VA.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)                                      
        WRITE(2,'('' DATA FOR VALVES      Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDVALV                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDVALV(NJ,ND),NJ=1,3),       
     ,         (WDVALV(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     Transmural fibre angle point - (intramural) 

      IF(NDFAP.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_TFA.IPDATA','NEW',              
     '            'SEQUEN',132,ERROR,*9999)                                       
        WRITE(2,'('' TRANSMURAL FIBRE ANGLE POINTS - location'',
     '            ''    Focal Length ='', E11.4)')FOCUS        
        DO ND=1,NDFAP                                                    
          WRITE(2,'(I5,6E11.3)')ND,(ZDFAP(NJ,ND),NJ=1,3),       
     ,         (WDFAP(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF
 
C ***     Myocardial Fibres

      IF(NDMYF.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_MY_F.IPDATA','NEW',              
     ,            'SEQUEN',132,ERROR,*9999)                                     
        WRITE(2,'('' FIBRE DATA FOR MY      Focal Length ='', E11.4)')
     ,             FOCUS                                     
        DO ND=1,NDMYF
          WRITE(2,'(I5,8E11.3)')ND,(ZDMYF(NJ,ND),NJ=1,4),         
     ,         (WDMYF(NJ,ND),NJ=1,4)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     LV Middle layer - fibres  

      IF(NDLVMF.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_LVMW_F.IPDATA','NEW',              
     ,            'SEQUEN',132,ERROR,*9999)                                     
        WRITE(2,'('' FIBRE DATA FOR LV MIDWALL      Focal Length ='',
     '             E11.4)')FOCUS                                     
        DO ND=1,NDLVMF
          WRITE(2,'(I5,8E11.3)')ND,(ZDLVMF(NJ,ND),NJ=1,4),         
     ,         (WDLVMF(NJ,ND),NJ=1,4)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

      IF(NDLVMF.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_LVMW_CHECK.IPDATA','NEW',              
     ,            'SEQUEN',132,ERROR,*9999)                                     
        WRITE(2,'('' CHECK FILE FOR LV MIDWALL      Focal Length ='',
     '             E11.4)')FOCUS                                     
        DO ND=1,NDLVMF
          WRITE(2,'(I5,4E11.3)')NNDT(ND),(XD(NJ,NNDT(ND)),NJ=1,4)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     Epicardial Electrodes

      IF(NDEE.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_EP_E.IPDATA','NEW',              
     ,            'SEQUEN',132,ERROR,*9999)                                    
        WRITE(2,'('' DATA FOR EE      Focal Length ='', E11.4)')FOCUS                                   
        DO ND=1,NDEE                
          NDD=IFROMC(EELABL(ND)(3:5))
          WRITE(2,'(I5,6E11.3,5X,A5)')NDD,(ZDEE(NJ,ND),NJ=1,3),         
     ,         (WDEE(NJ,ND),NJ=1,3),EELABL(ND)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)
      ENDIF

C ***     Plunge Electrodes

      IF(PLELEC) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_MY_E.IPDATA','NEW',              
     ,            'SEQUEN',132,ERROR,*9999)                                    
        WRITE(2,'('' DATA FOR PE      Focal Length ='', E11.4)')FOCUS                                   
        DO ND=1,NDPE                                                  
          NDD=IFROMC(PELABL(ND)(4:5))
          IF(MOD(ND,6).GT.0) THEN
            NDD1=IFROMC(PELABL(ND)(9:9))
          ELSE  
            NDD1=0
          ENDIF
          NDD=10*NDD+NDD1
          WRITE(2,'(I5,6E11.3,5X,A12)')NDD,(ZDPE(NJ,ND),NJ=1,3),         
     ,         (WDPE(NJ,ND),NJ=1,3),PELABL(ND)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)
       
        NPE=NDPE/6
        WRITE(3,'(''  PE number ='',I2, '' No. data pts. ='',I3)')
     '        NPE,NDPE                                                   
        NC=1              
        DO I=1,NPE
          DUMPEL=PELABL(NC)(3:5)
          CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_MY_E_'//DUMPEL//
     ,              '.IPDATA','NEW','SEQUEN',132,ERROR,*9999)
          WRITE(2,'('' DATA FOR PE'',A3,''   Focal Length ='', E11.4)')
     ,              DUMPEL,FOCUS
      
          DO ND=NC,NC+5
            WRITE(2,'(I5,6E11.3)')ND,(ZDPE(NJ,ND),NJ=1,3),
     ,           (WDPE(NJ,ND),NJ=1,3)
          ENDDO
          CALL CLOSE(2,ERROR,*9999)
          NC=NC+6
        ENDDO
      
        DO ND=1,NDPE-1,2
          NC135=MOD(ND,6)
          C135=CFROMI(NC135,'(I1)')
          DUMPEL=PELABL(ND)(3:5)
C         write (*,*) ND
          CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_MY_E_'//DUMPEL//
     '        '_'//C135//'.IPDATA','NEW','SEQUEN',132,ERROR,*9999)
          WRITE(2,'('' DATA FOR PE'',A3, ''_'',A1,
     ,           ''   Focal Length ='',E11.4)')DUMPEL,C135,FOCUS
          WRITE(2,'(I5,6E11.3)')ND,(ZDPE(NJ,ND),NJ=1,3),
     ,         (WDPE(NJ,ND),NJ=1,3)
C         write (*,*) ND
          CALL CLOSE(2,ERROR,*9999)
C         write (*,*) ND
        ENDDO

C ***     Plunge Electrode Checking File

        IF(NDPC.GT.0) THEN
          CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_MY_EC.IPDATA','NEW',              
     ,              'SEQUEN',132,ERROR,*9999)                                    
          WRITE(2,'('' DATA FOR PE CHECKING      Focal Length ='',
     ,               E11.4)')FOCUS                                   
          DO ND=1,NDPC                                                  
            WRITE(2,'(I5,6E11.3,5X,A12)')ND,(ZDPC(NJ,ND),NJ=1,3),         
     ,           (WDPC(NJ,ND),NJ=1,3),PCLABL(ND)
          ENDDO                                                   
          CALL CLOSE(2,ERROR,*9999)
        ENDIF

      ENDIF
      
C ***     Infarcted Zone Border - First level

      IF(NDIBL1.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_IB_L1.IPDATA','NEW',  
     ,            'SEQUEN',132,ERROR,*9999)                                     
        WRITE(2,'('' DATA FOR IB - FIRST LEVEL    Focal Length ='',
     ,             E11.4)')FOCUS                                     
        DO ND=1,NDIBL1
          WRITE(2,'(I5,8E11.3)')ND,(ZDIBL1(NJ,ND),NJ=1,4),         
     ,         (WDIBL1(NJ,ND),NJ=1,4)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     Infarcted Zone Border

      IF(NDIB.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_IB.IPDATA','NEW',              
     ,            'SEQUEN',132,ERROR,*9999)                                     
        WRITE(2,'('' DATA FOR INFARCTED ZONE BORDER    Focal Length ='',
     ,             E11.4)')FOCUS                                     
        DO ND=1,NDIB
          WRITE(2,'(I5,8E11.3)')ND,(ZDIB(NJ,ND),NJ=1,4),         
     ,         (WDIB(NJ,ND),NJ=1,4)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     RV FW Purkinje Fibre border        

      IF(NDRFPB.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_RF_PFB.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)       
        WRITE(2,'('' DATA FOR RF PF BORDER    Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDRFPB                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDRFPB(NJ,ND),NJ=1,3),       
     ,         (WDRFPB(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     RV FW Purkinje Bundle - endocardial  

      IF(NDRFPE.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_RF_PBE.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)       
        WRITE(2,'('' DATA FOR RF P BUNDLE ENDO   Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDRFPE                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDRFPE(NJ,ND),NJ=1,3),       
     ,         (WDRFPE(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     RV FW Purkinje Bundle - Free

      IF(NDRFPF.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_RF_PBF.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)       
        WRITE(2,'('' DATA FOR RF P BUNDLE FREE   Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDRFPF                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDRFPF(NJ,ND),NJ=1,3),       
     ,         (WDRFPF(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     RV FW Free Purkinje Bundle - insertion    

      IF(NDRFPI.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_RF_PBI.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)       
        WRITE(2,'('' DATA FOR RF P BUNDLE INSERT  Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDRFPI                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDRFPI(NJ,ND),NJ=1,3),       
     ,         (WDRFPI(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     RV FW Papillary muscle base          

      IF(NDRFPM.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_RF_PMB.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)       
        WRITE(2,'('' DATA FOR RF P MUSCLE BASE  Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDRFPM                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDRFPM(NJ,ND),NJ=1,3),       
     ,         (WDRFPM(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     RV SW Purkinje Fibre border        

      IF(NDRSPB.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_RS_PFB.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)       
        WRITE(2,'('' DATA FOR RS PF BORDER    Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDRSPB                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDRSPB(NJ,ND),NJ=1,3),       
     ,         (WDRSPB(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     RV SW Purkinje Bundle - endocardial  

      IF(NDRSPE.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_RS_PBE.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)       
        WRITE(2,'('' DATA FOR RS P BUNDLE ENDO   Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDRSPE                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDRSPE(NJ,ND),NJ=1,3),       
     ,         (WDRSPE(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     RV SW Purkinje Bundle - Free

      IF(NDRSPF.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_RS_PBF.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)       
        WRITE(2,'('' DATA FOR RS P BUNDLE FREE   Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDRSPF                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDRSPF(NJ,ND),NJ=1,3),       
     ,         (WDRSPF(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     RV SW Free Purkinje Bundle - insertion    

      IF(NDRSPI.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_RS_PBI.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)       
        WRITE(2,'('' DATA FOR RS P BUNDLE INSERT  Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDRSPI                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDRSPI(NJ,ND),NJ=1,3),       
     ,         (WDRSPI(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     RV SW Papillary muscle base          

      IF(NDRSPM.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_RS_PMB.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)       
        WRITE(2,'('' DATA FOR RS P MUSCLE BASE  Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDRSPM                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDRSPM(NJ,ND),NJ=1,3),       
     ,         (WDRSPM(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     LV EN Purkinje Fibre border        

      IF(NDLVPB.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_LV_PFB.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)       
        WRITE(2,'('' DATA FOR LV PF BORDER    Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDLVPB                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDLVPB(NJ,ND),NJ=1,3),       
     ,         (WDLVPB(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     LV EN Purkinje Bundle - endocardial  

      IF(NDLVPE.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_LV_PBE.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)       
        WRITE(2,'('' DATA FOR LV P BUNDLE ENDO   Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDLVPE                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDLVPE(NJ,ND),NJ=1,3),       
     ,         (WDLVPE(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     LV EN Purkinje Bundle - Free

      IF(NDLVPF.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_LV_PBF.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)       
        WRITE(2,'('' DATA FOR LV P BUNDLE FREE   Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDLVPF                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDLVPF(NJ,ND),NJ=1,3),       
     ,         (WDLVPF(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     LV EN Free Purkinje Bundle - insertion    

      IF(NDLVPI.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_LV_PBI.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)       
        WRITE(2,'('' DATA FOR LV P BUNDLE INSERT  Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDLVPI                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDLVPI(NJ,ND),NJ=1,3),       
     ,         (WDLVPI(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     LV EN Papillary muscle base          

      IF(NDLVPM.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_LV_PMB.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)       
        WRITE(2,'('' DATA FOR LV P MUSCLE BASE  Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDLVPM                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDLVPM(NJ,ND),NJ=1,3),       
     ,         (WDLVPM(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     EM Blocks                            

      IF(NDEPEM.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_EP_EM.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)       
        WRITE(2,'('' DATA FOR EM BLOCKS  Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDEPEM                                                
          WRITE(2,'(I5,6E11.3)')ND,(ZDEPEM(NJ,ND),NJ=1,3),       
     ,         (WDEPEM(NJ,ND),NJ=1,3)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF

C ***     Longitudinal sections - LV

      IF(NDLSLV.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_LV_LS.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)                                      
        WRITE(2,'('' DATA FOR LV LS  Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDLSLV                                                
          WRITE(2,'(I5,8E11.3)')ND,(ZDLSLV(NJ,ND),NJ=1,4),       
     ,         (WDLSLV(NJ,ND),NJ=1,4)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF


C ***     Longitudinal sections - RV

      IF(NDLSRV.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_RV_LS.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)                                      
        WRITE(2,'('' DATA FOR RV LS  Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDLSRV                                                
          WRITE(2,'(I5,8E11.3)')ND,(ZDLSRV(NJ,ND),NJ=1,4),       
     ,         (WDLSRV(NJ,ND),NJ=1,4)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF


C ***     Longitudinal sections - LV int

      IF(NDLSLI.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_LI_LS.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)                                      
        WRITE(2,'('' DATA FOR LV LS-int  Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDLSLI                                                
          WRITE(2,'(I5,8E11.3)')ND,(ZDLSLI(NJ,ND),NJ=1,4),       
     ,         (WDLSLI(NJ,ND),NJ=1,4)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF


C ***     Longitudinal sections - RV int

      IF(NDLSRI.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_RI_LS.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)                                      
        WRITE(2,'('' DATA FOR RV LS-int  Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDLSRI                                                
          WRITE(2,'(I5,8E11.3)')ND,(ZDLSRI(NJ,ND),NJ=1,4),       
     ,         (WDLSRI(NJ,ND),NJ=1,4)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF


C ***     Longitudinal sections - EP

      IF(NDLSEP.GT.0) THEN
        CALL OPEN(2,'DISK',FNAME(IBEG:IEND)//'_EP_LS.IPDATA','NEW',            
     ,            'SEQUEN',132,ERROR,*9999)                                      
        WRITE(2,'('' DATA FOR EP LS  Focal Length ='', E11.4)')
     ,             FOCUS                                   
        DO ND=1,NDLSEP                                                
          WRITE(2,'(I5,8E11.3)')ND,(ZDLSEP(NJ,ND),NJ=1,4),       
     ,         (WDLSEP(NJ,ND),NJ=1,4)
        ENDDO                                                   
        CALL CLOSE(2,ERROR,*9999)                                         
      ENDIF



C ***     Write main file sizes to screen           

      WRITE(3,1000) NDCO
 1000 FORMAT (' NDCO   = ',I5)

      WRITE (3,1100) NDEPIC,NDEPF
 1100 FORMAT (' NDEPIC = ',I5,5X,'NDEPF = ',I5)

      WRITE (3,1200) NDRVFW,NDRFF
 1200 FORMAT (' NDRVFW = ',I5,5X,'NDRFF = ',I5)

      WRITE (3,1300) NDRVSW,NDRSF      
 1300 FORMAT (' NDRVSW = ',I5,5X,'NDRSF = ',I5)

      WRITE (3,1400) NDMYF,NDIB
 1400 FORMAT (' NDMYF  = ',I5,5X,'NDIB  = ',I5)

      WRITE (3,1500) NDLVEN,NDLF
 1500 FORMAT (' NDLVEN = ',I5,5X,'NDLF  = ',I5)

      WRITE(3,1600) NDVALV
 1600 FORMAT (' NDVALV = ',I5)

      WRITE (3,1700) NDFAP,NDTFA
 1700 FORMAT (' NDFAP  = ',I5,5X,'NDTFA = ',I5)

      WRITE (3,1800) NDPE,NDPC,NDEE
 1800 FORMAT (' NDPE   = ',I5,5X,'NDPC  = ',I5,
     ,         5X,'NDEE   = ',I5)

      WRITE (3,1900) NDLVMF
 1900 FORMAT (' ND LV - MIDWALL LAYER    = ',I5)

      WRITE (3,2000) NDLSLV,NDLSRV
 2000 FORMAT (' NDLSLV  = ',I5,5X,'NDLSRV  = ',I5)

      WRITE (3,2100) NDLSLI,NDLSRI
 2100 FORMAT (' NDLSLI  = ',I5,5X,'NDLSRI  = ',I5)

      WRITE (3,2200) NDLSEP
 2200 FORMAT (' NDLSEP  = ',I5)

      CALL CLOSE(3,ERROR,*9999)                                         
      CALL CLOSE(1,ERROR,*9999)                                         
      STOP                                                              

 9999 WRITE(1,'(/132A)') ' Fatal error propagated to DATA'              
      CALL TRIM(ERROR,IBEG,IEND)                                        
      WRITE(1,'(132A)') ERROR(IBEG:IEND)//' DATA>'                      
      STOP                                                              
      END                                                               
                                                                        

                                                                        
      CHARACTER*(*) FUNCTION CFROMI(IDATA,FORMAT)                       
                                                                        
C**** Converts integer variable IDATA to character string CFROMI        
C**** as specified by the character string FORMAT.                      
                                                                        
      INTEGER IDATA                                                     
      CHARACTER FORMAT*(*),CDATA*500                                    
                                                                        
      WRITE(UNIT=CDATA,FMT=FORMAT) IDATA                                
      CFROMI=CDATA                                                      
      RETURN                                                            
      END                                                               
      
      
      INTEGER FUNCTION IFROMC(CDATA)

C**** Converts character string CDATA to an integer variable.

      CHARACTER CDATA*(*)

      READ (UNIT=CDATA,FMT=*) IDATA
      IFROMC=IDATA
      RETURN
      END

                                                                        
      SUBROUTINE CLOSE(IUNIT,ERROR,*)                                   
                                                                        
C**** Closes a file using the FORTRAN CLOSE command.                    
C**** IUNIT is the name by which the file is referred to in the program.
C**** Direct access files are truncated at their current record.        
C**** ERROR gives diagnostics in the event of failure.                  
C**** If an error is detected control is returned to the statement      
C**** number of the star.                                               
C**** ERROR is a character string.                                      
                                                                        
      CHARACTER ERROR*(*),CTRIM*500,CFROMI*500,ACCESS*10,UNIT*4,        
     '  STATUS*10,CIUNIT*500                                            
      LOGICAL   EXIST,OPENED                                            
                                                                        
C     CALL ENTERS('CLOSE')                                              
      INQUIRE(UNIT=IUNIT,IOSTAT=IOSTAT,EXIST=EXIST,OPENED=OPENED,       
     '  ACCESS=ACCESS,NEXTREC=IREC)                                     
      IF(EXIST.AND.OPENED) THEN                                         
        CLOSE(UNIT=IUNIT,IOSTAT=IOSTAT)                                 
        IF(IOSTAT.EQ.0) THEN                                            
          IF(ACCESS.EQ.'DIRECT') THEN                                   
            OPEN(UNIT=IUNIT,STATUS='OLD',ACCESS='SEQUENTIAL',           
     '           IOSTAT=IOSTAT)                                         
            IF(IOSTAT.EQ.0) THEN                                        
              DO 1 I=1,IREC-1                                           
                READ(UNIT=IUNIT,FMT='(1X)')                             
    1         CONTINUE                                                  
              WRITE(UNIT=IUNIT,FMT='(A)') ' END OF FILE'                
              CLOSE(UNIT=IUNIT,IOSTAT=IOSTAT)                           
              IF(IOSTAT.NE.0) THEN                                      
                CIUNIT=CFROMI(IUNIT,'(I10)')                            
                CALL TRIM(CIUNIT,IBEG,IEND)                             
                ERROR=' Direct access file close error. Unit = '//      
     '            CIUNIT(IBEG:IEND)                                     
                GOTO 9999                                               
              ENDIF                                                     
            ELSE                                                        
              CIUNIT=CFROMI(IUNIT,'(I10)')                              
              CALL TRIM(CIUNIT,IBEG,IEND)                               
              ERROR=' Direct access file open error. Unit = '//         
     '          CIUNIT(IBEG:IEND)                                       
              GOTO 9999                                                 
            ENDIF                                                       
          ENDIF                                                         
        ELSE                                                            
          CIUNIT=CFROMI(IUNIT,'(I10)')                                  
          CALL TRIM(CIUNIT,IBEG,IEND)                                   
          ERROR=' File close error. Unit = '//CIUNIT(IBEG:IEND)         
          GOTO 9999                                                     
        ENDIF                                                           
      ENDIF                                                             
      ERROR=' '                                                         
C     CALL EXITS('CLOSE')                                               
      RETURN                                                            
 9999 CALL TRIM(ERROR,IBEG,IEND)                                        
      ERROR=ERROR(1:IEND)//' >CLOSE'                                    
      RETURN 1                                                          
      END                                                               

      
      SUBROUTINE FACORR(XD,ZLABEL,NDT,NDMAX,PI)                            
                                                                        
C**** Corrects fibre angle from co-ordinate rig                        
C**** by adjusting for sloping surface.                                
                                                                        
      IMPLICIT REAL (A-H,O-Z)                                         
      REAL XD(4,NDT),LINES(75),D(75,100,5)                   
      CHARACTER ZLABEL(NDMAX)*12,DLABEL*9                                 

      WRITE(3,'('' Correcting Fibre Angles'')')                          
                                                                        
      ND=0                                                              
      NLEVEL=0
                                                                        
C     ** Fill array for new level **                               
                                                                        
 1    L=1                                                               
      M=1                                                               
      ID=0                                                              
 5    ND=ND+1                                                           
 8    IF(((INDEX(ZLABEL(ND),'EPIC').GT.0)
     ,   .AND.(INDEX(ZLABEL(ND),'BASE').EQ.0))                           
     ,   .OR.(INDEX(ZLABEL(ND),'MY').GT.0)                                       
     ,   .OR.((INDEX(ZLABEL(ND),'EN').GT.0)
     ,   .AND.(INDEX(ZLABEL(ND),'VA').EQ.0))) THEN                         
        DLABEL=ZLABEL(ND)(1:9)
        NLEVEL=NLEVEL+1
        NDD=ND-1
        WRITE(3,'(2X,I5,3X,A9)')ND,DLABEL
        GO TO 10                                                       
      ELSE       
        IF(ND.GE.NDT) GO TO 901
        GO TO 5
      ENDIF
 9    ND=ND+1                                                           
 10   M=M+1                                                             
      DO 20 N=1,4                                                       
        D(L,M,N)=XD(N,ND)                                              
 20   CONTINUE                                                          
                                                                        
C *** Test for end of level                                      
                                                                        
      IF(ZLABEL(ND+1)(1:9).NE.DLABEL) THEN                           
        ID=1                                                           
        LISTS=L                                                        
        ND=NDD                                                         
        GO TO 25                                                       
      ENDIF                                                             
                                                                        
C *** Test for end of annulus                                       
                                                                        
      IF(ABS(XD(3,ND)-XD(3,ND+1)).LT.1.0) THEN                       
        GO TO 9                                                           
      ENDIF                                                             
                                                                        
C *** Dummy first and last points                               
                                                                        
 25   DO 30  N=1,4                                                      
        D(L,1,N)=D(L,M,N)                                              
        D(L,M+1,N)=D(L,2,N)                                            
 30   CONTINUE                                                          
      LINES(L)=M                                                        
      IF(ID.EQ.1) GO TO 100                                          
      L=L+1                                                             
      M=1                                                               
      GO TO 9                                                           
                                                                        
C *** CORRECTION                                             
C *** ----------                                                
                                
 100  CONTINUE
      DO 900 L=1,LISTS                                                 
        DO 800 M=2,LINES(L)                                            
          ND=ND+1                                                     
          RC=D(L,M,1)                                                 
          THETA=D(L,M,2)                                              
          AXIS=D(L,M,3)                                               
          OFA=D(L,M,4)                                                
          LFA=OFA
                                                                        
C ***     BETA  CORRECTION  (B)                                    
                                                                        
          IF((LFA.EQ.0).OR.(INDEX(ZLABEL(ND),'NFA').GT.0))GO TO 800
          IF(ABS(OFA).LE.90.0) THEN                                
            MB=M-1                                                   
          ELSE                                                        
            MB=M+1                                                   
          ENDIF                                                                       
          RNB=D(L,MB,1)                                               
          RR =RC/RNB                                                  
C         WRITE(*,*) 'ND= ',ND,(D(L,M,NX),NX=1,4)
          DTHETA=ABS(D(L,M,2)-D(L,MB,2))                              
                                                                       
C ***     ? Past 360 ?  (DTHETA > 200  say)                         
                                                                        
          IF(DTHETA.GT.200.0) DTHETA=360.0-DTHETA                 
                                                                        
C ***     Step  too big ? -No  correction                           
                                                                        
          IF(DTHETA .GT. 60.0) THEN                                
            BETA=0.0                                                 
            GO TO 550                                                
          ENDIF                                                       
                                                                        
C ***     Calculate  BETA                                           
                                                                        
          DTHETR=DTHETA*PI/180.0                                      
          TOP=SIN(DTHETR)                      
          BOT=SQRT(1.0+RR*(RR-2.0*COS(DTHETR)))
	  IF(BOT.EQ.0.0) THEN
            WRITE(*,*) ' Divide by zero. Check data point for'
	    WRITE(*,*) ' duplication or single point in annulus.'
 	    WRITE(*,*) ' at point.     ',ND,(D(L,M,NX),NX=1,4)
	    GOTO 800
	  ENDIF
          TOB=TOP/BOT
	  IF(ABS(TOB).GT.1.0) THEN
	    TOB=1.0*ABS(TOB)/TOB
	  ENDIF
  150     BANGLR=ASIN(TOB)
          BANGLE=BANGLR*180.0/PI                                      
          IF(ABS(OFA).LE.90.0) THEN                              
            IF(RC/COS(DTHETR).GE.RNB) THEN                        
              BETA=90.0-BANGLE                                        
            ELSE                                                      
              BETA=-(90.0-BANGLE)                                     
            ENDIF                                                     
          ELSE                                                        
            IF(RC/COS(DTHETR).GE.RNB) THEN                       
              BETA=-(90.0-BANGLE)                                     
            ELSE                                                      
              BETA=90.0-BANGLE                                        
            ENDIF                                                     
          ENDIF      
                                                                        
C ***      GAMMA  CORRECTION  (G)                                    
                                                                        
C ***     Find Radius for GAMMA  Calculation                        
C ***     1) Special Cases - First and Last Annulae                
                                                                        
 550      IF(L.EQ.1) THEN                                         
            LG=2                                                      
            GO TO 580                                                 
          ENDIF                                                       
          IF(L.EQ.LISTS) THEN                                     
            LG=LISTS-1                                                
            GO TO 580                                                 
          ENDIF                                                       
                                                                        
C ***     2) Middle  Annulae                                       
                                                                        
          OFADUM=(ABS(OFA))/OFA                                       
          IF(OFADUM.LT.0.0) THEN                                  
            LG=L-1                                                    
            ELSE                                                        
            LG=L+1                                                    
          ENDIF                                                       
                                                                        
C ***     Search for THETA                                          
                                                                        
 580      MG=1                                                        
 600      MG=MG+1                                                     
          DUM1=D(LG,MG,2)                                           
          DUM2=D(LG,MG-1,2)                                         
          IF(DUM1.LT.DUM2) THEN                                   
            IF(DUM1.GE.THETA.AND.THETA.GE.0.0) GO TO 650      
            IF(DUM2.LT.THETA.AND.THETA.LE.360.0) GO TO 650    
          ENDIF                                                       
          IF(DUM1.GE.THETA.AND.DUM2.LT.THETA) GO TO 650        
          GO TO 600                                                   
 650      IF(D(LG,MG,2).EQ.THETA) THEN                            
            RNG=D(LG,MG,1)                                            
            GO TO 700                                                 
          ELSE                                                        
                                                                        
C ***       Calculate  New  Radius                                    
                                                                        
            DTG=(D(LG,MG,2)-D(LG,MG-1,2))                             
            IF(DTG/ABS(DTG).LT.0.0) DTG=360.0+DTG                
                                                                        
C ***       Step  too  big?  No  Correction                           
                                                                        
            IF(DTG.GT.60.0) THEN                                  
              GAMMAR=0.0                                             
              BETAR=BETA*PI/180.0                                    
              GO TO 720                                              
            ENDIF                                                     
            RNG1=D(LG,MG,1)                                           
            RNG2=D(LG,MG-1,1)                                         
            RRG=RNG1/RNG2                                             
            DTGR=DTG*PI/180.0                                         
            TOP=SIN(DTGR)                                             
            BOT=SQRT(1.0+RRG*(RRG-2.0*COS(DTGR)))                     
            IF(BOT.EQ.0.0) THEN
              WRITE(*,*) ' Divide by zero. Gamma calculation.'
              WRITE(*,*) ' Check data on adjacent annulus.'
              WRITE(*,*) ' point = ',ND
              GOTO 800
            ENDIF
            TOB=TOP/BOT
            IF(ABS(TOB).GT.1.0) THEN
              TOB=1.0*ABS(TOB)/TOB
            ENDIF
 660        ALPHAR=ASIN(TOB)                                      
            ALPHA=ALPHAR*180.0/PI                                     
            IF(RNG2.GE.RNG1/COS(DTGR)) ALPHA=180.0-ALPHA         
            OMEGA=180.0-(ALPHA-THETA+D(LG,MG,2))                      
            OMEGAR=OMEGA*PI/180.0                                     
            RNG=(RNG1*SIN(ALPHAR))/SIN(OMEGAR)                        
          ENDIF                                          
                                                                        
C ***     Calculate  GAMMA                                          
                                                                        
 700      BETAR=BETA*PI/180.0                                         
          PRC=RC*COS(BETAR)                                           
          PRNG=RNG*COS(BETAR)                                         
          AXISNG=D(LG,MG,3)                                           
          TOP=PRNG-PRC                                                
          BOT=AXISNG-AXIS                                             
          GAMMAR=ATAN(TOP/BOT)   
                                                                      
C ***     FIBRE  ANGLE  CORRECTION                              
                                                                       
 720      OFAR=OFA*PI/180.0                                           
          TOP=(COS(BETAR))*(SIN(OFAR))                               
          BOT=COS(GAMMAR)*COS(OFAR)+SIN(GAMMAR)*SIN(BETAR)*SIN(OFAR)  
          IF(BOT.EQ.0.0) THEN
            D(L,M,5)=90.0                              
          ELSE
            D(L,M,5)=(ATAN (TOP/BOT))*180.0/PI
          ENDIF              
          IF(D(L,M,5).EQ.0.0) THEN
            D(L,M,5)=0.01
            WRITE (*,*) 'NFA=0.0 at point',ND
          ENDIF
          DUM=D(L,M,4)/D(L,M,5)                                       
          IF(DUM.LT.0.0) THEN                                      
            IF(D(L,M,5).LT.0.0) THEN                             
              D(L,M,5)=D(L,M,5)+180.0                               
            ELSE                                                     
              D(L,M,5)=-(180.0-D(L,M,5))                            
            ENDIF                                                    
          ENDIF                                                       
          XD(4,ND)=D(L,M,5)
 800    CONTINUE 
 900  CONTINUE                                                          
      IF(ND.LT.NDT) GO TO 1                                          
 901  CONTINUE 
      WRITE(*,*) ' Fibre angles corrected. '
      WRITE(*,*) ' ND = ',ND , '    Should = NDT = ',NDT
      WRITE(*,*) ' Number of levels used   = ',NLEVEL
      RETURN                                                            
      END                                                               


      SUBROUTINE LEVINPUT(NL,LABLIST,NLL,MLAYER)

C**** Characters separated by commas are read into            
C**** an array MLAYER(). LABLIST contains choice available. 
C**** Called by GEOM

      CHARACTER LABLIST(40)*2,MLAYER(3)*2,CNUM*10,ASN*1

 2    WRITE(*,'(/,''    Enter LV MY layers to be'', 
     ,           '' included in midwall surface'')')
      WRITE(*,'(/,''  Layers available are:'',/)')
      WRITE(*,5) (LABLIST(N),N=1,NL)
      WRITE(*,'(/,'' * Maximum allowed = 3 * '')')
      WRITE(*,'(/,''   Eg. 09,10,11'')')
      READ(*,'(A10)') CNUM
      DO LCNUM=1,10
        IF(CNUM(LCNUM:LCNUM).EQ.' ') GO TO 4
      ENDDO
 4    LCNUM=LCNUM-1
      ICS=1
      NLL=0
      DO IC=1,LCNUM
        IF(CNUM(IC:IC).EQ.',') THEN
          ICE=IC-1
          NLL=NLL+1
          MLAYER(NLL)=CNUM(ICS:ICE)
          ICS=IC+1
        ELSE IF(IC.EQ.LCNUM) THEN
          ICE=IC
          NLL=NLL+1
          MLAYER(NLL)=CNUM(ICS:ICE)
      ENDIF
      ENDDO
      IF(NLL.GT.3) GOTO 2
      WRITE(*,'(/,''  Layers selected are:'')')
      WRITE(*,5) (MLAYER(N),N=1,NLL)
  5   FORMAT((' LV MY L',A2))
      WRITE(*,'(/,''  OK?    Y/N:'')')
      READ(*,'(A1)') ANS
      IF (ANS.EQ.'Y'.OR.ANS.EQ.'y') THEN
        GO TO 10
      ELSE
        GO TO 2
      ENDIF
 10   CONTINUE

      RETURN
      END


      SUBROUTINE MIDLVD(NLVBEG,NLVEND,NDT,XD,ZLABEL,
     ,                  NLL,MLAYER,NZZ,SEPANG,CROSS0)

C**** Decides whether a data point in the LV myocardium
C**** is within one of the layers chosen for the midwall
C**** surface (MLAYER). If so, decides whether the point is outside
C**** the part of that surface which will be modelled using
C**** the RV Septal data. If the point is outside this area
C**** it will be used for this surface and its label is changed
C**** so that it may be identified later. 

      REAL      XD(4,NDT),SEPANG(3,NZZ)
      CHARACTER ZLABEL(NDT)*12,MLAYER(NLL)*2
      LOGICAL   CROSS0(NZZ),CHANGE

      DO N=NLVBEG,NLVEND
        CHANGE=.FALSE.
        DO NL=1,NLL
          IF(ZLABEL(N)(8:9).EQ.MLAYER(NL)) THEN
            NZL=1
            DO NZ=1,NZZ
              DIFF=ABS(XD(3,N)-SEPANG(3,NZ))
              IF(NZ.EQ.1) THEN
                DUMDIF=DIFF
                NZL=1
              ELSE
                IF(DIFF.LT.DUMDIF) THEN
                  DUMDIF=DIFF
                  NZL=NZ
                ENDIF
              ENDIF
            ENDDO
            IF(DUMDIF.GT.2.6) THEN
              CHANGE=.TRUE.
            ELSE
              IF(CROSS0(NZL)) THEN
                IF(XD(2,N).GT.SEPANG(2,NZL).AND.
     ,                  XD(2,N).LT.SEPANG(1,NZL)) THEN
                  CHANGE=.TRUE.
                ENDIF
              ELSE
                IF(XD(2,N).GT.SEPANG(2,NZL).OR.
     ,              XD(2,N).LT.SEPANG(1,NZL)) THEN
                  CHANGE=.TRUE.
                ENDIF
              ENDIF
            ENDIF
          ENDIF
        ENDDO
        IF(CHANGE) THEN
          IF(ZLABEL(N)(10:10).NE.'F') THEN
            ZLABEL(N)='LV MIDWALL  '
          ENDIF
        ENDIF
      ENDDO
      RETURN
      END

                                                                        
      SUBROUTINE OPEN(IUNIT,DEVICE,FILE,STATUS,ACCESS,IRECL,ERROR,*)    
                                                                        
C**** Opens a file using the CMS FILEDEF command.                       
C**** IUNIT is the name by which the file is referred to in the program.
C**** Valid unit identifiers are non negative integers.                 
C**** DEVICE is the device on which the file resides.                   
C**** Valid devices are: 'DISK' 'TERM'                                  
C**** FILE is the filename [filetype [filemode]]                        
C**** STATUS is 'NEW' or 'OLD'                                          
C**** ACCESS specifies whether the file is 'DIRECT' or 'SEQUEN'         
C**** Direct access files have a length of 2000 records.                
C**** IRECL is the logical record length                                
C**** ERROR gives diagnostics in the event of failure.                  
C**** If an error is detected control is returned to the statement      
C**** number of the star.                                               
C**** DEVICE, FILE, ACCESS and ERROR are all character strings.         
                                                                        
      CHARACTER DEVICE*(*),FILE*(*),STATUS*(*),ACCESS*(*),ERROR*(*),    
     '  CTRIM*500,CFROMI*500,UNIT*4,RECL*5,STRING*128,DUMMY*132,        
     '  RECFM*1,XTENT*4,FORM*9                                          
      DATA RECFM/'F'/,XTENT/'2000'/,FORM/'FORMATTED'/                   

C     CALL ENTERS('OPEN')                                               
      UNIT=CFROMI(IUNIT,'(I4)')                                         
      RECL=CFROMI(IRECL,'(I5)')                                         
      CALL CLOSE(IUNIT,ERROR,*9999)                                     
C     STRING='FILEDEF '//UNIT//' CLEAR'                                 
C     CALL CMS(STRING,IERROR)                                           
C     IF(IERROR.NE.0) THEN                                              
C       ERROR=' Error occurred in CMS('//                               
C    '    CTRIM(STRING,IBEG,IEND)(IBEG:IEND)//')'                       
C       GOTO 9999                                                       
C     ENDIF                                                             
      IF(DEVICE.EQ.'TERM') THEN                                         
C       STRING='FILEDEF '//UNIT//' TERMINAL '//                         
C    '    '(RECFM '//RECFM//' LRECL '//RECL//                           
C    '    ' LOWCASE)%'                                                  
C       CALL CMS(STRING,IERROR)                                         
C       IF(IERROR.NE.0) THEN                                            
C         ERROR=' Error occurred in CMS('//                             
C    '      CTRIM(STRING,IBEG,IEND)(IBEG:IEND)//')'                     
C         GOTO 9999                                                     
C       ENDIF                                                           
        OPEN(UNIT=IUNIT,FILE=FILE,STATUS=STATUS,RECL=IRECL,             
     '    IOSTAT=IOSTAT)                                                
        IF(IOSTAT.NE.0) THEN                                            
          ERROR=CFROMI(IOSTAT,'(I3)')                                   
          ERROR=' Iostat='//ERROR(1:3)//                                
     '          ' error occurred in OPEN(UNIT='//UNIT//')'              
          GOTO 9999                                                     
        ENDIF                                                           
      ELSE IF(DEVICE.EQ.'DISK') THEN                                    
        IF(ACCESS.EQ.'SEQUEN') THEN                                     
C         STRING='FILEDEF '//UNIT//' DISK '//FILE//                     
C    '      '(RECFM '//RECFM//' LRECL '//RECL//                         
C    '      ' XTENT '//XTENT//' DSORG PS)%'                             
C         CALL CMS(STRING,IERROR)                                       
C         IF(IERROR.NE.0) THEN                                          
C           ERROR=' Error occurred in CMS('//                           
C    '        CTRIM(STRING,IBEG,IEND)(IBEG:IEND)//')'                   
C           GOTO 9999                                                   
C         ENDIF                                                         
          OPEN(UNIT=IUNIT,FILE=FILE,STATUS=STATUS,                      
     '         ACCESS='SEQUENTIAL',FORM=FORM,IOSTAT=IOSTAT)             
          IF(IOSTAT.NE.0) THEN                                          
            ERROR=CFROMI(IOSTAT,'(I3)')                                 
            ERROR=' Iostat='//ERROR(1:3)//                              
     '            ' error occurred in OPEN(UNIT='//UNIT//')'            
            GOTO 9999                                                   
          ENDIF                                                         
        ELSE IF(ACCESS.EQ.'DIRECT') THEN                                
C         STRING='FILEDEF '//UNIT//' DISK '//FILE//                     
C    '      '(RECFM '//RECFM//' LRECL '//RECL//                         
C    '      ' XTENT '//XTENT//' DSORG DA)%'                             
C         CALL CMS(STRING,IERROR)                                       
C         IF(IERROR.NE.0) THEN                                          
C           ERROR=' Error occurred in CMS('//                           
C    '        CTRIM(STRING,IBEG,IEND)(IBEG:IEND)//')'                   
C           GOTO 9999                                                   
C         ENDIF                                                         
          IF(STATUS.EQ.'OLD') THEN                                      
            OPEN(UNIT=99,FILE=FILE,STATUS='OLD',ACCESS='SEQUENTIAL',    
     '        FORM=FORM,RECL=IRECL,IOSTAT=IOSTAT,RECORDTYPE='VARIABLE') 
            IF(IOSTAT.EQ.0) THEN                                        
              OPEN(UNIT=IUNIT,FILE=FILE,STATUS='NEW',ACCESS='DIRECT',   
     '          FORM=FORM,RECL=IRECL,IOSTAT=IOSTAT,RECORDTYPE='FIXED')  
 10           READ(99,'(A)',IOSTAT=IOSTAT) DUMMY                        
              IF(IOSTAT.EQ.0) THEN                                      
                INQUIRE(UNIT=IUNIT,NEXTREC=IREC)                        
                WRITE(IUNIT,'(A)',REC=IREC,IOSTAT=IOSTAT) DUMMY         
                GO TO 10                                                
              ELSE                                                      
                CLOSE(UNIT=99)                                          
                CLOSE(UNIT=IUNIT)                                       
                OPEN(UNIT=IUNIT,FILE=FILE,STATUS='OLD',ACCESS='DIRECT', 
     '            FORM=FORM,RECL=IRECL,IOSTAT=IOSTAT,RECORDTYPE='FIXED')
C               WRITE(*,'('' Sequential,variable record-length file'',  
C    '            '' copied to direct,fixed record file'')')            
              ENDIF                                                     
            ELSE IF(IOSTAT.EQ.44) THEN                                  
             OPEN(UNIT=IUNIT,FILE=FILE,STATUS='OLD',ACCESS='DIRECT',    
     '          FORM=FORM,RECL=IRECL,IOSTAT=IOSTAT,RECORDTYPE='FIXED')  
C            WRITE(*,'('' Sequential,fixed record-length file'',        
C    '         '' re-opened as direct,fixed record file'')')            
            ENDIF                                                       
          ELSE IF(STATUS.EQ.'NEW') THEN                                 
            OPEN(UNIT=IUNIT,FILE=FILE,STATUS='NEW',ACCESS='DIRECT',     
     '        FORM=FORM,RECL=IRECL,IOSTAT=IOSTAT,RECORDTYPE='FIXED')    
          ENDIF                                                         
C         IF(IOSTAT.NE.0) THEN                                          
C           ERROR=CFROMI(IOSTAT,'(I3)')                                 
C           ERROR=' Iostat='//ERROR(1:3)//                              
C    '            ' error occurred in OPEN(UNIT='//UNIT//')'            
C           GOTO 9999                                                   
C         ENDIF                                                         
        ELSE                                                            
          ERROR=' ACCESS='//ACCESS//' is invalid'                       
          GOTO 9999                                                     
        ENDIF                                                           
      ELSE                                                              
        ERROR=' DEVICE='//DEVICE//' is invalid'                         
        GOTO 9999                                                       
      ENDIF                                                             
C     IF(IERROR.NE.0) THEN                                              
C       ERROR=' Error occurred in CMS('//                               
C    '    CTRIM(STRING,IBEG,IEND)(IBEG:IEND)//')'                       
C       GOTO 9999                                                       
C     ENDIF                                                             
      ERROR=' '                                                         
C     CALL EXITS('OPEN')                                                
      RETURN                                                            
 9999 CALL TRIM(ERROR,IBEG,IEND)                                        
      ERROR=ERROR(1:IEND)//' >OPEN'                                     
      RETURN 1                                                          
      END                                                               
                                                                  

      SUBROUTINE PELEC(ZD,ZPE,ZLABEL,ELABEL,NZPE,NPEP,NDT)                
                                                                        
C****  Calculates coordinates of stimulus sites                         
C****  from plunge electrode data.                                      
	                                                                        
      IMPLICIT REAL (A-H,O-Z)
      REAL ZD(4,NDT),ZPE(3,NPEP)
      INTEGER NZPE(NPEP)
      CHARACTER ZLABEL(NDT)*12,ELABEL(NPEP)*12,ELNUM*1,CDUMML*3
     '         , CFROMI*100
               
C     WRITE(*,*) ' Plunge Electrodes. '

      NPE=NPEP/6
      I=1

      DO 10 N=1,NPE

        K=NZPE(I)

        YO=ZPE(1,I)
        XO=ZPE(2,I)
        ZO=ZPE(3,I)

        I=I+3

        YEP=ZPE(1,I)
        XEP=ZPE(2,I)
        ZEP=ZPE(3,I)
               
        CDUMML=ELABEL(I)(7:9)
        IDUMML=IFROMC(CDUMML)

        DELL=0.1*FLOAT(IDUMML)  
          
        GRADY=(YO-YEP)/(XO-XEP)
        GRADZ=(ZO-ZEP)/(XO-XEP)
        DIRNX=(XEP-XO)/ABS(XEP-XO)

        DELX=(DELL-0.5)*DIRNX/SQRT(1+GRADY**2+GRADZ**2)
        DELY=GRADY*DELX
        DELZ=GRADZ*DELX

        DY=DELY*1.0/DELL
        DX=DELX*1.0/DELL
        DZ=DELZ*1.0/DELL

        DO 5 J=1,5
                                           
          ZD(1,K)=YEP+DELY-(DY*FLOAT(J-1))
          ZD(2,K)=XEP+DELX-(DX*FLOAT(J-1))
          ZD(3,K)=ZEP+DELZ-(DZ*FLOAT(J-1))

          ELNUM=CFROMI(J,'(I1)')
          ZLABEL(K)=ELABEL(I)(1:5)//'   '//ELNUM

          K=K+1

    5   CONTINUE

        ZD(1,K)=YEP
        ZD(2,K)=XEP
        ZD(3,K)=ZEP
        ZLABEL(K)=ELABEL(I)(1:5)//'  SP'

        I=I+3

   10 CONTINUE

      RETURN
      END
                                                                        
                  
      SUBROUTINE RESET(TRANS)                                           
                                                                        
C**** RESET resets the transformation matrix TRANS to the identity      
C**** matrix.                                                           
                                                                        
      IMPLICIT REAL (A-H,O-Z)                                         
      REAL TRANS(3,*)                                                 
                                                                        
C     CALL ENTERS('RESET')                                              
      DO 2 I=1,3                                                        
        DO 1 J=1,4                                                      
          TRANS(I,J)=0.0                                              
    1   CONTINUE                                                        
        TRANS(I,I)=1.0                                                
    2 CONTINUE                                                          
                                                                        
C     CALL EXITS('RESET')                                               
      RETURN                                                            
      END                                                               


      SUBROUTINE RSWANG(NDT,N,XD,SEPANG,TH1LOW,NZZ)
               
C**** Calculates the extent of RV septal endocardium:
C**** Sepang(1,nzz) to Sepang(2,nzz) at Z=sepang(3,nzz),
C**** and whether angle is increasing or decreasing from
C**** Sepang (1,nzz): Th1low(nzz) = T or F.
C**** Called from GEOM.

 
      REAL    SEPANG(3,50),XD(4,NDT)
      LOGICAL TH1LOW(50)

      IF(ABS(XD(3,N)-XD(3,N-1)).GT.1.0) THEN
        IF(ABS(XD(3,N)-XD(3,N+1)).LT.1.0)THEN
          NZZ=NZZ+1
          TH1LOW(NZZ)=.TRUE. !Assume true, change if not.
          SEPANG(1,NZZ)=XD(2,N) ! Theta-1
          SEPANG(3,NZZ)=XD(3,N) ! Z
          DIFF=XD(3,N+1)-XD(3,N)
          IF(ABS(DIFF).GT.200.0) THEN
            IF(DIFF.GT.0.0) THEN
              TH1LOW(NZZ)=.FALSE.
            ENDIF
          ELSE
            IF(DIFF.LT.0.0) THEN
              TH1LOW(NZZ)=.FALSE.
            ENDIF
          ENDIF
        ENDIF
      ENDIF
      IF(NZZ.GT.0) THEN
        SEPANG(2,NZZ)=XD(2,N)
      ENDIF

      RETURN
      END


      SUBROUTINE SEPSORT(NZZ,TH1LOW,SEPANG,CROSS0)
                                                                        
C *** Alter SEPANG so that RVS angle is increasing from SEPANG(1,n)
C *** Decide whether RVS area crosses theta=0 at each Z level
C *** (for unrotated coordinates)

      REAL    SEPANG(3,NZZ)
      LOGICAL TH1LOW(NZZ),CROSS0(NZZ)

      DO NZ=1,NZZ
        IF(.NOT.TH1LOW(NZ)) THEN
          SADUM=SEPANG(1,NZ)
          SEPANG(1,NZ)=SEPANG(2,NZ)
          SEPANG(2,NZ)=SADUM
        ENDIF
        CROSS0(NZ)=.FALSE.   
        IF((SEPANG(2,NZ)-SEPANG(1,NZ)).LT.0.0) THEN
          CROSS0(NZ)=.TRUE.
        ENDIF
      ENDDO

      RETURN
      END

                                                                        
      SUBROUTINE SHIFT(AXIS,AMOUNT,TRANS)                               
                                                                        
C**** SHIFT updates the transformation matrix TRANS with a shifting     
C**** by AMOUNT along the axis defined by AXIS.                         
                                                                        
      IMPLICIT REAL(A-H,O-Z)                                          
      REAL TRANS(3,*),AXIS(*)                                         
                                                                        
C     CALL ENTERS('SHIFT')                                              
      IF(AXIS(1).NE.0.0) THEN                                         
        IF(AXIS(2).NE.0.0) THEN                                       
          IF(AXIS(3).NE.0.0) THEN                                     
            AMOUN0=AMOUNT/SQRT(AXIS(1)**2+AXIS(2)**2+AXIS(3)**2)        
            CALL SHIFT1(AMOUN0*AXIS(1),TRANS)                           
            CALL SHIFT2(AMOUN0*AXIS(2),TRANS)                           
            CALL SHIFT3(AMOUN0*AXIS(3),TRANS)                           
          ELSE                                                          
            AMOUN0=AMOUNT/SQRT(AXIS(1)**2+AXIS(2)**2)                   
            CALL SHIFT1(AMOUN0*AXIS(1),TRANS)                           
            CALL SHIFT2(AMOUN0*AXIS(2),TRANS)                           
          ENDIF                                                         
        ELSE                                                            
          IF(AXIS(3).NE.0.0) THEN                                     
            AMOUN0=AMOUNT/SQRT(AXIS(1)**2+AXIS(3)**2)                   
            CALL SHIFT1(AMOUN0*AXIS(1),TRANS)                           
            CALL SHIFT3(AMOUN0*AXIS(3),TRANS)                           
          ELSE                                                          
            CALL SHIFT1( AMOUNT,TRANS)                                  
          ENDIF                                                         
        ENDIF                                                           
      ELSE                                                              
        IF(AXIS(2).NE.0.0) THEN                                       
          IF(AXIS(3).NE.0.0) THEN                                     
            AMOUN0=AMOUNT/SQRT(AXIS(2)**2+AXIS(3)**2)                   
            CALL SHIFT2(AMOUN0*AXIS(2),TRANS)                           
            CALL SHIFT3(AMOUN0*AXIS(3),TRANS)                           
          ELSE                                                          
            CALL SHIFT2(AMOUNT,TRANS)                                   
          ENDIF                                                         
        ELSE                                                            
          IF(AXIS(3).NE.0.0) THEN                                     
            CALL SHIFT3(AMOUNT,TRANS)                                   
          ELSE                                                          
C****       The axis of shift is undefined                              
          ENDIF                                                         
        ENDIF                                                           
      ENDIF                                                             
                                                                        
C     CALL EXITS('SHIFT')                                               
      RETURN                                                            
      END                                                               
                                                                        
                                                                        
      SUBROUTINE SHIFT1(AMOUNT,TRANS)                                   
                                                                        
C**** SHIFT1 updates the transformation matrix TRANS with a shifting    
C**** by AMOUNT along the Z(1) axis.                                    
                                                                        
      IMPLICIT REAL(A-H,O-Z)                                          
      REAL TRANS(3,*)                                                 
                                                                        
C     CALL ENTERS('SHIFT1')                                             
      IF(AMOUNT.NE.0.0) THEN                                          
        TRANS(1,4)=TRANS(1,4)+AMOUNT                                    
      ENDIF                                                             
                                                                        
C     CALL EXITS('SHIFT1')                                              
      RETURN                                                            
      END                                                               
                                                                        
                                                                        
      SUBROUTINE SHIFT2(AMOUNT,TRANS)                                   
                                                                        
C**** SHIFT2 updates the transformation matrix TRANS with a shifting    
C**** by AMOUNT along the Z(2) axis.                                    
                                                                        
      IMPLICIT REAL(A-H,O-Z)                                          
      REAL TRANS(3,*)                                                 
                                                                        
C     CALL ENTERS('SHIFT2')                                             
      IF(AMOUNT.NE.0.0) THEN                                          
        TRANS(2,4)=TRANS(2,4)+AMOUNT                                    
      ENDIF                                                             
                                                                        
C     CALL EXITS('SHIFT2')                                              
      RETURN                                                            
      END                                                               
                                                                        
                                                                        
      SUBROUTINE SHIFT3(AMOUNT,TRANS)                                   
                                                                        
C**** SHIFT3 updates the transformation matrix TRANS with a shifting    
C**** by AMOUNT along the Z(3) axis.                                    
                                                                        
      IMPLICIT REAL(A-H,O-Z)                                          
      REAL TRANS(3,*)                                                 
                                                                        
C     CALL ENTERS('SHIFT3')                                             
      IF(AMOUNT.NE.0.0) THEN                                          
        TRANS(3,4)=TRANS(3,4)+AMOUNT                                    
      ENDIF                                                             
                                                                        
C     CALL EXITS('SHIFT3')                                              
      RETURN                                                            
      END                                                               
                                                                        
                                                                        
      SUBROUTINE TRIM(CDATA,IBEGIN,IEND)                                
                                                                        
C**** Finds the first and last non-blank characters in a string CDATA.  
C**** On exit IBEGIN is the position of the first non-blank character   
C**** and IEND is the position of the last non-blank character.         
C**** If CDATA consists of only blank characters IEND will be 1 and     
C**** IBEGIN will be LEN(CDATA).                                        
                                                                        
      CHARACTER CDATA*(*)                                               
                                                                        
      LENGTH=LEN(CDATA)                                                 
      DO 1 L=1,LENGTH                                                   
        IF(CDATA(L:L).NE.' ') THEN                                      
          IBEGIN=L                                                      
          GOTO 2                                                        
        ENDIF                                                           
    1 CONTINUE                                                          
      IBEGIN=LENGTH                                                     
    2 DO 3 L=LENGTH,1,-1                                                
        IF(CDATA(L:L).NE.' ') THEN                                      
          IEND=L                                                        
          GOTO 4                                                        
        ENDIF                                                           
    3 CONTINUE                                                          
      IEND=1                                                            
    4 IF(IBEGIN.GT.IEND) THEN                                           
        IBEGIN=1                                                        
        IEND=1                                                          
      ENDIF                                                             
      RETURN                                                            
      END                                                               

                                                                        
      SUBROUTINE TWIST(AXIS,AMOUNT,TRANS)                               
                                                                        
C**** TWIST updates the transformation matrix TRANS with a rotation     
C**** of AMOUNT radians about the axis defined by AXIS.                 
                                                                        
      IMPLICIT REAL(A-H,O-Z)                                          
      REAL TRANS(3,*),AXIS(*)                                         
                                                                        
C     CALL ENTERS('TWIST')                                              
      IF(AXIS(1).NE.0.0) THEN                                         
        IF(AXIS(2).NE.0.0) THEN                                       
          IF(AXIS(3).NE.0.0) THEN                                     
            AMOUN1=ATAN2(AXIS(2),AXIS(3))                               
            AMOUN2=ATAN2(SQRT(AXIS(2)**2+AXIS(3)**2),AXIS(1))           
            CALL TWIST1( AMOUN1,TRANS)                                  
            CALL TWIST2(-AMOUN2,TRANS)                                  
            CALL TWIST3( AMOUNT,TRANS)                                  
            CALL TWIST2( AMOUN2,TRANS)                                  
            CALL TWIST1(-AMOUN1,TRANS)                                  
          ELSE                                                          
            AMOUN3=ATAN2(AXIS(2),AXIS(1))                               
            CALL TWIST3(-AMOUN3,TRANS)                                  
            CALL TWIST1( AMOUNT,TRANS)                                  
            CALL TWIST3( AMOUN3,TRANS)                                  
          ENDIF                                                         
        ELSE                                                            
          IF(AXIS(3).NE.0.0) THEN                                     
            AMOUN2=ATAN2(AXIS(1),AXIS(3))                               
            CALL TWIST2(-AMOUN2,TRANS)                                  
            CALL TWIST3( AMOUNT,TRANS)                                  
            CALL TWIST2( AMOUN2,TRANS)                                  
          ELSE                                                          
            CALL TWIST1( AMOUNT,TRANS)                                  
          ENDIF                                                         
        ENDIF                                                           
      ELSE                                                              
        IF(AXIS(2).NE.0.0) THEN                                       
          IF(AXIS(3).NE.0.0) THEN                                     
            AMOUN1=ATAN2(AXIS(3),AXIS(2))                               
            CALL TWIST1( AMOUN1,TRANS)                                  
            CALL TWIST2( AMOUNT,TRANS)                                  
            CALL TWIST1(-AMOUN1,TRANS)                                  
          ELSE                                                          
            CALL TWIST2( AMOUNT,TRANS)                                  
          ENDIF                                                         
        ELSE                                                            
          IF(AXIS(3).NE.0.0) THEN                                     
            CALL TWIST3( AMOUNT,TRANS)                                  
          ELSE                                                          
C****       The axis of rotation is undefined                           
          ENDIF                                                         
        ENDIF                                                           
      ENDIF                                                             
                                                                        
C     CALL EXITS('TWIST')                                               
      RETURN                                                            
      END                                                               
                                                                        
                                                                  
      SUBROUTINE TWIST1(AMOUNT,TRANS)                                   
                                                                        
C**** TWIST1 updates the transformation matrix TRANS with a rotation    
C**** of AMOUNT radians about the Z(1) axis.                            
                                                                        
      IMPLICIT REAL(A-H,O-Z)                                          
      REAL TRANS(3,*)                                                 
                                                                        
C     CALL ENTERS('TWIST1')                                             
      IF(AMOUNT.NE.0.0) THEN                                          
        SINA=SIN(AMOUNT)                                                
        COSA=COS(AMOUNT)                                                
        DO 1 N=1,4                                                      
          TRANS2=TRANS(2,N)                                             
          TRANS3=TRANS(3,N)                                             
          TRANS(2,N)=TRANS2*COSA-TRANS3*SINA                            
          TRANS(3,N)=TRANS2*SINA+TRANS3*COSA                            
    1   CONTINUE                                                        
      ENDIF                                                             
                                                                        
C     CALL EXITS('TWIST1')                                              
      RETURN                                                            
      END                                                               
                                                                        
                                                                        
      SUBROUTINE TWIST2(AMOUNT,TRANS)                                   
                                                                        
C**** TWIST2 updates the transformation matrix TRANS with a rotation    
C**** of AMOUNT radians about the Z(2) axis.                            
                                                                        
      IMPLICIT REAL(A-H,O-Z)                                          
      REAL TRANS(3,*)                                                 
                                                                        
C     CALL ENTERS('TWIST2')                                             
      IF(AMOUNT.NE.0.0) THEN                                          
        SINA=SIN(AMOUNT)                                                
        COSA=COS(AMOUNT)                                                
        DO 1 N=1,4                                                      
          TRANS3=TRANS(3,N)                                             
          TRANS1=TRANS(1,N)                                             
          TRANS(3,N)=TRANS3*COSA-TRANS1*SINA                            
          TRANS(1,N)=TRANS3*SINA+TRANS1*COSA                            
    1   CONTINUE                                                        
      ENDIF                                                             
                                                                        
C     CALL EXITS('TWIST2')                                              
      RETURN                                                            
      END                                                               
                                                                        
                                                                        
      SUBROUTINE TWIST3(AMOUNT,TRANS)                                   
                                                                        
C**** TWIST3 updates the transformation matrix TRANS with a rotation    
C**** of AMOUNT radians about the Z(3) axis.                            
                                                                        
      IMPLICIT REAL(A-H,O-Z)                                          
      REAL TRANS(3,*)                                                 
                                                                        
C     CALL ENTERS('TWIST3')                                             
      IF(AMOUNT.NE.0.0) THEN                                          
        SINA=SIN(AMOUNT)                                                
        COSA=COS(AMOUNT)                                                
        DO 1 N=1,4                                                      
          TRANS1=TRANS(1,N)                                             
          TRANS2=TRANS(2,N)                                             
          TRANS(1,N)=TRANS1*COSA-TRANS2*SINA                            
          TRANS(2,N)=TRANS1*SINA+TRANS2*COSA                            
    1   CONTINUE                                                        
      ENDIF                                                             
                                                                        
C     CALL EXITS('TWIST3')                                              
      RETURN                                                            
      END                                                               
                                                                        
      SUBROUTINE ZZ(Z1,Z2,TRANS)                                        
                                                                        
C**** Transforms rectangular cartesian coordinates Z1 to Z2 using the   
C**** transformation matrix TRANS.                                      
                                                                        
      IMPLICIT REAL (A-H,O-Z)                                         
      REAL Z1(*),Z2(*),TRANS(3,*),Z(3)                                
                                                                        
C     CALL ENTERS('ZZ')                                                 
      Z(1)=Z1(1)                                                        
      Z(2)=Z1(2)                                                        
      Z(3)=Z1(3)                                                        
      DO 2 I2=1,3                                                       
        Z2(I2)=0.0                                                    
        DO 1 I1=1,3                                                     
          Z2(I2)=Z2(I2)+Z(I1)*TRANS(I2,I1)                              
 1      CONTINUE                                                        
        Z2(I2)=Z2(I2)+TRANS(I2,4)                                       
 2    CONTINUE                                                          
                                                                        
C     CALL EXITS('ZZ')                                                  
      RETURN                                                            
      END                                                               
                                                                        

------------X-UIDL: I@l!!Aap"!c:>"!?Ce"!
Received: from mailhost.auckland.ac.nz (mailhost [130.216.1.4]) by esu1.auckland.ac.nz (980427.SGI.8.8.8/980728.SGI.AUTOCF) via ESMTP id OAA88224 for <stevens@esu1.auckland.ac.nz>; Wed, 7 Mar 2001 14:57:44 +1300 (NZDT)
Received: from auckland.ac.nz (i.legrice4.mph.auckland.ac.nz [130.216.22.112])
	by mailhost.auckland.ac.nz (8.9.2/8.9.2/8.9.2-ua) with ESMTP id OAA06274
	for <c.stevens@auckland.ac.nz>; Wed, 7 Mar 2001 14:57:43 +1300 (NZDT)
Message-ID: <3AA59699.E8AE9D29@auckland.ac.nz>
Date: Wed, 07 Mar 2001 15:02:02 +1300
From: Ian LeGrice <i.legrice@auckland.ac.nz>
Organization: University of Auckland
X-Mailer: Mozilla 4.73 (Macintosh; I; PPC)
X-Accept-Language: en
MIME-Version: 1.0
To: Carey Stevens <c.stevens@auckland.ac.nz>
Subject: heart
Content-Type: multipart/mixed;
 boundary="------------A7FC839A17B4C5BBA390D582"

This is a multi-part message in MIME format.