package MySubs ;
require 5.6.0;


=head1 MyUtils

Plotting functions 

=cut

BEGIN {
}


sub Define_Calcium_Level {
    # A function to define the current calcium activation factor - done
    # by writing out a temporary IPACTI file with the given level of
    # activation and then reading it in.
    
    # Parameters
    ($actn) = @_;

    # Create the IPACTI file
    CORE::open(IPACTI,">tmp_ipacti_file.ipacti") || die "Cannot open temporary ipacti file: $!\n";
    print IPACTI " CMISS Version 1.21 ipacti File Version 2\n";
    print IPACTI " Heading: Temp. ipacti file for activation level of $actn\n";
    print IPACTI " \n";
    print IPACTI " Specify type of contraction model [2]:\n";
    print IPACTI "   (1) SS tension-length-Ca relation (OLD)\n";
    print IPACTI "   (2) Hunter/McCulloch/ter Keurs (coupled to cell/grid model)\n";
    print IPACTI "   (3) Unused\n";
    print IPACTI "   (4) Outer hair cell stiffness\n";
    print IPACTI "    1\n";
    print IPACTI " Enter max isometric tension at ext.ratio=1 (Tref) [100kPa]: 0.10000D+03\n";
    print IPACTI " Enter non-dimensional slope parameter (beta) [1.45]: 0.14500D+01\n";
    print IPACTI " Enter c50 for [Ca]i saturation curve (0<c<1) [0.5]: 0.50000D+00\n";
    print IPACTI " Enter Hill coeff. for [Ca]i saturation curve (h) [3.0]: 0.30000D+01\n";
    print IPACTI " Enter max [Ca]i (Ca_max) [1]: 0.10000D+01\n";
    print IPACTI " Enter initial calcium level [Ca]i [0]: $actn\n";
    CORE::close IPACTI;
    # Define the current level of activation
    cmiss::cmiss(qq/fem define active;r;tmp_ipacti_file;/);
    # And delete the temporary file
    unlink "tmp_ipacti_file.ipacti";
    
} # sub DefineCalciumLevel






sub Create_File_strain_xi{
		my $strain = shift;
		my $OUTPUT_FILE = shift;
		my $NE = shift;
		my $XI1 = shift;
		my $XI2 = shift;

    cmiss::cmiss(qq/fem li strain;${OUTPUT_FILE}_tmp xi 3 xi_1 ${XI1} xi_2 ${XI2} points 9 element ${NE} $strain/);
    CORE::open(INPUTFILE,"<${OUTPUT_FILE}_tmp.opstra") or die("ERROR: Unable to open input file");
    CORE::open(OUTPUTFILE,">${OUTPUT_FILE}.dat") or die("ERROR: Unable to open output file");
    print(OUTPUTFILE "%depth exrFibre 11 exrSheet 12 22 exrSheetNormal 13 23 33   element $NE xi1 $XI1 xi2 $XI2\n");
    $in=0;
    while (<INPUTFILE>) 
    {
        @IPfileLine = split;
        $IPfileLine[3] =~ s/D/E/g;
        if ($IPfileLine[0] eq "Xi(i):") 
      	{
      			$depth = $IPfileLine[3]*-100+100;
  	    		print(OUTPUTFILE "$depth ");
  	    }
        if ($IPfileLine[0] eq "EG(1,j):") 
      	{
  	    		print(OUTPUTFILE "@IPfileLine[3] @IPfileLine[1] ");
            $in=1;
  	    } 
        if (($IPfileLine[0] eq "2") && ($in == 1)) 
      	{
  	    		print(OUTPUTFILE "@IPfileLine[4] @IPfileLine[1..2] ");
  	    } 
        if (($IPfileLine[0] eq "3") && ($in == 1) ) 
      	{
  	    		print(OUTPUTFILE "@IPfileLine[5] @IPfileLine[1..3] \n");
            $in=0;
  	    } 
    }
    CORE::close(INPUTFILE);
    CORE::close(OUTPUTFILE);
#    unlink "${OUTPUT_FILE}_tmp.opstra";
}


sub Create_File_stress_xi{
		my $strain = shift;
		my $OUTPUT_FILE = shift;
		my $NE = shift;
		my $XI1 = shift;
		my $XI2 = shift;

    cmiss::cmiss(qq/fem li stress;${OUTPUT_FILE}_tmp xi 3 xi_1 ${XI1} xi_2 ${XI2} points 9 element ${NE} $strain/);
    CORE::open(INPUTFILE,"<${OUTPUT_FILE}_tmp.opstre") or die("ERROR: Unable to open input file");
    CORE::open(OUTPUTFILE,">${OUTPUT_FILE}.dat") or die("ERROR: Unable to open output file");
    print(OUTPUTFILE "%depth 11 12 22 13 23 33   element $NE xi1 $XI1 xi2 $XI2\n");
    $in=0;
    while (<INPUTFILE>) 
    {
        @IPfileLine = split;
        $IPfileLine[3] =~ s/D/E/g;
        if ($IPfileLine[0] eq "Xi(i):") 
      	{
      			$depth = $IPfileLine[3]*-100+100;
  	    		print(OUTPUTFILE "$depth ");
  	    }
        if ($IPfileLine[0] eq "TG(1,i):") 
      	{
  	    		print(OUTPUTFILE "@IPfileLine[1] ");
            $in=1;
  	    } 
        if (($IPfileLine[0] eq "2") && ($in == 1)) 
      	{
  	    		print(OUTPUTFILE "@IPfileLine[1..2] ");
  	    } 
        if (($IPfileLine[0] eq "3") && ($in == 1) ) 
      	{
  	    		print(OUTPUTFILE "@IPfileLine[1..3] \n");
            $in=0;
  	    } 
    }
    CORE::close(INPUTFILE);
    CORE::close(OUTPUTFILE);
#    unlink "${OUTPUT_FILE}_tmp.opstre";
}


sub Create_File_strain_gauss{
		my $strain = shift;
		my $OUTPUT_FILE = shift;
		my $NE = shift;

    cmiss::cmiss(qq/fem li strain;${OUTPUT_FILE}_tmp gauss element ${NE} $strain/);
    CORE::open(INPUTFILE,"<${OUTPUT_FILE}_tmp.opstra") or die("ERROR: Unable to open input file");
    CORE::open(OUTPUTFILE,">${OUTPUT_FILE}.dat") or die("ERROR: Unable to open output file");
    print(OUTPUTFILE "%depth exrFibre 11 exrSheet 12 22 exrSheetNormal 13 23 33   element $NE \n");
    $in=0;
    while (<INPUTFILE>) 
    {
        @IPfileLine = split;
        $IPfileLine[3] =~ s/D/E/g;
        if ($IPfileLine[0] eq "Xi(i):") 
      	{
      			$depth = $IPfileLine[3]*-100+100;
  	    		print(OUTPUTFILE "$depth ");
  	    }
        if ($IPfileLine[0] eq "EG(1,j):") 
      	{
  	    		print(OUTPUTFILE "@IPfileLine[3] @IPfileLine[1] ");
            $in=1;
  	    } 
        if (($IPfileLine[0] eq "2") && ($in == 1)) 
      	{
  	    		print(OUTPUTFILE "@IPfileLine[4] @IPfileLine[1..2] ");
  	    } 
        if (($IPfileLine[0] eq "3") && ($in == 1) ) 
      	{
  	    		print(OUTPUTFILE "@IPfileLine[5] @IPfileLine[1..3] \n");
            $in=0;
  	    } 
    }
    CORE::close(INPUTFILE);
    CORE::close(OUTPUTFILE);
#    unlink "${OUTPUT_FILE}_tmp.opstra";
}

sub Create_File_stress_gauss{
		my $strain = shift;
		my $OUTPUT_FILE = shift;
		my $NE = shift;

    cmiss::cmiss(qq/fem li stress;${OUTPUT_FILE}_tmp element ${NE} $strain/);
    CORE::open(INPUTFILE,"<${OUTPUT_FILE}_tmp.opstre") or die("ERROR: Unable to open input file");
    CORE::open(OUTPUTFILE,">${OUTPUT_FILE}.dat") or die("ERROR: Unable to open output file");
    print(OUTPUTFILE "%depth 11 12 22 13 23 33   element $NE \n");
    $in=0;
    while (<INPUTFILE>) 
    {
        @IPfileLine = split;
        $IPfileLine[3] =~ s/D/E/g;
        if ($IPfileLine[0] eq "Xi(i):") 
      	{
      			$depth = $IPfileLine[3]*-100+100;
  	    		print(OUTPUTFILE "$depth ");
  	    }
        if ($IPfileLine[0] eq "TG(1,i):") 
      	{
  	    		print(OUTPUTFILE "@IPfileLine[1] ");
            $in=1;
  	    } 
        if (($IPfileLine[0] eq "2") && ($in == 1)) 
      	{
  	    		print(OUTPUTFILE "@IPfileLine[1..2] ");
  	    } 
        if (($IPfileLine[0] eq "3") && ($in == 1) ) 
      	{
  	    		print(OUTPUTFILE "@IPfileLine[1..3] \n");
            $in=0;
  	    } 
    }
    CORE::close(INPUTFILE);
    CORE::close(OUTPUTFILE);
#    unlink "${OUTPUT_FILE}_tmp.opstra";
}





sub Combine_Element_Files{
		my $INPUT_FILE_1 = shift;
		my $INPUT_FILE_2 = shift;
#		my $INPUT_FILE_3 = shift;
		my $OUTPUT_FILE = shift;


    CORE::open(INPUTFILE_1,"<${INPUT_FILE_1}.dat") or die("ERROR: Unable to open input file");
    CORE::open(INPUTFILE_2,"<${INPUT_FILE_2}.dat") or die("ERROR: Unable to open input file");
#    CORE::open(INPUTFILE_3,"<${INPUT_FILE_3}.dat") or die("ERROR: Unable to open input file");
    CORE::open(OUTPUTFILE,">${OUTPUT_FILE}.dat") or die("ERROR: Unable to open output file");
    print(OUTPUTFILE "%depth exr 11 12 22 13 23 33\n");
#    while (<INPUTFILE_3>) 
#    {
#        @IPfileLine = split;
#        if ($IPfileLine[0] ne "%depth") 
#      	{
#      			$depth = $IPfileLine[0]*0.3333333+66.6666666;
#  	    		print(OUTPUTFILE "$depth @IPfileLine[1..9] \n");
#  	    }
#    }
    while (<INPUTFILE_1>) 
    {
        @IPfileLine = split;
        if ($IPfileLine[0] ne "%depth") 
      	{
#      			$depth = $IPfileLine[0]*0.3333333;
      			$depth = $IPfileLine[0]*0.5+50;
  	    		print(OUTPUTFILE "$depth @IPfileLine[1..9] \n");
  	    }
    }
    while (<INPUTFILE_2>) 
    {
        @IPfileLine = split;
        if ($IPfileLine[0] ne "%depth") 
      	{
#      			$depth = $IPfileLine[0]*0.3333333+33.33333333;
      			$depth = $IPfileLine[0]*0.5;
  	    		print(OUTPUTFILE "$depth @IPfileLine[1..9] \n");
  	    }
    }
    CORE::close(INPUTFILE_1);
    CORE::close(INPUTFILE_2);
#    CORE::close(INPUTFILE_3);
    CORE::close(OUTPUTFILE);



}





sub Create_Slice_EXR_FILE{

    $OUTPUT_FILE="output_FM_GAUSS/full_heart_press_1_slice_exr.dat";
    CORE::open(OUTPUTFILE,">${OUTPUT_FILE}") or die("ERROR: Unable to open output file");
    print(OUTPUTFILE "% Gauss point fibre extension ratios in slice element sequence\n");
    
    @elements = (38,35,28,11,6,3,14,15,16,17,37,22,23,24,25,36);
    $count=0;
    $point=0;
    foreach $ne (@elements)
    {
	$INPUT_FILE="output_FM_GAUSS/full_heart_press_1_ne_".$ne."_fibre_strain_gauss.dat";
        CORE::open(INPUTFILE,"<${INPUT_FILE}") or die("ERROR: Unable to open input file");
        $gauss=-1;
	while (<INPUTFILE>)  
	{
	    @IPfileLine = split;
            $IPfileLine[1] =~ s/D/E/g;
	    $gauss++;
            if ($gauss == 13 || $gauss == 14 || $gauss == 15)
            {
                $point++;
		if($point != 1 && $point != 2 && $point != 17 && $point != 18) {
		    print(OUTPUTFILE "$point @IPfileLine[1] @IPfileLine[3] @IPfileLine[6] \n");
		}
	    }
	    if($point == 17 || $point ==33) {
		print(OUTPUTFILE "\n\n");
	    }
	}
	CORE::close(INPUTFILE);
	$count++;
    }
    CORE::close(OUTPUTFILE);
}



sub Create_Slice_EXR_FILE_infarct{

    $OUTPUT_FILE="output_FM_GAUSS.good_infarct/full_heart_press_0.9_slice_exr.dat";
    CORE::open(OUTPUTFILE,">${OUTPUT_FILE}") or die("ERROR: Unable to open output file");
    print(OUTPUTFILE "% Gauss point fibre extension ratios in slice element sequence\n");
    
#    @elements = (28,11,109,103,115,6);
#    @elements = (96,94,112,106,118,100);
    @elements = (94,112,106,118,100);
    $count=0;
    $point=0;
    foreach $ne (@elements)
    {
	$INPUT_FILE="output_FM_GAUSS.good_infarct/full_heart_press_0.9_ne_".$ne."_fibre_strain_gauss.dat";
        CORE::open(INPUTFILE,"<${INPUT_FILE}") or die("ERROR: Unable to open input file");
        $gauss=-1;
	while (<INPUTFILE>)  
	{
	    @IPfileLine = split;
            $IPfileLine[1] =~ s/D/E/g;
	    $gauss++;
#            if ($gauss == 22 || $gauss == 23 || $gauss == 24)
            if ($gauss == 4 || $gauss == 5 || $gauss == 6)
#            if ($gauss == 13 || $gauss == 14 || $gauss == 15)
            {
                $point++;
#		if($point != 1 && $point != 2 && $point != 17 && $point != 18) {
		    print(OUTPUTFILE "$point @IPfileLine[1] @IPfileLine[3] @IPfileLine[6] \n");
#		}
	    }
#	    if($point == 17 || $point ==33) {
#		print(OUTPUTFILE "\n\n");
#	    }
	}
	CORE::close(INPUTFILE);
	$count++;
    }
    CORE::close(OUTPUTFILE);




    $OUTPUT_FILE="output_FM_GAUSS.good_infarct_normal/full_heart_press_0.9_slice_exr.dat";
    CORE::open(OUTPUTFILE,">${OUTPUT_FILE}") or die("ERROR: Unable to open output file");
    print(OUTPUTFILE "% Gauss point fibre extension ratios in slice element sequence\n");
    
#    @elements = (28,11,109,103,115,6);
#    @elements = (96,94,112,106,118,100);
    @elements = (94,112,106,118,100);
    $count=0;
    $point=0;
    foreach $ne (@elements)
    {
	$INPUT_FILE="output_FM_GAUSS.good_infarct_normal/full_heart_press_0.9_ne_".$ne."_fibre_strain_gauss.dat";
        CORE::open(INPUTFILE,"<${INPUT_FILE}") or die("ERROR: Unable to open input file");
        $gauss=-1;
	while (<INPUTFILE>)  
	{
	    @IPfileLine = split;
            $IPfileLine[1] =~ s/D/E/g;
	    $gauss++;
#            if ($gauss == 22 || $gauss == 23 || $gauss == 24)
            if ($gauss == 4 || $gauss == 5 || $gauss == 6)
#            if ($gauss == 13 || $gauss == 14 || $gauss == 15)
            {
                $point++;
#		if($point != 1 && $point != 2 && $point != 17 && $point != 18) {
		    print(OUTPUTFILE "$point @IPfileLine[1] @IPfileLine[3] @IPfileLine[6] \n");
#		}
	    }
#	    if($point == 17 || $point ==33) {
#		print(OUTPUTFILE "\n\n");
#	    }
	}
	CORE::close(INPUTFILE);
	$count++;
    }
    CORE::close(OUTPUTFILE);


}




sub Create_Section_EXR_FILE{

    $OUTPUT_FILE1="output_FM_GAUSS/full_heart_press_1_section_exrFibre_epi.dat";
    $OUTPUT_FILE2="output_FM_GAUSS/full_heart_press_1_section_exrFibre_mid.dat";
    $OUTPUT_FILE3="output_FM_GAUSS/full_heart_press_1_section_exrFibre_endo.dat";
    CORE::open(OUTPUTFILE1,">${OUTPUT_FILE1}") or die("ERROR: Unable to open output file");
    CORE::open(OUTPUTFILE2,">${OUTPUT_FILE2}") or die("ERROR: Unable to open output file");
    CORE::open(OUTPUTFILE3,">${OUTPUT_FILE3}") or die("ERROR: Unable to open output file");
    print(OUTPUTFILE1 "% Gauss point fibre extension ratios in vertical section\n");
    print(OUTPUTFILE2 "% Gauss point fibre extension ratios in vertical section\n");
    print(OUTPUTFILE3 "% Gauss point fibre extension ratios in vertical section\n");
    
    @elements = (13,12,11,10,9);
    $count=0;
    $point1=0;
    $point2=0;
    $point3=0;
    foreach $ne (@elements)
    {
	$INPUT_FILE="output_FM_GAUSS/full_heart_press_1_ne_".$ne."_fibre_strain_gauss.dat";
        CORE::open(INPUTFILE,"<${INPUT_FILE}") or die("ERROR: Unable to open input file");
        $gauss=-1;
	while (<INPUTFILE>)  
	{
	    @IPfileLine = split;
            $IPfileLine[1] =~ s/D/E/g;
	    $gauss++;
            if ($gauss == 2 || $gauss == 5 || $gauss == 8)
            {
                $point3++;
		print(OUTPUTFILE3 "$point3 @IPfileLine[1] \n");
	    }
            if ($gauss == 11 || $gauss == 14 || $gauss == 17)
            {
                $point2++;
		print(OUTPUTFILE2 "$point2 @IPfileLine[1] \n");
	    }
            if ($gauss == 20 || $gauss == 23 || $gauss == 26)
            {
                $point1++;
		print(OUTPUTFILE1 "$point1 @IPfileLine[1] \n");
	    }
	}
	CORE::close(INPUTFILE);
	$count++;
    }
    CORE::close(OUTPUTFILE1);
    CORE::close(OUTPUTFILE2);
    CORE::close(OUTPUTFILE3);
}

sub Create_Slice_EXR_FILE_systole{

    $OUTPUT_FILE="output_FM_GAUSS/full_heart_def_active_slice_exr.dat";
    CORE::open(OUTPUTFILE,">${OUTPUT_FILE}") or die("ERROR: Unable to open output file");
    print(OUTPUTFILE "% Gauss point fibre extension ratios in slice element sequence\n");
    
    @elements = (38,35,28,11,6,3,14,15,16,17,37,22,23,24,25,36);
    $count=0;
    $point=0;
    foreach $ne (@elements)
    {
	$INPUT_FILE="output_FM_GAUSS.good/fullheart_def_active_30_press__ne_".$ne."_fibre_strain_gauss.dat";
        CORE::open(INPUTFILE,"<${INPUT_FILE}") or die("ERROR: Unable to open input file");
        $gauss=-1;
	while (<INPUTFILE>)  
	{
	    @IPfileLine = split;
            $IPfileLine[1] =~ s/D/E/g;
	    $gauss++;
            if ($gauss == 13 || $gauss == 14 || $gauss == 15)
            {
                $point++;
		if($point != 1 && $point != 2 && $point != 17 && $point != 18) {
		    print(OUTPUTFILE "$point @IPfileLine[1] @IPfileLine[3] @IPfileLine[6] \n");
		}
	    }
	    if($point == 17 || $point ==33) {
		print(OUTPUTFILE "\n\n");
	    }
	}
	CORE::close(INPUTFILE);
	$count++;
    }
    CORE::close(OUTPUTFILE);
}

sub Create_Section_EXR_FILE_systole{

    $OUTPUT_FILE1="output_FM_GAUSS/full_heart_def_active_section_exrFibre_epi.dat";
    $OUTPUT_FILE2="output_FM_GAUSS/full_heart_def_active_section_exrFibre_mid.dat";
    $OUTPUT_FILE3="output_FM_GAUSS/full_heart_def_active_section_exrFibre_endo.dat";
    CORE::open(OUTPUTFILE1,">${OUTPUT_FILE1}") or die("ERROR: Unable to open output file");
    CORE::open(OUTPUTFILE2,">${OUTPUT_FILE2}") or die("ERROR: Unable to open output file");
    CORE::open(OUTPUTFILE3,">${OUTPUT_FILE3}") or die("ERROR: Unable to open output file");
    print(OUTPUTFILE1 "% Gauss point fibre extension ratios in vertical section\n");
    print(OUTPUTFILE2 "% Gauss point fibre extension ratios in vertical section\n");
    print(OUTPUTFILE3 "% Gauss point fibre extension ratios in vertical section\n");
    
    @elements = (13,12,11,10,9);
    $count=0;
    $point1=0;
    $point2=0;
    $point3=0;
    foreach $ne (@elements)
    {
	$INPUT_FILE="output_FM_GAUSS.good/fullheart_def_active_30_press__ne_".$ne."_fibre_strain_gauss.dat";
        CORE::open(INPUTFILE,"<${INPUT_FILE}") or die("ERROR: Unable to open input file");
        $gauss=-1;
	while (<INPUTFILE>)  
	{
	    @IPfileLine = split;
            $IPfileLine[1] =~ s/D/E/g;
	    $gauss++;
            if ($gauss == 2 || $gauss == 5 || $gauss == 8)
            {
                $point3++;
		print(OUTPUTFILE3 "$point3 @IPfileLine[1] \n");
	    }
            if ($gauss == 11 || $gauss == 14 || $gauss == 17)
            {
                $point2++;
		print(OUTPUTFILE2 "$point2 @IPfileLine[1] \n");
	    }
            if ($gauss == 20 || $gauss == 23 || $gauss == 26)
            {
                $point1++;
		print(OUTPUTFILE1 "$point1 @IPfileLine[1] \n");
	    }
	}
	CORE::close(INPUTFILE);
	$count++;
    }
    CORE::close(OUTPUTFILE1);
    CORE::close(OUTPUTFILE2);
    CORE::close(OUTPUTFILE3);
}







sub Create_Section_EXR_FILE_infarct{

    $OUTPUT_FILE1="output_FM_GAUSS/full_heart_press_1_section_exrFibre_epi.dat";
    $OUTPUT_FILE2="output_FM_GAUSS/full_heart_press_1_section_exrFibre_mid.dat";
    $OUTPUT_FILE3="output_FM_GAUSS/full_heart_press_1_section_exrFibre_endo.dat";
    CORE::open(OUTPUTFILE1,">${OUTPUT_FILE1}") or die("ERROR: Unable to open output file");
    CORE::open(OUTPUTFILE2,">${OUTPUT_FILE2}") or die("ERROR: Unable to open output file");
    CORE::open(OUTPUTFILE3,">${OUTPUT_FILE3}") or die("ERROR: Unable to open output file");
    print(OUTPUTFILE1 "% Gauss point fibre extension ratios in vertical section\n");
    print(OUTPUTFILE2 "% Gauss point fibre extension ratios in vertical section\n");
    print(OUTPUTFILE3 "% Gauss point fibre extension ratios in vertical section\n");
    
    @elements = (13,12,11,10,9);
    $count=0;
    $point1=0;
    $point2=0;
    $point3=0;
    foreach $ne (@elements)
    {
	$INPUT_FILE="output_FM_GAUSS/full_heart_press_1_ne_".$ne."_fibre_strain_gauss.dat";
        CORE::open(INPUTFILE,"<${INPUT_FILE}") or die("ERROR: Unable to open input file");
        $gauss=-1;
	while (<INPUTFILE>)  
	{
	    @IPfileLine = split;
            $IPfileLine[1] =~ s/D/E/g;
	    $gauss++;
            if ($gauss == 2 || $gauss == 5 || $gauss == 8)
            {
                $point3++;
		print(OUTPUTFILE3 "$point3 @IPfileLine[1] \n");
	    }
            if ($gauss == 11 || $gauss == 14 || $gauss == 17)
            {
                $point2++;
		print(OUTPUTFILE2 "$point2 @IPfileLine[1] \n");
	    }
            if ($gauss == 20 || $gauss == 23 || $gauss == 26)
            {
                $point1++;
		print(OUTPUTFILE1 "$point1 @IPfileLine[1] \n");
	    }
	}
	CORE::close(INPUTFILE);
	$count++;
    }
    CORE::close(OUTPUTFILE1);
    CORE::close(OUTPUTFILE2);
    CORE::close(OUTPUTFILE3);
}













sub Plot_Transmural_Infarct_Strain_Holmes{
		my($OUTPUT_DIR) = @_;

# Transmural infarct strain from holmes data

$gnuplot_path = "/usr/local/bin/gnuplot";
$saveEps = 1;

    CORE::open GNUPLOT, "| $gnuplot_path";
      print GNUPLOT "set xlabel \"Transmural Depth (%)\"\n";
      print GNUPLOT "set yrange [-0.1:0.4]\n";
      print GNUPLOT "set format y \"\%4.3f\"\n";
      print GNUPLOT "set key outside\n";
      print GNUPLOT "set terminal postscript eps color \"Times-Roman\" 20\n";
      print GNUPLOT "set linestyle 1 lt -1 lw 2 pt 1 ps 2\n";
      print GNUPLOT "set linestyle 2 lt -1 lw 2 pt 2 ps 2\n";
      print GNUPLOT "set linestyle 3 lt -1 lw 2 pt 4 ps 2\n";
      print GNUPLOT "set linestyle 4 lt -1 lw 2 pt 6 ps 2\n";
      print GNUPLOT "set linestyle 5 lt -1 lw 2 pt 10 ps 2\n";
      print GNUPLOT "set linestyle 6 lt -1 lw 2 pt 8 ps 2\n";
       print GNUPLOT "set data style linespoints\n";

      print GNUPLOT "set ylabel \"Circumferential (E11)\"\n";
      print GNUPLOT "set output \"output/scp04_holmes_infarct_wall_11.eps\"\n\n";
      print GNUPLOT "plot \'scp04_infarct_strains.prn\' index 0 using 1:3 title \"5 mmHg\" ls 1, \'scp04_infarct_strains.prn\' index 1 using 1:3 title \"10 mmHg\" ls 2,  \'scp04_infarct_strains.prn\' index 2 using 1:3 title \"15 mmHg\" ls 3,  \'scp04_infarct_strains.prn\' index 3 using 1:3 title \"20 mmHg\" ls 4,  \'scp04_infarct_strains.prn\' index 4 using 1:3 title \"25 mmHg\" ls 5,  \'scp04_infarct_strains.prn\' index 5 using 1:3 title \"30 mmHg\" ls 6 \n";


      print GNUPLOT "set ylabel \"Longitudinal (E22)\"\n";
      print GNUPLOT "set yrange [0.0:0.9]\n";
      print GNUPLOT "set output \"output/scp04_holmes_infarct_wall_22.eps\"\n\n";
      print GNUPLOT "plot \'scp04_infarct_strains.prn\' index 0 using 1:4 title \"5 mmHg\" ls 1, \'scp04_infarct_strains.prn\' index 1 using 1:4 title \"10 mmHg\" ls 2,  \'scp04_infarct_strains.prn\' index 2 using 1:4 title \"15 mmHg\" ls 3,  \'scp04_infarct_strains.prn\' index 3 using 1:4 title \"20 mmHg\" ls 4,  \'scp04_infarct_strains.prn\' index 4 using 1:4 title \"25 mmHg\" ls 5,  \'scp04_infarct_strains.prn\' index 5 using 1:4 title \"30 mmHg\" ls 6 \n";


      print GNUPLOT "set ylabel \"Radial (E33)\"\n";
      print GNUPLOT "set yrange [-0.4:0] \n";
      print GNUPLOT "set output \"output/scp04_holmes_infarct_wall_33.eps\"\n\n";
      print GNUPLOT "plot \'scp04_infarct_strains.prn\' index 0 using 1:5 title \"5 mmHg\" ls 1, \'scp04_infarct_strains.prn\' index 1 using 1:5 title \"10 mmHg\" ls 2,  \'scp04_infarct_strains.prn\' index 2 using 1:5 title \"15 mmHg\" ls 3,  \'scp04_infarct_strains.prn\' index 3 using 1:5 title \"20 mmHg\" ls 4,  \'scp04_infarct_strains.prn\' index 4 using 1:5 title \"25 mmHg\" ls 5,  \'scp04_infarct_strains.prn\' index 5 using 1:5 title \"30 mmHg\" ls 6 \n";


      print GNUPLOT "set ylabel \"In-Plane (E12)\"\n";
      print GNUPLOT "set yrange [-0.4:0.2] \n";
      print GNUPLOT "set arrow from  0,0 to  100,0 nohead lt 0\n";
      print GNUPLOT "set output \"output/scp04_holmes_infarct_wall_12.eps\"\n\n";
      print GNUPLOT "plot \'scp04_infarct_strains.prn\' index 0 using 1:6 title \"5 mmHg\" ls 1, \'scp04_infarct_strains.prn\' index 1 using 1:6 title \"10 mmHg\" ls 2,  \'scp04_infarct_strains.prn\' index 2 using 1:6 title \"15 mmHg\" ls 3,  \'scp04_infarct_strains.prn\' index 3 using 1:6 title \"20 mmHg\" ls 4,  \'scp04_infarct_strains.prn\' index 4 using 1:6 title \"25 mmHg\" ls 5,  \'scp04_infarct_strains.prn\' index 5 using 1:6 title \"30 mmHg\" ls 6 \n";


      print GNUPLOT "set ylabel \"Transverse (E13)\"\n";
      print GNUPLOT "set yrange [-0.4:0.2] \n";
      print GNUPLOT "set arrow from  0,0 to  100,0 nohead lt 0\n";
      print GNUPLOT "set output \"output/scp04_holmes_infarct_wall_13.eps\"\n\n";
      print GNUPLOT "plot \'scp04_infarct_strains.prn\' index 0 using 1:7 title \"5 mmHg\" ls 1, \'scp04_infarct_strains.prn\' index 1 using 1:7 title \"10 mmHg\" ls 2,  \'scp04_infarct_strains.prn\' index 2 using 1:7 title \"15 mmHg\" ls 3,  \'scp04_infarct_strains.prn\' index 3 using 1:7 title \"20 mmHg\" ls 4,  \'scp04_infarct_strains.prn\' index 4 using 1:7 title \"25 mmHg\" ls 5,  \'scp04_infarct_strains.prn\' index 5 using 1:7 title \"30 mmHg\" ls 6 \n";


      print GNUPLOT "set ylabel \"Transverse (E23)\"\n";
      print GNUPLOT "set yrange [-0.4:0.2] \n";
      print GNUPLOT "set arrow from  0,0 to  100,0 nohead lt 0\n";
      print GNUPLOT "set output \"output/scp04_holmes_infarct_wall_23.eps\"\n\n";
      print GNUPLOT "plot \'scp04_infarct_strains.prn\' index 0 using 1:8 title \"5 mmHg\" ls 1, \'scp04_infarct_strains.prn\' index 1 using 1:8 title \"10 mmHg\" ls 2,  \'scp04_infarct_strains.prn\' index 2 using 1:8 title \"15 mmHg\" ls 3,  \'scp04_infarct_strains.prn\' index 3 using 1:8 title \"20 mmHg\" ls 4,  \'scp04_infarct_strains.prn\' index 4 using 1:8 title \"25 mmHg\" ls 5,  \'scp04_infarct_strains.prn\' index 5 using 1:8 title \"30 mmHg\" ls 6 \n";

    close GNUPLOT;

}


sub Plot_Transmural_Infarct_Strain_Holmes_Model{
		my($OUTPUT_DIR) = @_;

# Transmural infarct strain from holmes data

$gnuplot_path = "/usr/local/bin/gnuplot";
$saveEps = 1;


    CORE::open GNUPLOT, "| $gnuplot_path";
      print GNUPLOT "set xlabel \"Transmural Depth (%)\"\n";
      print GNUPLOT "set yrange [-0.1:0.4]\n";
      print GNUPLOT "set format y \"\%4.3f\"\n";
      print GNUPLOT "set key outside\n";
      print GNUPLOT "set terminal postscript eps color \"Times-Roman\" 20\n";
      print GNUPLOT "set linestyle 1 lt -1 lw 2 pt 1 ps 2\n";
      print GNUPLOT "set linestyle 2 lt -1 lw 2 pt 2 ps 2\n";
      print GNUPLOT "set linestyle 3 lt -1 lw 2 pt 4 ps 2\n";
      print GNUPLOT "set linestyle 4 lt -1 lw 2 pt 6 ps 2\n";
      print GNUPLOT "set linestyle 5 lt -1 lw 2 pt 10 ps 2\n";
      print GNUPLOT "set linestyle 6 lt -1 lw 2 pt 8 ps 2\n";
      print GNUPLOT "set linestyle 7 lt 1 lw 2 pt 1 ps 2\n";
      print GNUPLOT "set linestyle 8 lt 1 lw 2 pt 2 ps 2\n";
      print GNUPLOT "set linestyle 9 lt 1 lw 2 pt 4 ps 2\n";
      print GNUPLOT "set linestyle 10 lt 1 lw 2 pt 6 ps 2\n";
      print GNUPLOT "set linestyle 11 lt 1 lw 2 pt 10 ps 2\n";
      print GNUPLOT "set linestyle 12 lt 1 lw 2 pt 8 ps 2\n";
      print GNUPLOT "set data style linespoints\n";

      print GNUPLOT "set ylabel \"Circumferential (E11)\"\n";
      print GNUPLOT "set output \"output/scp04_holmes_model_infarct_wall_11.eps\"\n\n";
      print GNUPLOT "plot \'scp04_infarct_strains.prn\' index 0 using 1:3 title \"0.6 KPa\" ls 1, \'scp04_infarct_strains.prn\' index 1 using 1:3 title \"1.3 KPa\" ls 2,  \'scp04_infarct_strains.prn\' index 2 using 1:3 title \"2.0 KPa\" ls 3,  \'scp04_infarct_strains.prn\' index 3 using 1:3 title \"2.6 KPa\" ls 4,  \'scp04_infarct_strains.prn\' index 4 using 1:3 title \"3.3 KPa\" ls 5,  \'scp04_infarct_strains.prn\' index 5 using 1:3 title \"4.0 KPa\" ls 6 ,  \'output/scp04_def_0007_active_0000_infarct.dat\' index 0 using 1:2 title \"0.7 KPa\" ls 7,  \'output/scp04_def_0013_active_0000_infarct.dat\' index 0 using 1:2 title \"1.3 KPa\" ls 8,  \'output/scp04_def_0020_active_0000_infarct.dat\' index 0 using 1:2 title \"2.0 KPa\" ls 9,  \'output/scp04_def_0027_active_0000_infarct.dat\' index 0 using 1:2 title \"2.7 KPa\" ls 10,  \'output/scp04_def_0033_active_0000_infarct.dat\' index 0 using 1:2 title \"3.3 KPa\" ls 11,  \'output/scp04_def_0040_active_0000_infarct.dat\' index 0 using 1:2 title \"4.0 KPa\" ls 12  \n";



      print GNUPLOT "set ylabel \"Longitudinal (E22)\"\n";
      print GNUPLOT "set yrange [0.0:0.9]\n";
      print GNUPLOT "set output \"output/scp04_holmes_model_infarct_wall_22.eps\"\n\n";
      print GNUPLOT "plot \'scp04_infarct_strains.prn\' index 0 using 1:4 title \"0.6 KPa\" ls 1, \'scp04_infarct_strains.prn\' index 1 using 1:4 title \"1.3 KPa\" ls 2,  \'scp04_infarct_strains.prn\' index 2 using 1:4 title \"2.0 KPa\" ls 3,  \'scp04_infarct_strains.prn\' index 3 using 1:4 title \"2.6 KPa\" ls 4,  \'scp04_infarct_strains.prn\' index 4 using 1:4 title \"3.3 KPa\" ls 5,  \'scp04_infarct_strains.prn\' index 5 using 1:4 title \"4.0 KPa\" ls 6 ,  \'output/scp04_def_0007_active_0000_infarct.dat\' index 0 using 1:3 title \"0.7 KPa\" ls 7,  \'output/scp04_def_0013_active_0000_infarct.dat\' index 0 using 1:3 title \"1.3 KPa\" ls 8,  \'output/scp04_def_0020_active_0000_infarct.dat\' index 0 using 1:3 title \"2.0 KPa\" ls 9,  \'output/scp04_def_0027_active_0000_infarct.dat\' index 0 using 1:3 title \"2.7 KPa\" ls 10,  \'output/scp04_def_0033_active_0000_infarct.dat\' index 0 using 1:3 title \"3.3 KPa\" ls 11,  \'output/scp04_def_0040_active_0000_infarct.dat\' index 0 using 1:3 title \"4.0 KPa\" ls 12  \n";



      print GNUPLOT "set ylabel \"Radial (E33)\"\n";
      print GNUPLOT "set yrange [-0.4:0] \n";
      print GNUPLOT "set output \"output/scp04_holmes_model_infarct_wall_33.eps\"\n\n";
      print GNUPLOT "plot \'scp04_infarct_strains.prn\' index 0 using 1:5 title \"0.6 KPa\" ls 1, \'scp04_infarct_strains.prn\' index 1 using 1:5 title \"1.3 KPa\" ls 2,  \'scp04_infarct_strains.prn\' index 2 using 1:5 title \"2.0 KPa\" ls 3,  \'scp04_infarct_strains.prn\' index 3 using 1:5 title \"2.6 KPa\" ls 4,  \'scp04_infarct_strains.prn\' index 4 using 1:5 title \"3.3 KPa\" ls 5,  \'scp04_infarct_strains.prn\' index 5 using 1:5 title \"4.0 KPa\" ls 6 ,  \'output/scp04_def_0007_active_0000_infarct.dat\' index 0 using 1:4 title \"0.7 KPa\" ls 7,  \'output/scp04_def_0013_active_0000_infarct.dat\' index 0 using 1:4 title \"1.3 KPa\" ls 8,  \'output/scp04_def_0020_active_0000_infarct.dat\' index 0 using 1:4 title \"2.0 KPa\" ls 9,  \'output/scp04_def_0027_active_0000_infarct.dat\' index 0 using 1:4 title \"2.7 KPa\" ls 10,  \'output/scp04_def_0033_active_0000_infarct.dat\' index 0 using 1:4 title \"3.3 KPa\" ls 11,  \'output/scp04_def_0040_active_0000_infarct.dat\' index 0 using 1:4 title \"4.0 KPa\" ls 12  \n";



      print GNUPLOT "set ylabel \"In-Plane (E12)\"\n";
      print GNUPLOT "set yrange [-0.4:0.2] \n";
      print GNUPLOT "set arrow from  0,0 to  100,0 nohead lt 0\n";
      print GNUPLOT "set output \"output/scp04_holmes_model_infarct_wall_12.eps\"\n\n";
      print GNUPLOT "plot \'scp04_infarct_strains.prn\' index 0 using 1:6 title \"0.6 KPa\" ls 1, \'scp04_infarct_strains.prn\' index 1 using 1:6 title \"1.3 KPa\" ls 2,  \'scp04_infarct_strains.prn\' index 2 using 1:6 title \"2.0 KPa\" ls 3,  \'scp04_infarct_strains.prn\' index 3 using 1:6 title \"2.6 KPa\" ls 4,  \'scp04_infarct_strains.prn\' index 4 using 1:6 title \"3.3 KPa\" ls 5,  \'scp04_infarct_strains.prn\' index 5 using 1:6 title \"4.0 KPa\" ls 6 ,  \'output/scp04_def_0007_active_0000_infarct.dat\' index 0 using 1:5 title \"0.7 KPa\" ls 7,  \'output/scp04_def_0013_active_0000_infarct.dat\' index 0 using 1:5 title \"1.3 KPa\" ls 8,  \'output/scp04_def_0020_active_0000_infarct.dat\' index 0 using 1:5 title \"2.0 KPa\" ls 9,  \'output/scp04_def_0027_active_0000_infarct.dat\' index 0 using 1:5 title \"2.7 KPa\" ls 10,  \'output/scp04_def_0033_active_0000_infarct.dat\' index 0 using 1:5 title \"3.3 KPa\" ls 11,  \'output/scp04_def_0040_active_0000_infarct.dat\' index 0 using 1:5 title \"4.0 KPa\" ls 12  \n";



      print GNUPLOT "set ylabel \"Transverse (E13)\"\n";
      print GNUPLOT "set yrange [-0.4:0.2] \n";
      print GNUPLOT "set arrow from  0,0 to  100,0 nohead lt 0\n";
      print GNUPLOT "set output \"output/scp04_holmes_model_infarct_wall_13.eps\"\n\n";
      print GNUPLOT "plot \'scp04_infarct_strains.prn\' index 0 using 1:7 title \"0.6 KPa\" ls 1, \'scp04_infarct_strains.prn\' index 1 using 1:7 title \"1.3 KPa\" ls 2,  \'scp04_infarct_strains.prn\' index 2 using 1:7 title \"2.0 KPa\" ls 3,  \'scp04_infarct_strains.prn\' index 3 using 1:7 title \"2.6 KPa\" ls 4,  \'scp04_infarct_strains.prn\' index 4 using 1:7 title \"3.3 KPa\" ls 5,  \'scp04_infarct_strains.prn\' index 5 using 1:7 title \"4.0 KPa\" ls 6 ,  \'output/scp04_def_0007_active_0000_infarct.dat\' index 0 using 1:6 title \"0.7 KPa\" ls 7,  \'output/scp04_def_0013_active_0000_infarct.dat\' index 0 using 1:6 title \"1.3 KPa\" ls 8,  \'output/scp04_def_0020_active_0000_infarct.dat\' index 0 using 1:6 title \"2.0 KPa\" ls 9,  \'output/scp04_def_0027_active_0000_infarct.dat\' index 0 using 1:6 title \"2.7 KPa\" ls 10,  \'output/scp04_def_0033_active_0000_infarct.dat\' index 0 using 1:6 title \"3.3 KPa\" ls 11,  \'output/scp04_def_0040_active_0000_infarct.dat\' index 0 using 1:6 title \"4.0 KPa\" ls 12  \n";
 


      print GNUPLOT "set ylabel \"Transverse (E23)\"\n";
      print GNUPLOT "set yrange [-0.4:0.2] \n";
      print GNUPLOT "set arrow from  0,0 to  100,0 nohead lt 0\n";
      print GNUPLOT "set output \"output/scp04_holmes_model_infarct_wall_23.eps\"\n\n";
      print GNUPLOT "plot \'scp04_infarct_strains.prn\' index 0 using 1:8 title \"0.6 KPa\" ls 1, \'scp04_infarct_strains.prn\' index 1 using 1:8 title \"1.3 KPa\" ls 2,  \'scp04_infarct_strains.prn\' index 2 using 1:8 title \"2.0 KPa\" ls 3,  \'scp04_infarct_strains.prn\' index 3 using 1:8 title \"2.6 KPa\" ls 4,  \'scp04_infarct_strains.prn\' index 4 using 1:8 title \"3.3 KPa\" ls 5,  \'scp04_infarct_strains.prn\' index 5 using 1:8 title \"4.0 KPa\" ls 6 ,  \'output/scp04_def_0007_active_0000_infarct.dat\' index 0 using 1:7 title \"0.7 KPa\" ls 7,  \'output/scp04_def_0013_active_0000_infarct.dat\' index 0 using 1:7 title \"1.3 KPa\" ls 8,  \'output/scp04_def_0020_active_0000_infarct.dat\' index 0 using 1:7 title \"2.0 KPa\" ls 9,  \'output/scp04_def_0027_active_0000_infarct.dat\' index 0 using 1:7 title \"2.7 KPa\" ls 10,  \'output/scp04_def_0033_active_0000_infarct.dat\' index 0 using 1:7 title \"3.3 KPa\" ls 11,  \'output/scp04_def_0040_active_0000_infarct.dat\' index 0 using 1:7 title \"4.0 KPa\" ls 12  \n";
 


    close GNUPLOT;

}






sub Plot_Transmural_Control_Strain_Holmes_Model{
		my($OUTPUT_DIR) = @_;

# Transmural control strain from holmes data

$gnuplot_path = "/usr/local/bin/gnuplot";
$saveEps = 1;


    CORE::open GNUPLOT, "| $gnuplot_path";
      print GNUPLOT "set xlabel \"Transmural Depth (%)\"\n";
      print GNUPLOT "set format y \"\%4.3f\"\n";
      print GNUPLOT "set key outside\n";
      print GNUPLOT "set terminal postscript eps color \"Times-Roman\" 20\n";
      print GNUPLOT "set linestyle 1 lt -1 lw 2 pt 1 ps 2\n";
      print GNUPLOT "set linestyle 2 lt -1 lw 2 pt 2 ps 2\n";
      print GNUPLOT "set linestyle 3 lt -1 lw 2 pt 4 ps 2\n";
      print GNUPLOT "set linestyle 4 lt -1 lw 2 pt 6 ps 2\n";
      print GNUPLOT "set linestyle 5 lt -1 lw 2 pt 10 ps 2\n";
      print GNUPLOT "set linestyle 6 lt -1 lw 2 pt 8 ps 2\n";
      print GNUPLOT "set linestyle 7 lt 1 lw 2 pt 1 ps 2\n";
      print GNUPLOT "set linestyle 8 lt 1 lw 2 pt 2 ps 2\n";
      print GNUPLOT "set linestyle 9 lt 1 lw 2 pt 4 ps 2\n";
      print GNUPLOT "set linestyle 10 lt 1 lw 2 pt 6 ps 2\n";
      print GNUPLOT "set linestyle 11 lt 1 lw 2 pt 10 ps 2\n";
      print GNUPLOT "set linestyle 12 lt 1 lw 2 pt 8 ps 2\n";
      print GNUPLOT "set data style linespoints\n";

      print GNUPLOT "set ylabel \"Circumferential (E11)\"\n";
      print GNUPLOT "set yrange [-0.1:0.6]\n";
       print GNUPLOT "set output \"output/scp04_holmes_model_control_wall_11.eps\"\n\n";
      print GNUPLOT "plot \'scp04_control_strains.prn\' index 0 using 1:3 title \"0.6 KPa\" ls 1, \'scp04_control_strains.prn\' index 1 using 1:3 title \"1.3 KPa\" ls 2,  \'scp04_control_strains.prn\' index 2 using 1:3 title \"2.0 KPa\" ls 3,  \'scp04_control_strains.prn\' index 3 using 1:3 title \"2.6 KPa\" ls 4,  \'scp04_control_strains.prn\' index 4 using 1:3 title \"3.3 KPa\" ls 5,  \'scp04_control_strains.prn\' index 5 using 1:3 title \"4.0 KPa\" ls 6 ,  \'output/scp04_def_0007_active_0000_control.dat\' index 0 using 1:2 title \"0.7 KPa\" ls 7,  \'output/scp04_def_0013_active_0000_control.dat\' index 0 using 1:2 title \"1.3 KPa\" ls 8,  \'output/scp04_def_0020_active_0000_control.dat\' index 0 using 1:2 title \"2.0 KPa\" ls 9,  \'output/scp04_def_0027_active_0000_control.dat\' index 0 using 1:2 title \"2.7 KPa\" ls 10,  \'output/scp04_def_0033_active_0000_control.dat\' index 0 using 1:2 title \"3.3 KPa\" ls 11,  \'output/scp04_def_0040_active_0000_control.dat\' index 0 using 1:2 title \"4.0 KPa\" ls 12  \n";



      print GNUPLOT "set ylabel \"Longitudinal (E22)\"\n";
      print GNUPLOT "set yrange [0.0:0.9]\n";
      print GNUPLOT "set output \"output/scp04_holmes_model_control_wall_22.eps\"\n\n";
      print GNUPLOT "plot \'scp04_control_strains.prn\' index 0 using 1:4 title \"0.6 KPa\" ls 1, \'scp04_control_strains.prn\' index 1 using 1:4 title \"1.3 KPa\" ls 2,  \'scp04_control_strains.prn\' index 2 using 1:4 title \"2.0 KPa\" ls 3,  \'scp04_control_strains.prn\' index 3 using 1:4 title \"2.6 KPa\" ls 4,  \'scp04_control_strains.prn\' index 4 using 1:4 title \"3.3 KPa\" ls 5,  \'scp04_control_strains.prn\' index 5 using 1:4 title \"4.0 KPa\" ls 6 ,  \'output/scp04_def_0007_active_0000_control.dat\' index 0 using 1:3 title \"0.7 KPa\" ls 7,  \'output/scp04_def_0013_active_0000_control.dat\' index 0 using 1:3 title \"1.3 KPa\" ls 8,  \'output/scp04_def_0020_active_0000_control.dat\' index 0 using 1:3 title \"2.0 KPa\" ls 9,  \'output/scp04_def_0027_active_0000_control.dat\' index 0 using 1:3 title \"2.7 KPa\" ls 10,  \'output/scp04_def_0033_active_0000_control.dat\' index 0 using 1:3 title \"3.3 KPa\" ls 11,  \'output/scp04_def_0040_active_0000_control.dat\' index 0 using 1:3 title \"4.0 KPa\" ls 12  \n";



      print GNUPLOT "set ylabel \"Radial (E33)\"\n";
      print GNUPLOT "set yrange [-0.4:0] \n";
      print GNUPLOT "set output \"output/scp04_holmes_model_control_wall_33.eps\"\n\n";
      print GNUPLOT "plot \'scp04_control_strains.prn\' index 0 using 1:5 title \"0.6 KPa\" ls 1, \'scp04_control_strains.prn\' index 1 using 1:5 title \"1.3 KPa\" ls 2,  \'scp04_control_strains.prn\' index 2 using 1:5 title \"2.0 KPa\" ls 3,  \'scp04_control_strains.prn\' index 3 using 1:5 title \"2.6 KPa\" ls 4,  \'scp04_control_strains.prn\' index 4 using 1:5 title \"3.3 KPa\" ls 5,  \'scp04_control_strains.prn\' index 5 using 1:5 title \"4.0 KPa\" ls 6 ,  \'output/scp04_def_0007_active_0000_control.dat\' index 0 using 1:4 title \"0.7 KPa\" ls 7,  \'output/scp04_def_0013_active_0000_control.dat\' index 0 using 1:4 title \"1.3 KPa\" ls 8,  \'output/scp04_def_0020_active_0000_control.dat\' index 0 using 1:4 title \"2.0 KPa\" ls 9,  \'output/scp04_def_0027_active_0000_control.dat\' index 0 using 1:4 title \"2.7 KPa\" ls 10,  \'output/scp04_def_0033_active_0000_control.dat\' index 0 using 1:4 title \"3.3 KPa\" ls 11,  \'output/scp04_def_0040_active_0000_control.dat\' index 0 using 1:4 title \"4.0 KPa\" ls 12  \n";



      print GNUPLOT "set ylabel \"In-Plane (E12)\"\n";
      print GNUPLOT "set yrange [-0.4:0.2] \n";
      print GNUPLOT "set arrow from  0,0 to  100,0 nohead lt 0\n";
      print GNUPLOT "set output \"output/scp04_holmes_model_control_wall_12.eps\"\n\n";
      print GNUPLOT "plot \'scp04_control_strains.prn\' index 0 using 1:6 title \"0.6 KPa\" ls 1, \'scp04_control_strains.prn\' index 1 using 1:6 title \"1.3 KPa\" ls 2,  \'scp04_control_strains.prn\' index 2 using 1:6 title \"2.0 KPa\" ls 3,  \'scp04_control_strains.prn\' index 3 using 1:6 title \"2.6 KPa\" ls 4,  \'scp04_control_strains.prn\' index 4 using 1:6 title \"3.3 KPa\" ls 5,  \'scp04_control_strains.prn\' index 5 using 1:6 title \"4.0 KPa\" ls 6 ,  \'output/scp04_def_0007_active_0000_control.dat\' index 0 using 1:5 title \"0.7 KPa\" ls 7,  \'output/scp04_def_0013_active_0000_control.dat\' index 0 using 1:5 title \"1.3 KPa\" ls 8,  \'output/scp04_def_0020_active_0000_control.dat\' index 0 using 1:5 title \"2.0 KPa\" ls 9,  \'output/scp04_def_0027_active_0000_control.dat\' index 0 using 1:5 title \"2.7 KPa\" ls 10,  \'output/scp04_def_0033_active_0000_control.dat\' index 0 using 1:5 title \"3.3 KPa\" ls 11,  \'output/scp04_def_0040_active_0000_control.dat\' index 0 using 1:5 title \"4.0 KPa\" ls 12  \n";



      print GNUPLOT "set ylabel \"Transverse (E13)\"\n";
      print GNUPLOT "set yrange [-0.4:0.2] \n";
      print GNUPLOT "set arrow from  0,0 to  100,0 nohead lt 0\n";
      print GNUPLOT "set output \"output/scp04_holmes_model_control_wall_13.eps\"\n\n";
      print GNUPLOT "plot \'scp04_control_strains.prn\' index 0 using 1:7 title \"0.6 KPa\" ls 1, \'scp04_control_strains.prn\' index 1 using 1:7 title \"1.3 KPa\" ls 2,  \'scp04_control_strains.prn\' index 2 using 1:7 title \"2.0 KPa\" ls 3,  \'scp04_control_strains.prn\' index 3 using 1:7 title \"2.6 KPa\" ls 4,  \'scp04_control_strains.prn\' index 4 using 1:7 title \"3.3 KPa\" ls 5,  \'scp04_control_strains.prn\' index 5 using 1:7 title \"4.0 KPa\" ls 6 ,  \'output/scp04_def_0007_active_0000_control.dat\' index 0 using 1:6 title \"0.7 KPa\" ls 7,  \'output/scp04_def_0013_active_0000_control.dat\' index 0 using 1:6 title \"1.3 KPa\" ls 8,  \'output/scp04_def_0020_active_0000_control.dat\' index 0 using 1:6 title \"2.0 KPa\" ls 9,  \'output/scp04_def_0027_active_0000_control.dat\' index 0 using 1:6 title \"2.7 KPa\" ls 10,  \'output/scp04_def_0033_active_0000_control.dat\' index 0 using 1:6 title \"3.3 KPa\" ls 11,  \'output/scp04_def_0040_active_0000_control.dat\' index 0 using 1:6 title \"4.0 KPa\" ls 12  \n";
 


      print GNUPLOT "set ylabel \"Transverse (E23)\"\n";
      print GNUPLOT "set yrange [-0.4:0.2] \n";
      print GNUPLOT "set arrow from  0,0 to  100,0 nohead lt 0\n";
      print GNUPLOT "set output \"output/scp04_holmes_model_control_wall_23.eps\"\n\n";
      print GNUPLOT "plot \'scp04_control_strains.prn\' index 0 using 1:8 title \"0.6 KPa\" ls 1, \'scp04_control_strains.prn\' index 1 using 1:8 title \"1.3 KPa\" ls 2,  \'scp04_control_strains.prn\' index 2 using 1:8 title \"2.0 KPa\" ls 3,  \'scp04_control_strains.prn\' index 3 using 1:8 title \"2.6 KPa\" ls 4,  \'scp04_control_strains.prn\' index 4 using 1:8 title \"3.3 KPa\" ls 5,  \'scp04_control_strains.prn\' index 5 using 1:8 title \"4.0 KPa\" ls 6 ,  \'output/scp04_def_0007_active_0000_control.dat\' index 0 using 1:7 title \"0.7 KPa\" ls 7,  \'output/scp04_def_0013_active_0000_control.dat\' index 0 using 1:7 title \"1.3 KPa\" ls 8,  \'output/scp04_def_0020_active_0000_control.dat\' index 0 using 1:7 title \"2.0 KPa\" ls 9,  \'output/scp04_def_0027_active_0000_control.dat\' index 0 using 1:7 title \"2.7 KPa\" ls 10,  \'output/scp04_def_0033_active_0000_control.dat\' index 0 using 1:7 title \"3.3 KPa\" ls 11,  \'output/scp04_def_0040_active_0000_control.dat\' index 0 using 1:7 title \"4.0 KPa\" ls 12  \n";
 


    close GNUPLOT;

}






