package MyFunc;

# readPressure
# This file takes an ipinit file written out during isovolumetric 
# contraction. It finds the LV pressure of that file.
sub readPressure 
{
  my ($inputFile) = @_;
  
  # Check pressures at these elements
  my $LVElement = 1;
  #my $inputFile = "output/temp.ipinit";
  my $boolFinished = 0;
  my $line;
  my $element;
  my $LVPressure;

  if (!(-e $inputFile))
  {
    die "Input-file doesn't exist";
  }
  CORE::open(INFID, $inputFile) || die "Couldn't open $inputFile";
  while (($line = <INFID>) && !($boolFinished))
  {  
    if($line =~ m/ Dependent variable initial conditions:/)
    {
      $boolFinished = 1;
    }
  }
  # Find LV pressur from element 1.
  $boolFinished = 0;
  while (($line = <INFID>) && !($boolFinished))
  {  
    if($line =~ m% Enter element #s/name \[EXIT\]:(.+)%)
    {
      $element = $1;
      if($element == $LVElement)
      {
        $line = <INFID>;
        if($line =~ m/ The initial value of auxiliary variable number 1 is \[.+\]:(.+)/)
        {
          $LVPressure = $1;
          #print $LVPressure, "\n";
        }
      }
    }
  }
  CORE::close(INFID);
  return($LVPressure);
} #readPressure


# changeCalcium
# Function creates/edit the file calcium0xx.ipacti and enters the received parameter as the "Enter initial calcium level [Ca]i [0]: xxx"
sub changeCalcium
{
  my ($Cactn) = @_;

  my $outputFile = "calcium0xx.ipacti";
  CORE::open(OUTFID, ">$outputFile") || die "Couldn't open $outputFile";
  print OUTFID " CMISS Version 1.21 ipacti File Version 2\n";
  print OUTFID " Heading: ipacti file. Calcium level: $Cactn\n";
  print OUTFID "\n";
  print OUTFID " Specify type of contraction model [1]:\n";
  print OUTFID "   (1) SS tension-length-Ca relation\n";
  print OUTFID "  *(2) Hill\n";
  print OUTFID "   (3) Fading memory\n";
  print OUTFID "   (4) Outer hair cell stiffness\n";
  print OUTFID "    1\n";
  print OUTFID " Enter max isometric tension at ext.ratio=1 (Tref) [100kPa]: 0.10000D+03\n";
  print OUTFID " Enter non-dimensional slope parameter (beta) [1.45]: 0.14500D+01\n";
  print OUTFID " Enter c50 for [Ca]i saturation curve (0<c<1) [0.5]: 0.50000D+00\n";
  print OUTFID " Enter Hill coeff. for [Ca]i saturation curve (h) [3.0]: 0.30000D+01\n";
  print OUTFID " Enter max [Ca]i (Ca_max) [1]: 0.10000D+01\n";

  print OUTFID " Specify whether the initial calcium level [Ca]i is [1]:\n";
  print OUTFID "  (1) Constant spatially\n";
  print OUTFID "  (2) Piecewise constant (defined by elements)\n";
  print OUTFID "  (3) Defined by Gauss points\n";
  print OUTFID "   3\n";
  print OUTFID " Enter element #s/name [EXIT]: 1..4,17..20,33..36\n";
  print OUTFID " Gauss Point #s[EXIT]: 1..3,10..12,19..21\n";
  print OUTFID sprintf(" The value is [0]: %f\n", $Cactn*0.6);
  print OUTFID " Gauss Point #s[EXIT]: 4..6,13..15,22..24\n";
  print OUTFID sprintf(" The value is [0]: %f\n", $Cactn*0.8);
  print OUTFID " Gauss Point #s[EXIT]: 7..9,16..18,25..27\n";
  print OUTFID sprintf(" The value is [0]: %f\n", $Cactn*0.9);
  print OUTFID " Gauss Point #s[EXIT]: 0\n";
  print OUTFID " Enter element #s/name [EXIT]: 5..16,21..32,37..48\n";
  print OUTFID " Gauss Point #s[EXIT]: 1..27\n";
  print OUTFID sprintf(" The value is [0]: %f\n", $Cactn*1.0);
  print OUTFID " Gauss Point #s[EXIT]: 0\n";
  print OUTFID " Enter element #s/name [EXIT]: 0\n";
  CORE::close(OUTFID);
  #print $Cactn, "\n";
  return(1);
} #changeCalcium



# editInitFile
# Function copies the information from temp.ipinit into ejection.ipinit
# and adds an increment in Mu ($delMu) and P ($delP).
sub editInitFile
{
  my ($delP) = @_;
  
  my $inputFile = "output/temp.ipinit";
  my $outputFile = "output/ejection.ipinit";
  my $boolFinishedP = 0;
  my $boolContiueWriting = 0;
  my $line;
  my $node;
  my $element;

  if (!(-e $inputFile))
  {
    die "Input-file doesn't exist";
  }

  CORE::open(INFID, $inputFile) || die "Couldn't open $inputFile";
  CORE::open(OUTFID, ">$outputFile") || die "Couldn't open $outputFile";
  while ($line = <INFID>)
  {
    print OUTFID $line;
    if (!$boolFinishedP)
    {
      if($line =~ m% Dependent variable/equation number 4 :%)
      {
        #print OUTFID $line;
        print OUTFID " Enter node #s/name [EXIT]: 0\n";
        print OUTFID " Enter element #s/name [EXIT]: 1..16\n";
        print OUTFID " Do you want to prescribe auxiliary variable/rhs number 1 [N]? y\n";
        print OUTFID " The increment is [0.0]: $delP\n";
        print OUTFID " Do you want to prescribe auxiliary variable/rhs number 2 [N]? n\n";
        #print OUTFID " The increment is [0.0]: 0\n";
        print OUTFID " Enter element #s/name [EXIT]: 0\n";
        print OUTFID "\n";
        $boolFinishedP = 1;

        # Search for the finishing of the Dependent variable/equation number 4.
        while (($line = <INFID>) && !($boolContiueWriting))
        {
          if($line =~ m% Enter element #s/name \[EXIT\]:(.+)%)
          {
            $element = $1;
            if($element == 0)
            {
              $boolContiueWriting = 1;
            }
          }
        }
      }
    }
  }
  CORE::close(INFID);
  CORE::close(OUTFID);
  return(1);
} #editInitFile



# findCalcium
# function reads in the calcium level [Ca]i in file calciumEndIsovolContr.ipacti
# and returns mu.
sub findCalcium
{
  my ($inputFile) = @_;
  
  #my $inputFile = "output/calciumEndIVC.ipacti";
  my $ca = 0;
  my $boolFinished = 0;

  if (!(-e $inputFile))
  {
    die "Input-file doesn't exist";
  }

  CORE::open(INFID, $inputFile) || die "Couldn't open $inputFile";
  while ($line = <INFID>)
  {
    if($line =~ m/ The value is \[.+\]:(.+)/)
    {
      $ca = $1;
      #print "Calcium = ", $ca, "\n";
    }
  }
  CORE::close(INFID);
  return($ca);
}


sub findVolumeGivenP
{
  # findVolumeGivenP finds (interpolates) volume for a given pressure.
  my ($p0,$V_ED,$EF_at_peak_P,$P_start_ejection,$P_max) = @_;
  
  my @Peject=(10.0,10.2,10.4,10.6,10.8,11.0,11.2,11.4,11.6,11.8,
           12.0,12.2,12.4,12.6,12.8,13.0,13.2,13.4,13.6,13.8,
           14.0,14.2,14.4,14.6,14.8,15.0);
  my @Veject=(1.0000,0.9800,0.9601,0.9401,0.9202,0.8992,0.8748,
           0.8504,0.8262,0.8070,0.7878,0.7662,0.7441,0.7219,
           0.6844,0.6301,0.6008,0.5548,0.5067,0.4697,0.4299,
           0.3670,0.3058,0.2367,0.1458,0.0000);
  my $maxIndex = $#Peject;
  
  my $dP = $P_max - $P_start_ejection;
  my $V_at_peak_P = $V_ED*(100-$EF_at_peak_P)/100;
  my $index=0;
  my $v=0; my $p1=0; my $p2=0; my $v1=0; my $v2=0;my $a=0;
  
  foreach my $v (@Veject)
  {
    $Veject[$index]=$v*($V_ED-$V_at_peak_P) + $V_at_peak_P;
    #print "$index: ", $Veject[$index], "\n";
    $index++;
  }
  if(($P_start_ejection != 10) || ($P_max != 15))
  {
    $index=0;
    foreach my $p (@Peject)
    {
      $Peject[$index]=($p-10)/5*$dP+$P_start_ejection;
      #print "$index: ", $Peject[$index], "\n";
      $index++;
    }
  }
  
  $index=0;
  foreach my $p (@Peject)
  {
    if($p>=$p0)
    {
      $p2=$p;
      $v2=$Veject[$index];
      if($index>0)
      {
        $p1=$Peject[$index-1];
        $v1=$Veject[$index-1];
      } else
      {
        $p1=$p;
        $v1=$Veject[$index];
      }
      last;
    }
    $index++;
  }
  if($p2==$p1)
  {
    $a = 0;
  } else
  {
    $a = ($v2-$v1)/($p2-$p1);
  }
  $v = $a*($p0-$p1) + $v1;
 
  if($p0 > $Peject[$maxIndex])
  {
    print "Warning: Subroutine findVolumeGivenP: input pressure exceeds max pressure!\n";
    $v=$Veject[$maxIndex];
  }
  print "p1: $p1, p2: $p2, v1: $v1, v2: $v2, v: $v\n";
  return($v); 
}

sub findVolume
{
  my ($inputFile, $basis) = @_;
  
  #my $inputFile = "output/calciumEndIVC.ipacti";
  my $vol = 0;
  my $boolFinished = 0;

  if (!(-e $inputFile))
  {
    die "Input-file doesn't exist";
  }

  CORE::open(INFID, $inputFile) || die "Couldn't open $inputFile";
  while ($line = <INFID>)
  {
    if($line =~ m/    Total     Volume =(.+)\(Basis type  $basis\)/)
    {
      $vol = $1;
      #print "Volume = ", $vol, "\n";
    }
  }
  CORE::close(INFID);
  return($vol);
}


sub writeCalciumLevel
{
  my ($ca, $time) = @_;

  my $outputFile = "Ca_of_time.txt";
  CORE::open(OUTFID, ">> $outputFile") || die "Error: Couldn't open $outputFile\n";
  print OUTFID sprintf("%03d  %1.4f\n",$time,$ca);
  CORE::close(OUTFID);
  return(1);
}

1;
 
__END__
