package MySubs ;
require 5.6.0;


=head1 MyUtils

Plotting functions 

=cut

BEGIN {
}


# readPressure
# This file takes an ipinit file written out during isovolumetric 
# contraction. It finds the LV pressure of that file.
sub readPressure 
{
  my ($inputFile) = @_;
  
  # Check pressures at these elements
  my $LVElement = 1;
  #my $inputFile = "output/temp.ipinit";
  my $boolFinished = 0;
  my $line;
  my $element;
  my $LVPressure;

  if (!(-e $inputFile))
  {
    die "Input-file doesn't exist";
  }
  CORE::open(INFID, $inputFile) || die "Couldn't open $inputFile";
  while (($line = <INFID>) && !($boolFinished))
  {  
    if($line =~ m/ Dependent variable initial conditions:/)
    {
      $boolFinished = 1;
    }
  }
  # Find LV pressur from element 1.
  $boolFinished = 0;
  while (($line = <INFID>) && !($boolFinished))
  {  
    if($line =~ m% Enter element #s/name \[EXIT\]:(.+)%)
    {
      $element = $1;
      if($element == $LVElement)
      {
        $line = <INFID>;
        if($line =~ m/ The initial value of auxiliary variable number 1 is \[.+\]:(.+)/)
        {
          $LVPressure = $1;
          #print $LVPressure, "\n";
        }
      }
    }
  }
  CORE::close(INFID);
  return($LVPressure);
} #readPressure


sub writeResult
{
  my ($t,$ca,$lv_v,$lv_p) = @_;

  my $outputFile = "result.m";
  CORE::open(OUTFID, ">> $outputFile") || die "Error: Couldn't open $outputFile\n";
  if (-z $outputFile)
  {
    print OUTFID "res = [\n";    
  }
  print OUTFID sprintf("%3d     %1.4f  %10.2f  %10.2f\n",$t,$ca,$lv_v,$lv_p);
  CORE::close(OUTFID);
}

sub finishResult
{
  my $outputFile = "result.m";
  CORE::open(OUTFID, ">> $outputFile") || die "Error: Couldn't open $outputFile\n";
  print OUTFID "];\n";    
  CORE::close(OUTFID);
}



# changeCalcium
# Function creates/edit the file calcium0xx.ipacti and enters the received parameter as the "Enter initial calcium level [Ca]i [0]: xxx"
sub changeCalcium
{
  my ($Cactn) = @_;

  my $outputFile = "calcium0xx.ipacti";
  CORE::open(OUTFID, ">$outputFile") || die "Couldn't open $outputFile";
  print OUTFID " CMISS Version 1.21 ipacti File Version 2\n";
  print OUTFID " Heading: ipacti file. Calcium level: $Cactn\n";
  print OUTFID "\n";
  print OUTFID " Specify type of contraction model [1]:\n";
  print OUTFID "   (1) SS tension-length-Ca relation\n";
  print OUTFID "  *(2) Hill\n";
  print OUTFID "   (3) Fading memory\n";
  print OUTFID "   (4) Outer hair cell stiffness\n";
  print OUTFID "    1\n";
  print OUTFID " Enter max isometric tension at ext.ratio=1 (Tref) [100kPa]: 0.10000D+03\n";
  print OUTFID " Enter non-dimensional slope parameter (beta) [1.45]: 0.14500D+01\n";
  print OUTFID " Enter c50 for [Ca]i saturation curve (0<c<1) [0.5]: 0.50000D+00\n";
  print OUTFID " Enter Hill coeff. for [Ca]i saturation curve (h) [3.0]: 0.30000D+01\n";
  print OUTFID " Enter max [Ca]i (Ca_max) [1]: 0.10000D+01\n";

  print OUTFID " Specify whether the initial calcium level [Ca]i is [1]:\n";
  print OUTFID "  (1) Constant spatially\n";
  print OUTFID "  (2) Piecewise constant (defined by elements)\n";
  print OUTFID "  (3) Defined by Gauss points\n";
  print OUTFID "   3\n";
  print OUTFID " Enter element #s/name [EXIT]: 1..4,17..20,33..36\n";
  print OUTFID " Gauss Point #s[EXIT]: 1..3,10..12,19..21\n";
  print OUTFID sprintf(" The value is [0]: %f\n", $Cactn*0.6);
  print OUTFID " Gauss Point #s[EXIT]: 4..6,13..15,22..24\n";
  print OUTFID sprintf(" The value is [0]: %f\n", $Cactn*0.8);
  print OUTFID " Gauss Point #s[EXIT]: 7..9,16..18,25..27\n";
  print OUTFID sprintf(" The value is [0]: %f\n", $Cactn*0.9);
  print OUTFID " Gauss Point #s[EXIT]: 0\n";
  print OUTFID " Enter element #s/name [EXIT]: 5..16,21..32,37..48\n";
  print OUTFID " Gauss Point #s[EXIT]: 1..27\n";
  print OUTFID sprintf(" The value is [0]: %f\n", $Cactn*1.0);
  print OUTFID " Gauss Point #s[EXIT]: 0\n";
  print OUTFID " Enter element #s/name [EXIT]: 0\n";
  CORE::close(OUTFID);
  #print $Cactn, "\n";
  return(1);
} #changeCalcium


sub findCaOfTime
{
  my ($time) = @_;

  my $ca=-1;
  my $inputFile = "Ca_of_time.txt";
  CORE::open(INFID, $inputFile) || die "Couldn't open $inputFile";
  while (($line = <INFID>) && !($boolFinished))
  {  
    if($line =~ m/(.+)  (.+)/)
    {
      $t=$1;
      if($t == $time)
      {
        $ca = $2;
      }
    }
  }
  if($ca==-1)
  {
    print "Warning: no calcium found for time $time!\nReturning ca=-1.\n";
  }
  return($ca);
}
