  #
  # Set up equations, material properties and initial conditions
  # for cavity region
  fem define equation;r;equationRegions region $WALL,$LV_CAVITY lock
  fem define material;r;materialCavityIVC region $LV_CAVITY
  fem define initial;r;initCavity region $LV_CAVITY
  #
  # Define cavity-to-wall coupling and solution procedure information
  fem define coupling;r;coupled
  fem define solve;r;solveRegion coupled region $WALL,$LV_CAVITY
  #
  # Transfer wall deformation from the inflation problem to 
  # geometric dependent variables for the cavity region
  fem update solution coupled source_region $WALL
  fem update solution cavity_reference average 104 in 1 node 14..17 region $LV_CAVITY

  # Check that the current coupled solution is converged (as expected!)
  #fem evaluate residuals wrt geom_params coupled
  fem evaluate reactions coupled
  fem evaluate residuals wrt geom_params coupled

  $name = "cavityOriginal2"
  $file = $output.$name
  fem export nodes;$file field as $name offset 0 reg $LV_CAVITY
  fem export elements;$file field as $name offset_elem 0 reg $LV_CAVITY

  fem list element total region $LV_CAVITY
  fem list element deformed total region $LV_CAVITY
  #$file = $output."cavity_volume";
  #fem list element;$file deformed total region $LV_CAVITY
  #$DIASTOLIC_LV_CAVITY_VOLUME = MyFunc::findVolume("$file.opelem",3);  
  #print "\nLV cavity volume = $DIASTOLIC_LV_CAVITY_VOLUME\n";
  #
  #die("Stop")

  # Save deformed configurations for end diastole
  fem define initial;w;$output."ED_coupled_inflated" reg $WALL,$LV_CAVITY
  #fem define initial;r;$output."ED_coupled_inflated" region $WALL,$LV_CAVITY

  ##
  # ############################################
  # Start increasing calcium
  # ############################################
  $iterations = 30
  #stop
  if (1)
  {
    # Increment intracell calcium conc and solve for isovolumic contraction
    $Cactn = 0.10;
    $return = MyFunc::changeCalcium($Cactn);
    fem define active;r;"calcium0xx"
    fem solve coupled increment 0.0 iter $iterations error 1.0e-3
    fem define initial;w;$output."temp" region $WALL
    fem define initial;w;$outputInit."lv_def_ivc_$time" reg $WALL
    $return = MyFunc::writeCalciumLevel($Cactn,$time);
    $time=$time+1;

    read com;export

    $boolContinue = 1;
    #while ($boolContinue & ($Cactn<0.25))
    while ( $boolContinue )
    {         
      $Plv = MyFunc::readPressure($output."temp.ipinit");
      print "\n\n\nLV Pressure: ", $Plv, "\n";
      if ($Plv < $PstartEjection)
      {
        if(($Cactn>=0.21) && ($Cactn<0.23))
        {
          $Cactn = $Cactn+0.005;
        }  elsif($Cactn>=0.23)
        {
          $Cactn = $Cactn+0.001;
        } else
        {
          $Cactn = $Cactn+0.04;
        } 
        if($Cactn>=1.0)
        {
          $boolContinue=0;
        }
        print "\n***************************************************\n";
        print "\n\n\nCalcium: ", $Cactn, "\n\n\n\n";
        print "\n***************************************************\n";
        $return = MyFunc::changeCalcium($Cactn);
        fem define active;r;"calcium0xx"
        fem solve coupled increment 0.0 iter $iterations error 1.0e-6
        fem define initial;w;$outputInit."lv_def_ivc_$time" reg $WALL
        $return = MyFunc::writeCalciumLevel($Cactn,$time);
        $time=$time+1;
        fem define initial;w;$output."temp" region $WALL

        read com;export
      } else
      {
        $boolContinue = 0;
      }
    }
    fem solve coupled increment 0.0 iter $iterations error 1.0e-6
    fem define initial;w;$outputInit."lv_def_ivc_$time" reg $WALL
    $return = MyFunc::writeCalciumLevel($Cactn,$time);
    $time=$time+1;
    #
    # Save deformed configurations for end isovolumic contraction
    fem define initial;w;$output."wall_endIVC" region $WALL
    fem define initial;w;$output."cavity_endIVC" region $LV_CAVITY
    fem define initial;w;$output."bothReg_endIVC" region $WALL,$LV_CAVITY
    fem define active;w;$output."calciumEndIVC"
  }
  print "Last time step: $time\n";
  
