#Example_95  Simulating water and heat transfer in a central airway model
#This example simulates the transfer of water and heat from surrounding 
#tissue/capillary bed, through the tissue and the airway surface 
#liquid, and ultimately into unconditioned air in the lumen.  The 
#geometric model consists of the central airways (trachea to lobar 
#bronchi) and a lumped parameter model in place of each lobe.

# If the example path is not set, default to current directory
if (! defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}

#Parameters for mesh geometry
$RADIUS = 1;
$ALVEOLI= 2;

#Parameters for problem solution
$INSPIRATION = 1;
$EXPIRATION = 2;
$dt = 0.05;
$T_insp = 1.0;
$T_expn = 1.0;
$Temp_in = 31.3;
$time = 0.0;
$flow_in = 0.5; #litres
$flow_out = -0.5; #litres
$FIXED_NODE = 1;
$FLUX_NODE = 1;
$NUM_BREATHS = 10;
$NUM_EXPORTS = 1;

fem de param;r;$example/central;
fem de coor;r;$example/central;
fem de base;r;$example/central;

fem de node;r;$example/central;
fem de elem;r;$example/central;

fem de mesh;c airway as conducting_airways field;
fem evaluate ordering;

fem group elem 1..101 as trachea;
fem group elem 102 as right_main_bronchus;
fem group elem 103 as left_main_bronchus;
fem group elem 104 as right_lower_bronchus;
fem group elem 105 as right_upper_lobe_bronchus;
fem group elem 107 as left_upper_lobe_bronchus;
fem group elem 106 as left_lower_lobe_bronchus;
fem group elem 109 as right_middle_lobe_bronchus;
fem group elem 108 as right_lower_lobe_bronchus;

fem update mesh geometry elem trachea radius_field $RADIUS constant 16;
fem update mesh geometry elem right_main_bronchus radius_field $RADIUS constant 11.1;
fem update mesh geometry elem left_main_bronchus radius_field $RADIUS constant 12.0;
fem update mesh geometry elem right_lower_bronchus radius_field $RADIUS constant 8.9;
fem update mesh geometry elem right_upper_lobe_bronchus radius_field $RADIUS constant 7.3;
fem update mesh geometry elem left_upper_lobe_bronchus radius_field $RADIUS constant 7.5;
fem update mesh geometry elem left_lower_lobe_bronchus radius_field $RADIUS constant 8.0;
fem update mesh geometry elem right_middle_lobe_bronchus radius_field $RADIUS constant 5.2;
fem update mesh geometry elem right_lower_lobe_bronchus radius_field $RADIUS constant 6.4;

fem de;add mesh;c airway lumped_parameter basis_function 1 as respiratory field radius_field $RADIUS alveolar_field $ALVEOLI;
fem group node in element respiratory as respiratory;
fem update mesh geometry elem respiratory radius_field $RADIUS lumped volume 5e6;

#DEFINE THE PROBLEM SETUP FOR INSPIRATION
fem de equa;r;$example/central class $INSPIRATION;
fem de mate;r;$example/central class $INSPIRATION;

fem de moti;c lung elem respiratory uniform flow_field 3 inlet_flow $flow_in class $INSPIRATION;
fem de init;c lung fixed node $FIXED_NODE temperature $Temp_in rh 100 core 37 class $INSPIRATION;
fem de solve;r;$example/central class $INSPIRATION;

#DEFINE THE PROBLEM SETUP FOR EXPIRATION
fem de equa;r;$example/central class $EXPIRATION;
fem de mate;r;$example/central class $EXPIRATION;
fem de moti;c elem respiratory uniform flow_field 3 lung inlet_flow $flow_out class $EXPIRATION;
fem de init;c lung flux node $FLUX_NODE class $EXPIRATION;
fem de solve;r;$example/central class $EXPIRATION;

#SOLVE FOR FIRST_BREATH
$restart = "";
$time_end = 0.0;

for (my $j = 0; $j < $NUM_BREATHS; $j++){

    for (my $i = 0; $i < $NUM_EXPORTS; $i++){
	$time_end = $time_end + $T_insp/$NUM_EXPORTS;
	print "FEM solve $restart to $time_end delta_t $dt class $INSPIRATION\n";
	fem solve $restart to $time_end delta_t $dt class $INSPIRATION;
	$N++;
	$restart = "restart";

    }

    fem update solution class $EXPIRATION substitute solution class $INSPIRATION;
    
    for (my $i = 0; $i < $NUM_EXPORTS; $i++){
	$time_end = $time_end + $T_expn/$NUM_EXPORTS;
	print "FEM solve $restart to $time_end delta_t $dt class $EXPIRATION\n";
	fem solve $restart to $time_end delta_t $dt class $EXPIRATION;
	$N++;
    }
    
    fem update solution class $INSPIRATION substitute solution class $EXPIRATION;
    fem update solution class $INSPIRATION nodes $FIXED_NODE substitute constant ($Temp_in+273.15);
}

