# Example b211m - Implict Luo-Rudy phase 2 over a small, one element,
# cube. If the geometry and fields are exported, they can be viewed 
# using the draw.com command file. The signal file can be viewed using 
# UnEMAP.

#Set up problem size
fem define para;r;cube;example

#Define geometry of the problem
fem define coor;r;cube;example
fem define node;r;cube;example
fem define base;r;cube;example
fem define elem;r;cube;example
fem define fibr;d
fem define elem;r;cube;example fibre
fem define grid;r;cube;example
fem update grid geometry
fem update grid metric
#  Some useful groups
fem group grid external as boundary
fem group grid grid 92,93,101 as stimulus_site

#Define equation to solve
#  Define monodomain with LR2 kinetics
fem define equa;r;lr;example

#Define material properties for the problem

#  Define cellular material properties
fem define cell;r;lr;example
#    Define cell variants and spatially varying material parameters
fem define mate;r;cube;example cell
#  Define continuum material parameters
fem define mate;r;cube;example
fem update grid material

#Define continuum initial conditions
#  Set no-flux boundary condition
fem define init;r;cube;example

#Define the implicit solve, using the adams integrator
fem define solv;r;implicit;example

#The geometry can be exported to CMGUI via these commands
#fem export node;3dcube as cube              !Export node geometry
#fem export elem;3dcube as cube              !Export element geometry

#Solve the problem

#  Set the number of threads to use when using a multiprocessing 
#  executable
set num_threads 16

#  Initialise the solve
fem solve to 0

#  Open a binary history file to store the membrane potential (YQ(1))
fem open history;cube write variables yq niqlist 1 binary

#  Loop through time (solve for 3ms)

# Command updated by fixcom.sh on Wed Aug 23 17:49:55 NZT 2000
# Old command: DO HUN=0..0
for $HUN ( 0..0 ) 
{


# Command updated by fixcom.sh on Wed Aug 23 17:49:55 NZT 2000
# Old command: DO TEN=0..0
for $TEN ( 0..0 ) 
{


# Command updated by fixcom.sh on Wed Aug 23 17:49:55 NZT 2000
# Old command: DO UNIT=0..3
for $UNIT ( 0..3 ) 
{


#  Solve for the current time

# Command updated by fixcom.sh on Wed Aug 23 17:49:55 NZT 2000
# Old command:   fem solve restart to $HUN//$TEN//$UNIT
fem solve restart to "${HUN}${TEN}$UNIT"


#  Write the membrane potential to the history file

# Command updated by fixcom.sh on Wed Aug 23 17:49:55 NZT 2000
# Old command:   fem write history time $HUN//$TEN//$UNIT variables yq binary
fem write history time "${HUN}${TEN}$UNIT" variables yq binary


#  Export the field to CMGUI
#  fem export elem;gridfield_1//HUN//TEN//UNIT field as cube


# Command updated by fixcom.sh on Wed Aug 23 17:49:55 NZT 2000
# Old command: ENDDO
}



# Command updated by fixcom.sh on Wed Aug 23 17:49:55 NZT 2000
# Old command: ENDDO
}



# Command updated by fixcom.sh on Wed Aug 23 17:49:55 NZT 2000
# Old command: ENDDO
}



#  Close the history file
fem close history binary

#  Calculate the binary signal file from the history file
fem evaluate electrode;cube history cube from grid binary

#  Define UnEmap export properties
fem define export;r;cube;example

#  Export the binary signal file to UnEmap
fem export signal;cube electrode signal cube
