# Example b211u
# 2/3 heart mesh in RC coordinates - 520K grid points
#  with a Noble 98 ionic current model - monodomain -
#  implicit solution with Adams Moulton integrator.
#
# Output: dipoles.ipsour - dipole source file for use
#          inside passive torso
#         activ.exelem - activation times from each
#          collocation point
#
# WARNING - takes approx 4 days to solve on HPC1 with
#  8 threads.

# Geometry file string
$GEOMSTRING = "translated";

# Stimulus sites
$ACTIV1 = "307003,416419,409772,307643,307644,307659,308283";
$ACTIV2 = "316265,425989,317184,317185,340563,317208,318105";
$ACTIV3 = " 36057,192817, 37376, 37377, 37378, 37410, 38697";

#=================================================

{
  eval { require Time::HiRes };
  if( $@ )
    {
      print "warning: $@";
    }
  else
    {
      import Time::HiRes qw(time);
    }
}

sub cpu_time()
  {
    # Add user/system parent/child cpu_times.
    my $result = 0;
    map { $result += $_ } times;
    $result;
  }

sub print_timing($$$)
  {
    my ($message,$wall,$cpu) = @_;
    printf "%s: %.3fs wall, %.3fs cpu.\n", $message, $wall, $cpu;
  }

sub print_memory_use()
  {
    # Different operating systems have different data and ps uses different
    # arguments so try a series of commands in an attempt to collect the
    # information.
    system qw(ps -o vsz -p), $$;
    system qw(ps -l -p), $$;
    system qw(ps v), $$;
  }

# Read in the 2/3 rc heart mesh
FEM define para;r;$GEOMSTRING;example;
FEM define coor 3,1;
FEM define node;r;$GEOMSTRING;example;
FEM define base;r;$GEOMSTRING;example;
FEM define elem;r;$GEOMSTRING;example;
FEM define fibr;r;$GEOMSTRING;example;
FEM define elem;r;$GEOMSTRING;example fibre;

# Create a collocation mesh
FEM define grid;r;full_res;example;

FEM update grid geometry;
FEM update grid metric;
FEM group grid external as boundary;
FEM group grid grid $ACTIV1 as stimulus1;
FEM group grid grid $ACTIV2 as stimulus2;
FEM group grid grid $ACTIV3 as stimulus3;

# Set up the problem ready for solution
FEM define equa;r;mono;example;
FEM define mate;r;621;example;
FEM define cell;r;n98;example;
FEM define mate;r;n98;example cell;
FEM update grid material;
FEM define init;r;n98;example;
FEM define solv;r;implicit_05;example;
FEM solve to 0 calc_activ;

($wall_setup,$cpu_setup) = (time, cpu_time);
print_timing 'setup', $wall_setup - $^T, $cpu_setup;

$iterations = $full_problem ? 500 : 2;

for( my $i = 0; $i < $iterations; $i++ ) {
    FEM solve restart to "$i" calc_activ;
    FEM define sour;c grid grregion 1 grclass 1 time "$i" fixed;
}

($wall_solve,$cpu_solve) = (time, cpu_time);
print_timing 'solve', $wall_solve - $wall_setup, $cpu_solve - $cpu_setup;

#Export the geometry of the mesh
FEM export node;heart as heart;
FEM export elem;heart as heart;
FEM update grid activation_times;
FEM export elem;activ as heart iy 4 field;
FEM define sour;w;dipoles grid grregion 1 grclass 1;

print_memory_use;
