# This example solves the bidomain, Beeler-Reuter equation on
# a 6 vertex wedge type element. The equations are discretised
# using the lattice grid method.

# The solution is exported at each time-step to exelem files
# and can be visualised using cmgui


# Set up for bidomain.
$VM=1
$PHI_E=2


fem define parameters;r;6vertwedge
fem define coordinates;r;6vertwedge

fem define node;r;6vertwedge

fem define base;r;6vertwedge

fem define elem;r;6vertwedge

fem define grid;r;6vertwedge
fem update grid geometry
fem update grid metric

# Export the surface normals the the file normals.exdata
fem ex data;normals normals as normals

# Group grid points
fem group grid external as boundary
fem group grid xi2 high as stim_point oneoff element 1

fem def equa;r;6vertwedge class $VM,$PHI_E
fem def mate;r;6vertwedge class $VM,$PHI_E
fem def cell;r;6vertwedge class $VM,$PHI_E

fem update grid material class $VM,$PHI_E

fem def init;r;6vertwedge class $VM,$PHI_E

fem def solve;r;6vertwedge class $VM
fem def solve;r;6vertwedge class $PHI_E
 
fem solve to 0 class $VM,$PHI_E

# Export the grid elements and nodes
fem export node;Domain as Domain reg 1
fem export elem;Domain grid_numbers as Domain reg 1
fem export elem;"MemPot_000" field as Solution class $VM

for $M ( 0..10 )
 {
 for $L ( 0..9 )
  {  
  #Solve for new time
  print "Time step: $M.$L\n"; 
  fem solve restart to "${M}.$L" class $VM,$PHI_E

  # Export solution
  fem export elem;"MemPot_${M}.$L" field as Solution class $VM
 }
}

