
############################################################
# This example illustrates FFD-mechanics of the two 
# hamstring muscles (semimembranosus and semitendinosus)
# during a hip flexion movement. The muscles are defined 
# inside host meshes which also have a similar shape to
# the muscle, as in extended-FFD. Mechanics is solved on
# the hosts, with different material properties for tendon
# and muscle, and volume conservation. The embedded muscles
# are then updated at each load step. Similar to example
# d27, contact is between the two embedded slave bodies and 
# the muscle hosts are constrained to ensure no overlap
# between the muscles.
############################################################

fem def para;r;FARO;example
fem def region;r;FARO;example
fem def coord 3,1

fem def base;r;bicubic_linear_tricubic;example
$REG=1;

################ muscle host MESHES ################
# host_list 

@host_list = ('semitendinosus','semimembranosus');
foreach $component(@host_list)
{ 
# Host mesh
fem def node;r;host_$component;example region $REG 
fem def element;r;host_$component;example region $REG

fem up node der 1 linear reg $REG
fem up node der 2 linear reg $REG
fem up node der 3 linear reg $REG
fem update scale_factors normalise reg $REG

if($REG==1){fem group elements 8001,8004 as TEN2 reg $REG}
if($REG==1){fem group elements 8002,8003 as MUSCLE2 reg $REG}
if($REG==1){fem group nodes 8009..8020 as PRESCRIBE2 reg $REG}
if($REG==1){fem group nodes 8001..8008 as FREE2 reg $REG}

if($REG==2){fem group elements 7001,7003 as TEN reg $REG}
if($REG==2){fem group elements 7002,7004 as MUSCLE reg $REG}
if($REG==2){fem group nodes 7009..7016 as PRESCRIBE reg $REG}
if($REG==2){fem group nodes 7001..7008,7021..7024 as FREE reg $REG}

fem def fibre;r;host_$component;example region $REG
fem def elem;r;host_$component;example fibre region $REG

fem def field;r;host_$component;example region $REG
fem def elem;r;host_$component;example field region $REG

#fem exp nodes;host_zero_$component as host_$component region $REG;
#fem exp elem;host_zero_$component as host_$component region $REG;

# host 
fem def equa;r;host_$component;example reg $REG
fem def mate;r;host_$component;example reg $REG
fem def init;r;host_$component;example reg $REG

# put initial host geometry into fields 1,2,3
fem update field 1 reg $REG subst geometry x reg $REG;
fem update field 2 reg $REG subst geometry y reg $REG;
fem update field 3 reg $REG subst geometry z reg $REG;
fem update field 4 reg $REG subst geometry x reg $REG;
fem update field 5 reg $REG subst geometry y reg $REG;
fem update field 6 reg $REG subst geometry z reg $REG;
fem update field 7 reg $REG subst geometry x reg $REG;
fem update field 8 reg $REG subst geometry y reg $REG;
fem update field 9 reg $REG subst geometry z reg $REG;

$REG=$REG+1;
}

################ bone MESHES ################ 

# bone_list 
@bone_list = ('femur','tibia','hip');
foreach $component(@bone_list)
{ 
# Read in bone
fem def node;r;test_$component;example region $REG 
fem def element;r;test_$component;example region $REG

fem change nodes translate by -241,-170,877 reg $REG

# Field 
fem def field;r;simulate_$component;example region $REG 
fem def elem;r;simulate_$component;example field region $REG 

## Export initial bone
#fem exp nodes;initial_$component as $component region $REG
#fem exp elem;initial_$component as $component region $REG

# put initial geometry into fields 1,2,3
fem update field 1 reg $REG subst geometry x reg $REG;
fem update field 2 reg $REG subst geometry y reg $REG;
fem update field 3 reg $REG subst geometry z reg $REG;
fem update field 4 reg $REG subst geometry x reg $REG;
fem update field 5 reg $REG subst geometry y reg $REG;
fem update field 6 reg $REG subst geometry z reg $REG;
fem update field 7 reg $REG subst geometry x reg $REG;
fem update field 8 reg $REG subst geometry y reg $REG;
fem update field 9 reg $REG subst geometry z reg $REG;

$REG=$REG+1;
}

################ SLAVE_muscles MESH ################ 


# muscle_list
@muscle_list = ('semitendinosus','semimembranosus')
foreach $component(@muscle_list)
{ 
# Read in muscle

fem def node;r;simulate_$component;example region $REG
fem def element;r;simulate_$component;example region $REG

fem change nodes translate by -241,-170,877 reg $REG

# Field 
fem def field;r;simulate_$component;example region $REG
fem def elem;r;simulate_$component;example field region $REG

## Export initial muscle
#fem exp nodes;initial_$component as $component region $REG
#fem exp elem;initial_$component as $component region $REG

# slave
fem def equa;r;simulate_$component;example reg $REG
fem def mate;r;simulate_$component;example reg $REG
fem def init;r;simulate_$component;example reg $REG

# put initial geometry into fields 1,2,3
fem update field 1 reg $REG subst geometry x reg $REG;
fem update field 2 reg $REG subst geometry y reg $REG;
fem update field 3 reg $REG subst geometry z reg $REG;
fem update field 4 reg $REG subst geometry x reg $REG;
fem update field 5 reg $REG subst geometry y reg $REG;
fem update field 6 reg $REG subst geometry z reg $REG;
fem update field 7 reg $REG subst geometry x reg $REG;
fem update field 8 reg $REG subst geometry y reg $REG;
fem update field 9 reg $REG subst geometry z reg $REG;

if($REG==6){fem def xi;c nodes contain of $REG in 1 loose}
#if($REG==6){fem def xi;w;$component nodes contain of $REG in 1}
if($REG==7){fem def xi;c nodes contain of $REG in 2 loose}
#if($REG==7){fem def xi;w;$component nodes contain of $REG in 2}

$REG=$REG+1;
}

#######################################

fem def solve;r;contact;example coupled reg 1,2

# Load steps
#for ($j=1;$j<61;$j++)
for ($j=1;$j<5;$j++) # for testing
{
# Precribe rotation
fem change nodes translate by 0,-0.33,1.25 node 8009..8012 reg 1
fem change nodes translate by 0,0.083,0.83 node 7009..7012 reg 2
fem change nodes rotate by -1 about 0,0,0 axis 1,0,0 reg 5

# Update muscle in host and export
fem update field 1 reg 1 subst geometry x reg 1;
fem update field 2 reg 1 subst geometry y reg 1;
fem update field 3 reg 1 subst geometry z reg 1;
fem update geometry x reg 1 subst field 4 reg 1
fem update geometry y reg 1 subst field 5 reg 1
fem update geometry z reg 1 subst field 6 reg 1
fem def xi;r;semitendinosus;example nodes contain of 6 in 1 reg 6
fem update mesh position fit of 6 in 1
fem update geometry x reg 1 subst field 1 reg 1
fem update geometry y reg 1 subst field 2 reg 1
fem update geometry z reg 1 subst field 3 reg 1
fem update field 4 reg 1 subst geometry x reg 1;
fem update field 5 reg 1 subst geometry y reg 1;
fem update field 6 reg 1 subst geometry z reg 1;

# Update muscle in host and export
fem update field 1 reg 2 subst geometry x reg 2;
fem update field 2 reg 2 subst geometry y reg 2;
fem update field 3 reg 2 subst geometry z reg 2;
fem update geometry x reg 2 subst field 4 reg 2
fem update geometry y reg 2 subst field 5 reg 2
fem update geometry z reg 2 subst field 6 reg 2
fem def xi;r;semimembranosus;example nodes contain of 7 in 2 reg 7
fem update mesh position fit of 7 in 2
fem update geometry x reg 2 subst field 1 reg 2
fem update geometry y reg 2 subst field 2 reg 2
fem update geometry z reg 2 subst field 3 reg 2
fem update field 4 reg 2 subst geometry x reg 2;
fem update field 5 reg 2 subst geometry y reg 2;
fem update field 6 reg 2 subst geometry z reg 2;

fem update solution from geometry substitute reg 1,2

$CONVERGED=0;
while ($CONVERGED==0) 
#for ($z=1;$z<2;$z++)
{

######################################################contact setup

## Faces: (slave) semitendinosus
## 3418,3421,3424,3426,3429,3431,3434,3436,3439,3441

## Faces: (master) semimembranosus
## 3487,3491,3495,3498,3502,3505,3509,3512

fem def xi;c contact faces 3418,3421,3424,3426,3429,3431,3434,3436,3439,3441 points 6 reg 6

# Define data at xi positions 
fem def data;c from_xi reg 6
# Calculate xi in slave host
fem def xi;c closest reg 1

# Update slave info
fem update data field to slave reg 1

# Project onto target face (master surface)
fem group faces 3487,3491,3495,3498,3502,3505,3509,3512 as TARGET reg 7
fem def xi;c closest_face faces TARGET reg 7

# store projection gap in data fields
fem update data field from gap reg 7  

# Change Master object to host
# Create data from xi points
fem def data;c from_xi reg 7

# Generate xi in host
fem def xi;c closest reg 2

# Update master info
fem update data field to master reg 2 

######################################################solve

# Place initial geometry YP(ny,3) into XP
FEM up geom from sol YP_index 3 to 1..3 reg 1,2
fem solve error 0.1 iterate 15 coupled AUGMENTATIONS 0 CONTACT_STIFFNESS 0.1 TIED_STIFFNESS 1.0 reg 1,2
# update XP from YP
FEM up geom from sol to 1..3 reg 1,2

# Update muscle in host and export
fem update field 1 reg 1 subst geometry x reg 1;
fem update field 2 reg 1 subst geometry y reg 1;
fem update field 3 reg 1 subst geometry z reg 1;
fem update geometry x reg 1 subst field 4 reg 1
fem update geometry y reg 1 subst field 5 reg 1
fem update geometry z reg 1 subst field 6 reg 1
fem def xi;r;semitendinosus;example nodes contain of 6 in 1 reg 6
fem update mesh position fit of 6 in 1
fem update geometry x reg 1 subst field 1 reg 1
fem update geometry y reg 1 subst field 2 reg 1
fem update geometry z reg 1 subst field 3 reg 1
fem update field 4 reg 1 subst geometry x reg 1;
fem update field 5 reg 1 subst geometry y reg 1;
fem update field 6 reg 1 subst geometry z reg 1;

# Update muscle in host and export
fem update field 1 reg 2 subst geometry x reg 2;
fem update field 2 reg 2 subst geometry y reg 2;
fem update field 3 reg 2 subst geometry z reg 2;
fem update geometry x reg 2 subst field 4 reg 2
fem update geometry y reg 2 subst field 5 reg 2
fem update geometry z reg 2 subst field 6 reg 2
fem def xi;r;semimembranosus;example nodes contain of 7 in 2 reg 7
fem update mesh position fit of 7 in 2
fem update geometry x reg 2 subst field 1 reg 2
fem update geometry y reg 2 subst field 2 reg 2
fem update geometry z reg 2 subst field 3 reg 2
fem update field 4 reg 2 subst geometry x reg 2;
fem update field 5 reg 2 subst geometry y reg 2;
fem update field 6 reg 2 subst geometry z reg 2;

}

##export tibia and host
#fem exp nodes;hip_$j as hip region 5
#fem exp nodes;host_semitendinosus_$j as host_semitendinosus reg 1
#fem exp nodes;host_semimembranosus_$j as host_semimembranosus reg 2

# Update muscle in host and export
fem update field 1 reg 1 subst geometry x reg 1;
fem update field 2 reg 1 subst geometry y reg 1;
fem update field 3 reg 1 subst geometry z reg 1;
fem update geometry x reg 1 subst field 4 reg 1
fem update geometry y reg 1 subst field 5 reg 1
fem update geometry z reg 1 subst field 6 reg 1
fem def xi;r;semitendinosus;example nodes contain of 6 in 1 reg 6
fem update mesh position fit of 6 in 1
#fem exp nodes;semitendinosus_$j as semitendinosus region 6
fem update geometry x reg 1 subst field 1 reg 1
fem update geometry y reg 1 subst field 2 reg 1
fem update geometry z reg 1 subst field 3 reg 1
fem update field 4 reg 1 subst geometry x reg 1;
fem update field 5 reg 1 subst geometry y reg 1;
fem update field 6 reg 1 subst geometry z reg 1;

# Update muscle in host and export
fem update field 1 reg 2 subst geometry x reg 2;
fem update field 2 reg 2 subst geometry y reg 2;
fem update field 3 reg 2 subst geometry z reg 2;
fem update geometry x reg 2 subst field 4 reg 2
fem update geometry y reg 2 subst field 5 reg 2
fem update geometry z reg 2 subst field 6 reg 2
fem def xi;r;semimembranosus;example nodes contain of 7 in 2 reg 7
fem update mesh position fit of 7 in 2
#fem exp nodes;semimembranosus_$j as semimembranosus reg 7
fem update geometry x reg 2 subst field 1 reg 2
fem update geometry y reg 2 subst field 2 reg 2
fem update geometry z reg 2 subst field 3 reg 2
fem update field 4 reg 2 subst geometry x reg 2;
fem update field 5 reg 2 subst geometry y reg 2;
fem update field 6 reg 2 subst geometry z reg 2;

## Update scale factors so will equal arc-lengths
#fem update scale_factors normalise reg 6,7
#fem list lines;lines_$j 

}
