#!/usr/bin/perl -w
#
# This is a perl script to renumber nodes in an ipnode file
# and renumber the corresponding nodes in the ipelem file
#
# Usage:
#   renumber_ipnode_.pl filename1 filename2 filename3 filename4
#
# Created:
#   Yunn Lee and Carey Stevens Tue Dec 12 2000
# 

use strict;
use File::Compare;
use File::Find;
use Getopt::Std;

# array of node numbers declared global
my @node_numbers;

sub renumber_nodes
{
        # Given the input filename this routine parses
        # the given file and renumbers the nodes
        my $input = shift;
        my $output = shift;
 	my $line;
        my $new_number = 201;
	my $last; 
	my $current_node_num; 
	
	# Looks for lines in input file beginning "Node number..."
	my $new_node_line = " Node number ";
	
	# Opens input and output node files
        open(INPUT, "<$input") || die "Could not open file $input: $!\n";
        open(OUTPUT, ">$output") || die "Could not open file $output: $!\n";
	
	# Start of loop
	while (defined($line = <INPUT>))  #As long as there is a defined line, loop
        {
	   my @fields = split (' ',$line);  #Split each line (field) into "words" by spaces
           $last = scalar @fields; #Let the number of words in the line be the scalar variable "last"
           if (($line =~ m|^\Q$new_node_line\E|))  #If the line begins with "Node Number..." enter loop
           {
               # Fills node_numbers array with the Node number in each line beginning with Node number
               $current_node_num = $fields[$last-1];
               $node_numbers[$current_node_num] = $new_number;

	       # Print output old node number and new reordered node number
	       # printf("old and new node numbers %s %s\n",$node_numbers[$new_number],$new_number);
	       print OUTPUT " Node number [ 1]: ",$new_number,"\n";
	       
	       # Add 1 to new node number for next cycle
               $new_number++;
           }
           else
           {
	       printf OUTPUT $line;  
           }
        }

        # Close files
        close (INPUT);
	close (OUTPUT);
}


sub renumber_elements
{
        # Given the input filename this routine parses
        # the given file and renumbers the elements
        my $input = shift;
        my $output = shift;
 	my $line;
        my $new_number = 201;
	my $last; 
	my $current_node_num; 

	# Looks in the input file for the lines beginning "Enter the 4..."	       
        my $nodes_line = " Enter the 4";
	 
        # Opens both input and output ipelem files
	open(INPUT, "<$input") || die "Could not open file $input: $!\n";
        open(OUTPUT, ">$output") || die "Could not open file $output: $!\n";
	
	my $element_count=0;
	# Start of loop
	while (defined($line = <INPUT>))
        {
	   my @fields = split (' ',$line);   #Split each line (field) into "words" by spaces
           $last = scalar @fields;   #Let the number of words in the line be the scalar variable "last" i.e. 4
           if (($line =~ m|^\Q$nodes_line\E|))  #If the line begins with "Enter the 4..." enter loop
           {
	       print $line;
	       $fields[$last-4]=@node_numbers[$fields[$last-4]];
	       $fields[$last-3]=@node_numbers[$fields[$last-3]];
	       $fields[$last-2]=@node_numbers[$fields[$last-2]];
	       $fields[$last-1]=@node_numbers[$fields[$last-1]];

	       # Print the output renumbered node numbers of each element
	       printf("last 4 numbers %s %s %s %s\n",$fields[$last-4],$fields[$last-3],$fields[$last-2],$fields[$last-1]); 
               print OUTPUT " Enter the 4 global numbers for basis 1:  ",$fields[$last-4]," ",$fields[$last-3]," ",$fields[$last-2]," ",$fields[$last-1],"\n"; 

           } elsif($last >=2 && $fields[0] eq "Element" && $fields[1] eq "number")
	    {
		$element_count++;
		printf OUTPUT " Element number [    1]: %5d\n",$element_count;
	   }
           else
           {
#	print "$fields[0] $fields[1]\n";
	       printf OUTPUT $line;  
           }
        }


        # Close files
	close (INPUT);
	close (OUTPUT);
}


renumber_nodes($ARGV[0],$ARGV[1],@node_numbers);
renumber_elements($ARGV[2],$ARGV[3],@node_numbers);











